<?php

namespace App\Http\Controllers;

use Illuminate\Http\Request;
use DB;
use File;
use App\Library\CommonController;
use datetime;
use Exception;
use JWTAuth;

class BillCheckingTransaction extends Controller
{
    // check authority when create bill
    public function getCheckAuthority($user_id)
    {
        $token_data = JWTAuth::parseToken()->authenticate();

        $access['TCS_Applicable'] = $this->checkAuthorityAccess($user_id, 'Bill_Checking', 'TCS_Applicable');
        $access['Internal_Audit'] = $this->checkAuthorityAccess($user_id, 'Bill_Checking', 'Internal_Audit');
        $access['Original_Bill_Needed'] = $this->checkAuthorityAccess($user_id, 'Bill_Checking', 'Original_Bill_Needed');

        return response()->json(['status' => 1, 'message' => 'Found Data', 'access' => $access]);
    }

    // get employee name dropdown
    public function getEmployeeName(Request $request)
    {
        $token_data = JWTAuth::parseToken()->authenticate();
        $tenantId = $token_data->tenant_id;

        $data = DB::table('tai_employee_master')
            ->select('id', 'tenant_id', 'account_for', 'employee_id', DB::raw("CONCAT(first_name,' ',last_name) AS employee"))
            ->where('is_active', 1)
            ->where('tenant_id', $tenantId)->get();
        return response()->json(['status' => 1, 'message' => 'Found Data', 'data' => $data]);
    }

    public function getStatusDropdown()
    {
        $token_data = JWTAuth::parseToken()->authenticate();

        $data = DB::table('tai_bc_status_conventions')->select('id', 'convention_name')->where('used_for', 'Payment_Status')->where('is_active', 1)->get()->toArray();
        $confirm = DB::table('tai_bc_status_conventions')->select('id', 'convention_name')->where('used_for', 'Billchecking_Payment_template')->where('is_active', 1)->get()->toArray();
        $data = array_merge($data, $confirm);
        return response()->json(['status' => 1, 'message' => 'Found Data', 'data' => $data]);
    }

    // get bill checking status solved/unsolved and canceled bill
    public function getBillCheckingStatus()
    {
        $token_data = JWTAuth::parseToken()->authenticate();

        $data = DB::table('tai_bc_status_conventions')->select('id', 'convention_name')->where('used_for', 'Billchecking_bill_status')->where('is_active', 1)->get()->toArray();
        if (count($data) > 0) {
            return response()->json(['status' => 1, 'message' => 'Found Data', 'data' => $data]);
        } else {
            return response()->json(['status' => 0, 'message' => 'Data Not Found', 'data' => $data]);
        }
    }

    // get data
    public function getBillCheck(Request $request, $user_id, $type = 0)
    {

        $token_data = JWTAuth::parseToken()->authenticate(); //added by PUNAM 30-05-2023
        $tenantId = $token_data->tenant_id;

        // $this->checkBillDate(1, '2022-11-30', '2022-11-29');

        $subQuery = DB::table('tai_bc_bill_multi_approval_authority AS bma')
            ->select(DB::raw('DISTINCT(bil_type_id)'))
            ->where('employee_id', $user_id)
            ->where('is_required', 1)
            ->where('bma.amount', '>', 'bct.bill_amount')
            ->whereRaw('bil_approval_level = bct.level + 1');

        $transactions = DB::table('tai_bc_bill_checking_transactions AS bct')
            ->whereIn('bill_type', $subQuery)
            ->get()->toArray();

        // return $transactions;

        foreach ($transactions as $key => $val) {
            $already = DB::table('tai_bc_bill_checking_approval_history as ah')
                ->where('bill_type_id', $val->bill_type)
                ->where('level', $val->level + 1)
                ->where('employee_id', $user_id)
                ->where('bc_id', $val->id)
                ->first();

            if ($already != null) {
                unset($transactions[$key]);
            }
        }

        $ids = array_column($transactions, 'id');

        // return ($ids);

        $user = $user_id;
        $result = DB::table('tai_bc_bill_checking_transactions as b')
            ->leftJoin('tai_employee_master as e', 'e.id', 'b.assign_to')
            ->leftJoin('tai_department_master as d', 'd.id', 'b.department_name')
            ->leftJoin('tai_bc_bill_type_master as bill', 'bill.id', 'b.bill_type')
            ->leftJoin('tai_bc_vendor_master as v', 'v.id', 'b.vendor_name')
            // ->where('b.id',$id)
            ->select(
                'b.*',
                DB::raw("CONCAT(e.first_name,' ',e.last_name) AS assign_to_name"),
                'd.department as department_name',
                'v.vendor_name as vendor_id_name',
                'bill.bill_type as bill_type_name',
                'b.created_by as creator',
                DB::raw("CONCAT(em.first_name,' ',em.last_name) AS created_by"),
                DB::raw("CONCAT(emp.first_name,' ',emp.last_name) AS updated_by"),
                DB::raw("(select bil_approval_level from tai_bc_bill_multi_approval_authority where bil_type_id = b.bill_type and amount > b.bill_amount limit 1) as total_level")
            )
            ->leftJoin('tai_employee_master as emp', 'emp.id', '=', 'b.updated_by')
            ->leftJoin('tai_employee_master as em', 'em.id', '=', 'b.created_by')

            // ->leftJoin('tai_bc_employee_bill_type_mapping as bm', 'bill.id','=','bm.bill_type_id')
            // ->leftJoin('tai_bc_employee_bill_type_mapping as bm1','b.assign_to','=','bm1.employee_id')
            ->where('b.assign_to', $user_id)
            ->orwhereIn('b.id', $ids)

            // ->get();
            // ->where('b.is_active',1)
        ;
        // if($user)
        // {
        //     $result = $result->where('b.assign_to',$user);

        // }

        if ($type != 0) {
            $result = $result->where('b.is_active', 1)->orderby('id', 'DESC')->get();
        } else {
            $result = $result->orderby('id', 'DESC')->get();
        }

        $path_name = explode('/public', $request->url())[0];

        foreach ($result as $r) {
            // dd($r->id);
            if (in_array($r->id, $ids)) {
                $r->is_approver = 1;
            } else {
                $r->is_approver = 0;
            }

            if ($r->assign_to == $user_id) {
                $r->is_assign_to = 1;
            } else {
                $r->is_assign_to = 0;
            }

            $data = DB::table('tai_attachments')->where('type', '=', 'BILL_CHECK')->where('ref_id', $r->id)->where('is_active', 1)->get();
            // dd($data);
            if (count($data) > 0) {
                foreach ($data as $d) {
                    $d->path = $path_name . $d->path;
                }
                $val = $data;
            } else {
                $val = [];
            }
            $r->attachment = $val;

            // bill status from payment details
            if ($r->is_active == 0) {
                $r->payment_status = 'Cancelled';
            } else {
                $payment_status = DB::table('tai_bc_payment_details')->where('tai_bc_bill_checking_transactions_id', $r->id)->where('payment_status', '!=', 15)->get();
                if (count($payment_status) >= 0) {
                    $r->payment_status = 'Unsolved';
                } else {
                    $r->payment_status = 'Solved';
                }
            }
            // dd($r);
            $tds_section_name = DB::table('tai_bc_section_master')->where('id', $r->tds_section)->first();
            $r->tds_section_name = $tds_section_name ? $tds_section_name->section_name : 'NA';

            $tds_constitution_name = DB::table('tai_bc_section_constitution_mapping')->where('id', $r->tds_constitution)->first();
            // dd($tds_constitution_name);
            $r->tds_constitution_name = $tds_constitution_name ? $tds_constitution_name->constitution_name : 'NA';

            if ($r->id) {

                $r->remark_history = [];
                $remark_data = DB::table('tai_bc_bill_checking_transactions_history')->select('remark', 'updated_by', 'updated_at', 'created_at', 'created_by', 'updated_by')->where('main_id', $r->id)->get();
                foreach ($remark_data as $val) {
                    if ($val->updated_by == null) {
                        $emp_name = DB::table('tai_employee_master as e')->select(DB::raw("CONCAT(e.first_name,' ',e.last_name) AS employee"))->where('e.id', $val->created_by)->first();
                        if ($emp_name) {
                            $date_time = date('d/m/Y H:i A', strtotime($val->created_at));
                            $value = $val->remark . ' - ' . $emp_name->employee . ' - ' . $date_time;
                        }
                    } else {
                        $emp_name = DB::table('tai_employee_master as e')->select(DB::raw("CONCAT(e.first_name,' ',e.last_name) AS employee"))->where('e.id', $val->updated_by)->first();
                        if ($emp_name) {
                            $date_time = date('Y-m-d H:i A', strtotime($val->updated_at));
                            $value = $val->remark . ' - ' . $emp_name->employee . ' - ' . $date_time;
                        }
                    }
                    // dd($value);
                    $r->remark_history[] = $value;
                }

                // dd($r->remark_history);

            }
            // $r->remark_history = DB::table('tai_bc_bill_checking_transactions_history')->distinct()->where('main_id', $r->id)->pluck('remark','updated_by','updated_at');
        }

        // dd($result);
        if ($result) {
            return response()->json(['status' => 1, 'message' => "Data Found !!!", 'data' => $result]);
        } else {
            return response()->json(['status' => 0, 'message' => "Required Used For Filed !!!", 'data' => NULL]);
        }
    }

    // get data by id
    public function getBillCheckById($id, $user_id, Request $request)
    {

        $token_data = JWTAuth::parseToken()->authenticate(); //added by PUNAM 30-05-2023
        $tenantId = $token_data->tenant_id;

        if (!$id) {
            return response()->json(['status' => 0, 'message' => 'Bill Id Not Found', 'data' => null]);
        }

        $result = DB::table('tai_bc_bill_checking_transactions as b')
            // ->leftJoin('tai_employee_master as e','e.id','b.assign_to')
            // ->leftJoin('tai_department_master as d','d.id','b.department_name')
            // ->leftJoin('tai_bc_bill_type_master as bill','bill.id','b.bill_type')
            // ->leftJoin('tai_bc_vendor_master as v','v.id','b.vendor_name')
            ->where('b.id', $id)
            // ->select('b.*',DB::raw("CONCAT(e.first_name,' ',e.last_name) AS assign_to"),'d.department as department_name','v.vendor_name')
            ->first();


        $levels = DB::table('tai_bc_bill_type_master as e')
            ->select(
                DB::raw("(CASE WHEN approver1 = null THEN 0 ELSE approver1 END) AS approver1"),
                DB::raw("(CASE WHEN approver2 = null THEN 0 ELSE approver2 END) AS approver2"),
                DB::raw("(CASE WHEN approver3 = null THEN 0 ELSE approver3 END) AS approver3"),
                DB::raw("(CASE WHEN approver4 = null THEN 0 ELSE approver4 END) AS approver4"),
            )
            ->where('id', $result->bill_type)->first();
        $totalLevels = 1;
        $level = ['approver2', 'approver3', 'approver4'];
        foreach ($level as $l) {
            if ($levels->$l != null) {
                $totalLevels++;
            }
        }
        // dd($totalLevels);


        if (is_null($result)) {
            return response()->json(['status' => 0, 'message' => 'Data Not Found', 'data' => null]);
        }

        $data = DB::table('tai_attachments')->where('type', '=', 'BILL_CHECK')->where('ref_id', $id)->where('is_active', 1)->get();

        if (count($data) > 0) {
            foreach ($data as $key => $value) {
                $value->path = explode('/public', $request->url())[0] . $value->path;
            }
            $result->attachment = $data;
        } else {
            $result->attachment = null;
        }

        $result->remark_history = DB::table('tai_bc_bill_checking_transactions_history')->where('main_id', $id)->pluck('remark');

        $result->remark_history = [];
        $remark_data = DB::table('tai_bc_bill_checking_transactions_history')->select('remark', 'updated_by', 'updated_at', 'created_at', 'created_by', 'updated_by')->where('main_id', $id)->get();
        foreach ($remark_data as $r_val) {
            // $remark_value = '';
            if ($r_val->updated_by == null && $r_val->remark) {

                $emp_name = DB::table('tai_employee_master as e')->select(DB::raw("CONCAT(e.first_name,' ',e.last_name) AS employee"))->where('e.id', $r_val->created_by)->first();
                if ($emp_name) {
                    $date_time = date('d/m/Y H:i A', strtotime($r_val->created_at));
                    $remark_value = $r_val->remark . ' - ' . $emp_name->employee . ' - ' . $date_time;
                    $result->remark_history[] = $remark_value;
                }
            } else {

                if ($r_val->remark) {
                    $emp_name = DB::table('tai_employee_master as e')->select(DB::raw("CONCAT(e.first_name,' ',e.last_name) AS employee"))->where('e.id', $r_val->updated_by)->first();
                    if ($emp_name) {
                        $date_time = date('Y-m-d H:i A', strtotime($r_val->updated_at));
                        $remark_value = $r_val->remark . ' - ' . $emp_name->employee . ' - ' . $date_time;
                        $result->remark_history[] = $remark_value;
                    }
                }
            }
            // dd($remark_value);

        }

        $check_id = DB::table('tai_employee_master')->where('id', $user_id)->where('is_active', 1)->first();
        if (!$check_id) {
            return response()->json(['status' => 0, 'message' => "Invalid User !!!", 'data' => NULL]);
        }

        // authorities
        $access['Update_Bill'] = $this->checkAuthorityAccess($user_id, 'Bill_Checking', 'Update_Bill');
        $access['Authorized_By_HOD'] = $this->checkAuthorityAccess($user_id, 'Bill_Checking', 'Authorized_By_HOD');
        $access['Authorized_By_Management'] = $this->checkAuthorityAccess($user_id, 'Bill_Checking', 'Authorized_By_Management');
        $access['Cancel_Bill'] = $this->checkAuthorityAccess($user_id, 'Bill_Checking', 'Cancel_Bill');
        $access['Internal_Audit'] = $this->checkAuthorityAccess($user_id, 'Bill_Checking', 'Internal_Audit');
        $access['External_Audit'] = $this->checkAuthorityAccess($user_id, 'Bill_Checking', 'External_Audit');
        $access['Received_Date'] = $this->checkAuthorityAccess($user_id, 'Bill_Checking', 'Received_Date');
        $access['TCS_Applicable'] = $this->checkAuthorityAccess($user_id, 'Bill_Checking', 'TCS_Applicable');
        $access['All_Update_Bill'] = $this->checkAuthorityAccess($user_id, 'Bill_Checking', 'All_Update_Bill');
        $access['Original_Bill_Needed'] = $this->checkAuthorityAccess($user_id, 'Bill_Checking', 'Original_Bill_Needed');
        // $access['Audit_Remark'] = $this->checkAuthorityAccess($user_id, 'Bill_Checking', 'Audit_Remark');

        $result->access = $access;
        $last_approved_by = DB::table('tai_bc_bill_checking_approval_history as ah')
            ->leftJoin("tai_employee_master as e", 'e.id', 'ah.created_by')
            ->where('bc_id', $id)
            // ->where('level', $result->level + 1)
            ->orderBy('ah.id')
            ->select(DB::raw("CONCAT(e.first_name,' ',e.last_name) AS employee"))
            ->distinct()
            ->where('status', 1)
            ->get()->toArray();
        if (count($last_approved_by) > 0) {
            $last_approved_by = implode(", ", array_column($last_approved_by, 'employee'));
        } else {
            $last_approved_by = "";
        }

        $level_approver = DB::table('tai_bc_bill_multi_approval_authority as ah')
            ->leftJoin("tai_employee_master as e", 'e.id', 'ah.employee_id')
            ->where('bil_type_id', $result->bill_type)
            ->where('bil_approval_level', $result->level + 1)
            ->where('is_required', 1)
            ->select(DB::raw("CONCAT(e.first_name,' ',e.last_name) AS employee"))
            ->get()->toArray();
        if (count($level_approver) > 0) {
            $level_approver = implode(", ", array_column($level_approver, 'employee'));
        } else {
            $level_approver = "";
        }

        $result->last_approved_by = $last_approved_by;
        $result->level_approver = $level_approver;

        // return $result;
        if ($result) {
            $result->total_levels = $totalLevels;
            return response()->json(['status' => 1, 'message' => "Data Found !!!", 'data' => $result]);
        } else {
            return response()->json(['status' => 0, 'message' => "Data Not Found !!!", 'data' => NULL]);
        }
    }

    // get assign person details
    public function getAssignPersonDetail($value = '')
    {
        $token_data = JWTAuth::parseToken()->authenticate(); //added by PUNAM 30-05-2023
        $tenantId = $token_data->tenant_id;

        // $data = DB::table('tai_bc_bill_checking_assign_person_history')->where('tai_bc_bill_checking_transactions_id', $value)
        //                                                         ->select('id','tai_bc_bill_checking_transactions_id','assign_to','days_count','created_at')
        //                                                         ->groupBy('assign_to')
        //                                                         ->orderBy('updated_at', 'asc')
        //                                                         ->get();

        $data = DB::table('tai_bc_bill_checking_assign_person_history as h')
            ->leftJoin('tai_employee_master as e', 'e.id', '=', 'h.assign_to')
            ->leftJoin('tai_bc_bill_checking_transactions as t', 't.id', '=', 'h.tai_bc_bill_checking_transactions_id')
            ->where('h.tai_bc_bill_checking_transactions_id', $value)
            ->groupBy('h.id')
            ->orderBy('h.updated_at', 'asc')
            ->select(
                'h.id',
                't.bc_id as tai_bc_bill_checking_transactions_id',
                'h.days_count',
                'h.created_at',
                DB::raw("CONCAT(e.first_name,' ',e.last_name) AS assign_to")
            )
            ->get();

        return response()->json(['status' => 1, 'message' => "Data Found !!!", 'data' => $data]);
    }

    public function get_yearcycle_code($input_date, $type = '')
    {
        $input_date_array = explode("-", $input_date);
        $input_year = $input_date_array[0];
        $input_month = $input_date_array[1];

        if ($input_month > 3) {
            $cycle_year = $input_year + 1;
        } else {
            $cycle_year = $input_year;
        }

        //$final_date = $cycle_year . "-03-31";

        $start_year = 2019;
        $start_char = 65;

        if ($type  == "padm_doc") {
            $start_char = 69;
        }

        $diff_years = $cycle_year - $start_year;

        if ($diff_years >= 0) {
            if ($diff_years > 25) {
                $result = "Z" . ($diff_years - 25);
            } else {
                $result = chr($diff_years + $start_char);
            }

            return $result;
        } else {
            return -1;
        }
    }

    public function get_billchecking_id($input_date)
    {
        $year_code = $this->get_yearcycle_code($input_date);
        $table_name = 'tai_bc_bill_checking_transactions';

        $data_bpd = DB::table($table_name)->select('bc_id')->where('bc_id', 'LIKE', $year_code . "%")->orderBy('id', 'desc')->first();

        if (isset($data_bpd->bc_id)) {
            $bcid_parse = explode($year_code, $data_bpd->bc_id);
            $bc_id = $year_code . ($bcid_parse[1] + 1);
            // $bc_id = $year_code . ($bcid_parse[1] + 2);
        } else {
            $bc_id = $year_code . "1";
        }

        return $bc_id;
    }

    // create bill
    public function createBillCheck(Request $request)
    {
        $token_data = JWTAuth::parseToken()->authenticate(); //added by PUNAM 30-05-2023
        $tenantId = $token_data->tenant_id;

        $bc_id = $this->get_billchecking_id(date('Y-m-d', strtotime($request->bill_date)));

        // return [$request->all()];
        // dd($bc_id);
        // dd($bc_id, $valid);
        // if (!$lastId || $lastId == null) {
        //     $bc_id = "2022";
        // } else {
        //     $a = substr($lastId->bc_id, 1);
        //     $bc_id = (int)$a + 1;
        // }

        // check extension for file
        $extensions = ['jpg', 'pdf', 'jpeg', 'png', 'xls', 'xlsx', 'docs'];

        // validation
        if (!$request->created_by) {
            return response()->json(['status' => 0, 'message' => "Created by user Required !!!", 'data' => null]);
        }
        if (!$request->bill_type) {
            return response()->json(['status' => 0, 'message' => "Bill Type Required !!!", 'data' => null]);
        } elseif (!$request->assign_to) {
            return response()->json(['status' => 0, 'message' => "Assign To Field Required !!!", 'data' => null]);
        } elseif (!$request->vendor_name) {
            return response()->json(['status' => 0, 'message' => "Vendor Name Required !!!", 'data' => null]);
        } elseif (!$request->vendor_bill_no) {
            return response()->json(['status' => 0, 'message' => "Vendor Bill No Required !!!", 'data' => null]);
        } elseif (!$request->bill_date) {
            return response()->json(['status' => 0, 'message' => "Bill date Required !!!", 'data' => null]);
        }
        // elseif (!$request->taxable_amount){
        //     return response()->json([ 'status'=> 0, 'message'=> "Taxable Amount Required !!!",'data' => null]);
        // }elseif (!$request->tcs){
        //     return response()->json([ 'status'=> 0, 'message'=> "TCS Required !!!",'data' => null]);
        // }elseif (!$request->remark){
        //     return response()->json([ 'status'=> 0, 'message'=> "Remark Required !!!",'data' => null]);
        // }elseif (!$request->audit_remark){
        //     return response()->json([ 'status'=> 0, 'message'=> "Audit Remark Required !!!",'data' => null]);
        // }

        if (strlen($request->vendor_bill_no) >= 26) {
            return response()->json(['status' => 0, 'message' => "Vendor Bill No. should accept maximum length of 26 values!!!", 'data' => null]);
        }


        $extensions = ['jpg', 'pdf', 'jpeg', 'png'];

        // attachment validation
        if ($request->attachment) {
            // if (is_array($request->attachment) && count($request->attachment) >= 0) {
            //     for ($i = 0; $i < count($request->attachment); $i++) {
            //         // return($request->file('attachment'));
            //         if ($request->file('attachment')[$i] && !in_array($request->file('attachment')[$i]->getClientOriginalExtension(), $extensions)) {
            //             return response()->json(['status' => 0, 'message' =>  "Invalid File Format !!!", 'data' => null]);
            //         }
            //     }
            // } else {
            //     if ($request->file('attachment') && !in_array($request->file('attachment')->getClientOriginalExtension(), $extensions)) {
            //         return response()->json(['status' => 0, 'message' =>  "Invalid File Format !!!", 'data' => null]);
            //     }
            // }
        } else {
            return response()->json(['status' => 0, 'message' =>  "required invoice attachment !!!", 'data' => null]);
        }

        if ($request->vendor_bill_no != "NA") {
            $check = DB::table('tai_bc_bill_checking_transactions')->where('vendor_bill_no', $request->vendor_bill_no)->first();
            if ($check) {
                return response()->json(['status' => 0, 'message' => "Vendor Bill No. Already Exists !!!", 'data' => null]);
            }
        }

        // create a array 
        $result = array(
            'bc_id' =>  $bc_id,
            'bill_type'    => $request->bill_type,
            'assign_to'    => $request->assign_to,
            'vendor_name'    => $request->vendor_name ? $request->vendor_name : 'NA',
            'vendor_bill_no' => $request->vendor_bill_no,
            'bill_date' => $request->bill_date,
            'received_date'        => date('Y-m-d'), // cureent date
            "debit_advance" => $request->debit_advance,
            "taxable_amount" => $request->taxable_amount,
            "tcs" => $request->tcs ?  $request->tcs : 0,
            "round_off" => $request->round_off,
            "is_igst_applicable" => $request->is_igst_applicable ? 1 : 0,
            "is_tds_applicable" => $request->is_tds_applicable ? 1 : 0,
            "is_tds_applicable" => $request->is_tds_applicable ? 1 : 0,
            "tds_percentage" => $request->tds_percentage ? $request->tds_percentage : 0,
            "tds_section" => $request->tds_section ? $request->tds_section : 0,
            "tds_constitution" => $request->tds_constitution ? $request->tds_constitution : 0,
            "tds_amount" => $request->tds_amount ? $request->tds_amount : 0,
            "net_payment" => $request->net_payment,
            "net_payment_in_word" => $request->net_payment_in_word,
            "is_active" => 1,
            "remark" => $request->remark,
            "narration" => $request->narration,
            'audit_remark' => $request->audit_remark,
            'created_at' => $request->created_at ? $request->created_at : now(),
            'created_by' => $request->created_by
        );


        // set template agent setting date
        // get payment template date
        $pay_date =  $this->checkBillDate($request->vendor_name, $request->bill_date, $request->recieved_date);

        if ($pay_date) {

            // return $pay_date['result'];
            $request->payment_date = $pay_date['result']->payment_date;
            $result['payment_date'] = $request->payment_date;
        } else {
            return response()->json(['status' => 0, 'message' => "Invalid payment date", 'data' => null]);
        }

        // check gst & IGST

        if ($request->is_igst_applicable) {
            $result['gst_amount'] = 0;
            // $result['igst_amount'] = $request->igst_amount;
            $result['bill_amount'] = $request->taxable_amount + $request->igst_amount + $request->round_off + $request->tcs;
            $result['igst_amount'] = $request->igst_amount;
            // $request->gst_amount = 0;
            // $result['gst_amount'] = 0;
        } else {
            $result['bill_amount'] = $request->taxable_amount + $request->gst_amount + $request->round_off + $request->tcs;
            $result['gst_amount'] = $request->gst_amount;
            // $request->igst_amount = 0;
            $result['igst_amount'] = 0;
        }

        // if ($request->igst_amount) {
        //     // Bill Amount = Taxable Amount + IGST + Round Off + TCS
        //     $result['bill_amount'] = $request->taxable_amount + $request->igst_amount + $request->round_off + $request->tcs;
        //     $result['igst_amount'] = $request->igst_amount;
        //     // $request->gst_amount = 0;
        //     $result['gst_amount'] = 0;
        // } else {
        //     // Bill Amount = Taxable Amount + GST + Round Off + TCS
        //     $result['bill_amount'] = $request->taxable_amount + $request->gst_amount + $request->round_off + $request->tcs;
        //     $result['gst_amount'] = $request->gst_amount;
        //     // $request->igst_amount = 0;
        //     $result['igst_amount'] = 0;
        // }

        // return [$result, 1];
        // get fixed 2 decimnal number 
        $bill_amount_arr = explode('.', $result['bill_amount']);
        if (count($bill_amount_arr) > 1) {
            $bill_amount_arr[1] = $bill_amount_arr[1] ? substr($bill_amount_arr[1], 0, 2) : '';
            $result['bill_amount'] = implode('.', $bill_amount_arr);
        } else {
            $result['bill_amount'] = $result['bill_amount'];
        }

        // return $result['bill_amount'];

        // return [round($request->bill_amount), round($result['bill_amount']), ceil($result['bill_amount'])];
        if (round($request->bill_amount) != round($result['bill_amount'])) {
            return response()->json(['status' => 0, 'message' => "Bill Amount Is Invalid !!!", 'data' => NULL]);
        }

        // check debit advance
        if ($request->debit_advance > $result['bill_amount']) {
            return response()->json(['status' => 0, 'message' => "Debit Advance Should Not be Greater Than Bill Amount !!!", 'data' => NULL]);
        }

        // net payment
        // Net Payment = Bill Amount - Debit Advance - TDS Amount 
        if ($request->net_payment) {
            $request->tds_amount ? $request->tds_amount : $request->tds_amount = 0;
            $result['net_payment'] = round($result['bill_amount'] - $request->debit_advance - $request->tds_amount);
        }
        if ($request->net_payment != $result['net_payment']) {
            return response()->json(['status' => 0, 'message' => "Net Payment Amount Is Invalid !!!", 'data' => NULL]);
        }

        // TDS AMOUNT = bill amount * (TDS% /100)
        if ($request->tds_amount > 0) {
            $result['tds_amount'] = ceil($request->taxable_amount * ($request->tds_percentage / 100));
            if ($result['tds_amount'] != $request->tds_amount) {
                return response()->json(['status' => 0, 'message' => "TDS Amount Is Invalid !!!", 'data' => NULL]);
            }
        }

        $result['is_active'] = 1;

        // dd($result);
        // insert all data
        $data = DB::table('tai_bc_bill_checking_transactions')->insertGetId($result);
        $history_id = CommonController::generateHistory("INSERT", 'tai_bc_bill_checking_transactions', $data);

        if ($data) {
            $this->insertPaymentDetails($data); // insert payment details

            $attachment_ids = [];
            // insert attachments
            if ($request->attachment) {
                if (is_array($request->attachment) && count($request->attachment) >= 0) {
                    //  if(count($request->attachment) >= 0){
                    foreach ($request->file('attachment') as $f) {
                        // upload file
                        $fileName = $f->getClientOriginalName();

                        // $file = 'storage/app/Attachment/' . 'BillChecking/Vendor/'. $name.'/';
                        // $path = storage_path().'/app/Attachment/BillChecking/Vendor/'. $name.'/';
                        $base =  storage_path() . '/app/Attachment/BillChecking/Bill_Invoice/';

                        $filePath = $base . "/" . $fileName;

                        $f->move($base, $fileName);

                        $file_url = '/storage/app/Attachment/' . 'BillChecking/Bill_Invoice/' . $fileName;

                        $att_res = DB::table('tai_attachments')->insertGetId([
                            'ref_id' => $data,
                            'name' => $f->getClientOriginalName(),
                            'path' => $file_url,
                            'type' => 'BILL_CHECK',
                            'is_active' => 1,
                            'created_at' => now(),
                            'created_by' => $request->created_by ? $request->created_by : 1
                        ]);
                        $attachment_ids[] = $att_res;
                    }
                }
            }

            $table = 'tai_bc_bill_checking_transactions';
            $result = (array) DB::table('tai_bc_bill_checking_transactions')->where('id', $data)->first();
            // $history_id = CommonController::generateHistory("INSERT", $table, $data);
            foreach ($attachment_ids as $id) {
                $this->createAttachmentHistory($id, $history_id);
            }


            DB::table('tai_bc_bill_checking_assign_person_history')
                ->insert(['tai_bc_bill_checking_transactions_id' => $result['id'], 'assign_to' => $result['assign_to'], 'created_at' => now()]);

            return response()->json(['status' => 1, 'message' => "Bill No " . $bc_id . " Has Been Added Successfull !!!", 'data' => $result]);
        } else {
            return response()->json(['status' => 0, 'message' => "Failed To Add Bill !!!", 'data' => NULL]);
        }
    }

    public function updateBillChecktesting(Request $request, $id)
    {

        // $checkEdit = DB::table('tai_bc_bill_multi_approval_authority as mt')
        //     ->select('ct.*')
        //     ->join('tai_bc_employee_bill_type_mapping as btm', 'btm.employee_id', '=', 'mt.employee_id')
        //     ->leftjoin('tai_bc_bill_checking_transactions as ct', 'ct.bill_type', '=', 'mt.bil_type_id')
        //     ->where('ct.bill_type', $request->bill_type)
        //     ->where('btm.employee_id', $id)
        //     ->where('mt.is_required', 1)
        //     ->where('ct.level', 0)
        //     ->distinct('ct.bill_type')
        //     ->get();
        // dd($checkEdit);
        // if ($checkEdit) {
        //     foreach ($checkEdit as $edit) {
        //         $levelUp = $edit->level;
        //         if ($levelUp == 0) {
        //             $levelUp++;
        //         }
        //         $insertData = DB::table('tai_bc_bill_checking_approval_history')->insert([
        //             'bill_type_id' => $request->bill_type,
        //             'level' => $levelUp,
        //             'created_at' => date('Y-m-d h:m:s'),
        //         ]);
        //     }
        // } else {

        //     return 'is not editable';
        // }

        // $checkEdit = DB::table('tai_bc_bill_checking_transactions as bct')
        //     ->select('bct.*')
        //     ->whereIn('bct.bill_type', (function ($query) {
        //         $query->from('tai_bc_bill_multi_approval_authority as ma')
        //             ->where('ma.employee_id', '=', 455)
        //             ->where('ma.is_required', '=', 1)
        //             ->where('ma.bil_approval_level', '=', DB::raw('bct.level'));
        //     }))
        //     ->get();


        // $employeeId = $id;
        // $subQuery = DB::table('tai_bc_bill_multi_approval_authority AS bma')
        //     ->select(DB::raw('DISTINCT(bil_type_id)'))
        //     ->where('employee_id', $employeeId)
        //     ->where('is_required', 1)
        //     ->whereRaw('bil_approval_level = bct.level + 1');
        // $transactions = DB::table('tai_bc_bill_checking_transactions AS bct')
        //     ->whereIn('bill_type', $subQuery)
        //     ->whereNotIn('bct.id', function ($query) {
        //         $query->select('cht.bill_type_id')
        //             ->from('tai_bc_bill_checking_approval_history as cht')
        //             ->where('cht.bill_type_id', '!=' ,DB::raw('tai_bc_bill_checking_transactions.bill_type'));
        //     })
        //     ->get();
        //     dd($transactions);

        // $employeeId = $id;
        // $subQuery = DB::table('tai_bc_bill_multi_approval_authority AS bma')
        //     ->select(DB::raw('DISTINCT(bil_type_id)'))
        //     ->where('employee_id', $employeeId)
        //     ->where('is_required', 1)
        //     ->whereRaw('bil_approval_level = bct.level + 1');
        // $transactions = DB::table('tai_bc_bill_checking_transactions AS bct')
        //     ->whereIn('bill_type', $subQuery)
        //     ->get();

        // foreach ($transactions as $result) {
        //     // dd($result);   
        //     $notInTable = DB::table('tai_bc_bill_checking_approval_history')
        //         ->where('bc_id', '!=', $result->id)
        //         ->where('employee_id', '!=', $id)
        //         ->having('level', '>=', 1)
        //         ->get();
        //     if (count($notInTable) >= 1) {
        //         $transactions1 = DB::table('tai_bc_bill_checking_transactions AS bct')
        //             ->where('level', 0)->where('id', '!=', $notInTable->bc_id)
        //             ->get();
        //     }
        // }

        $employeeId = $id;
        $insertQuery = DB::table('tai_bc_bill_checking_transactions AS bct')
            ->whereIn('bill_type', function ($query) use ($employeeId) {
                $query->select(DB::raw('DISTINCT(bil_type_id)'))
                    ->from('tai_bc_bill_multi_approval_authority AS bma')
                    ->where('employee_id', $employeeId)
                    ->where('is_required', 1)
                    ->where('bma.is_active', 1)
                    ->whereRaw('bil_approval_level = bct.level+1');
            })->whereNotIn('id', function ($query) use ($id) {
                $query->select('employee_id')
                    ->from('tai_bc_bill_checking_approval_history')
                    ->where('employee_id', '!=', $id);
            })
            ->get();

        if (count($insertQuery) > 1) {
            foreach ($insertQuery as $insert) {

                $resultTwo = array(
                    'bill_type'    => $insert->bill_type,
                    'assign_to'    => $insert->assign_to,
                    'vendor_bill_no' => $insert->vendor_bill_no,
                    'bill_date' => $insert->bill_date,
                    'received_date'        => date('Y-m-d'), //current date
                    "debit_advance" => $insert->debit_advance,
                    "taxable_amount" => $insert->taxable_amount ? $insert->taxable_amount : 0,
                    "tcs" => $insert->tcs ? $insert->tcs : 0,
                    "round_off" => $insert->round_off,
                    "bill_amount" => $insert->bill_amount,
                    "tds_percentage" => $insert->tds_percentage ? $insert->tds_percentage : 0, // tds checkbox is active
                    "is_igst_applicable" => $insert->igst_amount  ? 1 : 0,
                    "is_tds_applicable" => $insert->is_tds_applicable  ? 1 : 0,
                    "tds_section" => $insert->tds_section ? $insert->tds_section : 0, //// tds checkbox is active
                    "tds_constitution" => $insert->tds_constitution, // // tds checkbox is active
                    "tds_amount" => $insert->tds_amount ? $request->tds_amount : 0,  // tds checkbox is active
                    "net_payment" => $insert->net_payment,
                    "net_payment_in_word" => $insert->net_payment_in_word,
                    "is_active" => $insert->is_active,
                    "remark" => $insert->remark,
                    'audit_remark' => $insert->audit_remark,
                    'external_audit_remark' => $insert->external_audit_remark,
                    'updated_at' => $insert->updated_at,
                    'updated_by' => $insert->updated_by
                );
            }
        }


        //dd($transactions);
        // $bct = DB::table('tai_bc_bill_checking_transactions AS bct')
        //     ->whereIn('bill_type', function ($query) {
        //         $query->select(DB::raw('DISTINCT(bil_type_id)'))
        //             ->from('tai_bc_bill_multi_approval_authority AS bma')
        //             ->where('employee_id', 40)
        //             ->where('is_required', '=', 1)
        //             ->whereRaw('bil_approval_level = bct.level + 1');
        //     })->whereNotIn('id', function ($query) {
        //         $query->select('bc_id')
        //             ->from('tai_bc_bill_checking_approval_history');
        //     })->whereNull('bc_id')
        //     ->get();
        // dd($bct);

        // $checkBcId = DB::table('tai_bc_bill_checking_transactions')->select('id')->where('id', $id)->first();
        // // dd($checkBcId->id);
        // if ($transactions) {
        //     $insertData = DB::table('tai_bc_bill_checking_approval_history')->insert([
        //         'bill_type_id' => $request->bill_type,
        //         'employee_ id' =>  $employeeId,
        //         'bc_id' =>  $checkBcId->id,
        //         'level' => 2,
        //         'created_at' => date('Y-m-d h:m:s'),
        //     ]);
        //     //dd( $insertData);
        // }
        // $checkExists = DB::table('tai_bc_bill_checking_transactions as ct')
        //     ->select('ct.*')
        //     ->leftjoin('tai_bc_bill_checking_approval_history as ch', 'ch.bc_id', '=', 'ct.id')
        //     ->join('tai_bc_bill_multi_approval_authority as ma', 'ma.bil_type_id', '=', 'ct.bill_type')
        //     ->whereNull('ct.id')
        //     ->get();



        $extensions = ['jpg', 'pdf', 'jpeg', 'png'];

        if (!$request->bill_type) {
            return response()->json(['status' => 0, 'message' => "Bill Type Required !!!", 'data' => null]);
        } elseif (!$request->assign_to) {
            return response()->json(['status' => 0, 'message' => "Assign To Field Required !!!", 'data' => null]);
        } elseif (!$request->vendor_bill_no) {
            return response()->json(['status' => 0, 'message' => "Vendor Bill No Required !!!", 'data' => null]);
        } elseif (!$request->bill_date) {
            return response()->json(['status' => 0, 'message' => "Bill date Required !!!", 'data' => null]);
        }

        // Vendor Bill No. should accept maximum length of 15 values
        if (strlen($request->vendor_bill_no) >= 26) {
            return response()->json(['status' => 0, 'message' => "Vendor Bill No. should accept maximum length of 26 values!!!", 'data' => null]);
        }

        if ($request->vendor_bill_no != "NA") {
            $check = DB::table('tai_bc_bill_checking_transactions')->where('id', '!=', $id)->where('vendor_bill_no', $request->vendor_bill_no)->first();
            if ($check) {
                return response()->json(['status' => 0, 'message' => "Vendor Bill No. Already Exists !!!", 'data' => null]);
            }
        }

        if (!$request->bill_type) {
            return response()->json(['status' => 0, 'message' => "Bill Type Required !!!", 'data' => null]);
        }

        // create array 
        $result = array(
            'bill_type'    => $request->bill_type,
            'assign_to'    => $request->assign_to,
            'vendor_bill_no' => $request->vendor_bill_no,
            'bill_date' => $request->bill_date,
            'received_date'        => date('Y-m-d'), //current date
            "debit_advance" => $request->debit_advance,
            "taxable_amount" => $request->taxable_amount ? $request->taxable_amount : 0,
            "tcs" => $request->tcs ? $request->tcs : 0,
            "round_off" => $request->round_off,
            "bill_amount" => $request->bill_amount,
            "tds_percentage" => $request->tds_percentage ? $request->tds_percentage : 0, // tds checkbox is active
            "is_igst_applicable" => $request->igst_amount  ? 1 : 0,
            "is_tds_applicable" => $request->is_tds_applicable  ? 1 : 0,
            "tds_section" => $request->tds_section ? $request->tds_section : 0, //// tds checkbox is active
            "tds_constitution" => $request->tds_constitution, // // tds checkbox is active
            "tds_amount" => $request->tds_amount ? $request->tds_amount : 0,  // tds checkbox is active
            "net_payment" => $request->net_payment,
            "net_payment_in_word" => $request->net_payment_in_word,
            "is_active" => $request->is_active,
            "remark" => $request->remark,
            'audit_remark' => $request->audit_remark,
            'external_audit_remark' => $request->external_audit_remark,
            'updated_at' => $request->updated_at,
            'updated_by' => $request->updated_by
        );
        // bill_type

        // 
        // check gst & IGST
        if ($request->igst_amount) {
            // Bill Amount = Taxable Amount + IGST + Round Off + TCS
            $result['bill_amount'] = $request->taxable_amount + $request->igst_amount + $request->round_off + $request->tcs;
            $result['igst_amount'] = $request->igst_amount;
        }
        if ($request->gst_amount) {
            // Bill Amount = Taxable Amount + GST + Round Off + TCS
            $result['bill_amount'] = $request->taxable_amount + $request->gst_amount + $request->round_off + $request->tcs;
            $result['gst_amount'] = $request->gst_amount;
        }

        // get fixed 2 decimnal number 
        $bill_amount_arr = explode('.', $result['bill_amount']);
        if (count($bill_amount_arr) > 1) {
            $bill_amount_arr[1] = $bill_amount_arr[1] ? substr($bill_amount_arr[1], 0, 2) : '';
            $result['bill_amount'] = implode('.', $bill_amount_arr);
        } else {
            $result['bill_amount'] = $result['bill_amount'];
        }


        // dd(round($request->bill_amount) , round($result['bill_amount']));

        if (round($request->bill_amount) != round($result['bill_amount'])) {
            return response()->json(['status' => 0, 'message' => "Bill Amount Is Invalid !!!", 'data' => NULL]);
        }


        // check debit advance
        if ($request->debit_advance >= $result['bill_amount']) {
            return response()->json(['status' => 0, 'message' => "Debit Advance Should Not be Greater Than Bill Amount !!!", 'data' => NULL]);
        }

        // net payment
        // Net Payment = Bill Amount - Debit Advance - TDS Amount 
        if ($request->net_payment) {
            $request->tds_amount ? $request->tds_amount : $request->tds_amount = 0;
            $result['net_payment'] = round($result['bill_amount'] - $request->debit_advance - $request->tds_amount);
        }
        if ($request->net_payment != $result['net_payment']) {
            return response()->json(['status' => 0, 'message' => "Net Payment Amount Is Invalid !!!", 'data' => NULL]);
        }

        // dd($request);

        // TDS AMOUNT = bill amount * (TDS% /100)
        if ($request->tds_amount > 0) {
            $result['tds_amount'] = ceil($request->taxable_amount * ($request->tds_percentage / 100));
            if ($result['tds_amount'] != $request->tds_amount) {
                return response()->json(['status' => 0, 'message' => "TDS Amount Is Invalid !!!", 'data' => NULL]);
            }
        }
        // dd($result);
        $data = DB::table('tai_bc_bill_checking_transactions')->where('id', $id)->update($result);
        $this->insertPaymentDetails($id, $request->source ? $request->source : null);

        if ($data) {

            $this->insertPaymentDetails($id, $request->source ? $request->source : null); // insert payment details

            $attachment_ids = [];

            // insert attachments
            if ($request->attachment) {
                // if(is_array($request->attachment) &&
                if (is_array($request->attachment) && count($request->attachment) >= 0) {
                    foreach ($request->attachment as $f) {
                        // upload file
                        $fileName = time() . $f->getClientOriginalName();

                        // $base=  storage_path('BillChecking/Bill_Invoice/');
                        $base =  storage_path() . '/app/Attachment/BillChecking/Bill_Invoice/';

                        $filePath = $base . "/" . $fileName;

                        $f->move($base, $fileName);

                        $file_url = 'storage/app/Attachment/' . 'BillChecking/Bill_Invoice/' . $fileName;

                        $res = DB::table('tai_attachments')->insertGetId([
                            'ref_id' => $id,
                            'name' => $f->getClientOriginalName(),
                            'path' => $file_url,
                            'type' => 'BILL_CHECK',
                            'is_active' => 1,
                            'created_at' => now(),
                            'created_by' => $request->created_by ? $request->created_by : 1
                        ]);

                        $attachment_ids[] = $res;
                    }
                }
            }

            $table = 'tai_bc_bill_checking_transactions';
            $result = (array) DB::table('tai_bc_bill_checking_transactions')->where('id', $id)->first();
            $history_id = CommonController::generateHistory("UPDATE", $table, $id);
            foreach ($attachment_ids as $id) {
                $this->createAttachmentHistory($id, $history_id);
            }

            $check_bill = DB::table('tai_bc_bill_checking_assign_person_history')
                ->where('tai_bc_bill_checking_transactions_id', $result['id'])
                ->orderby('created_at', 'desc')
                ->select('id', 'tai_bc_bill_checking_transactions_id', 'assign_to', 'days_count', 'created_at')
                ->first();

            // assign person days count
            if ($check_bill == null) {
                DB::table('tai_bc_bill_checking_assign_person_history')->insert(['tai_bc_bill_checking_transactions_id' => $result['id'], 'assign_to' => $result['assign_to'], 'created_at' => now()]);
            } elseif ($result['assign_to'] != $check_bill->assign_to) {
                $current_date = date('Y-m-d H:i:s');
                $date = date_create(date('Y-m-d'));
                $created_at = date_create(date('Y-m-d', strtotime($check_bill->created_at)));
                $datediff = date_diff($date, $created_at);

                DB::table('tai_bc_bill_checking_assign_person_history')->where('id', $check_bill->id)->update(['updated_at' => now(), 'days_count' => $datediff->days]);

                DB::table('tai_bc_bill_checking_assign_person_history')->insert(['tai_bc_bill_checking_transactions_id' => $result['id'], 'assign_to' => $result['assign_to']]);
            }


            // return response()->json(['status' => 1, 'message' => "Bill Update Successfull !!!", 'data' => $result]);
            return response()->json(['status' => 1, 'message' => "Bill Update Successfull !!!", 'data' => $resultTwo]);
        } else {
            return response()->json(['status' => 0, 'message' => "Failed To Update Bill !!!", 'data' => NULL]);
        }
    }

    // update a bill
    public function updateBillCheck(Request $request, $id)
    {

        $token_data = JWTAuth::parseToken()->authenticate(); //added by PUNAM 30-05-2023
        $tenantId = $token_data->tenant_id;

        // if ($request->bill_type) {
        //     $check_editable = DB::table('tai_bc_bill_multi_approval_authority as mt')
        //         ->select('mt.bil_type_id as bil_type_id ')
        //         ->join('tai_bc_bill_checking_transactions as ct', 'ct.bill_type', '=', 'mt.bil_type_id')
        //         ->where('mt.bil_type_id', $request->bill_type)->get();
        //     dd($check_editable);
        // }

        $complete = DB::table('tai_bc_bill_checking_approval_history as ah')
            ->where('ah.bc_id', $id)
            ->where('ah.level', $request->level + 1);
        $approved = $complete->where('status', 1)->get()->count();
        $complete = $complete->get()->count();

        $levels_required = DB::table('tai_bc_bill_multi_approval_authority as mt')
            ->where('mt.bil_type_id', $request->bill_type)
            ->where('mt.bil_approval_level', $request->level + 1)
            ->first()->is_required_number;

        if ($levels_required) {
            $new_level = $request->level;
        }
        // $new_level = $request->level;
        if ($levels_required == $approved + 1 && $request->status == 1) {
            $new_level = $request->level + 1;
        }
        // dd($complete, $approved, $levels_required, $new_level);
        // return [$complete, $approved, $levels_required, $new_level];

        $result = array(
            'bill_type'    => $request->bill_type,
            'assign_to'    => $request->assign_to,
            'vendor_bill_no' => $request->vendor_bill_no,
            'bill_date' => $request->bill_date,
            'received_date'        => date('Y-m-d'), //current date
            "debit_advance" => $request->debit_advance,
            "taxable_amount" => $request->taxable_amount ? $request->taxable_amount : 0,
            "tcs" => $request->tcs ? $request->tcs : 0,
            "round_off" => $request->round_off,
            "bill_amount" => $request->bill_amount,
            "tds_percentage" => $request->tds_percentage ? $request->tds_percentage : 0, // tds checkbox is active
            "is_igst_applicable" => $request->igst_amount  ? 1 : 0,
            "is_tds_applicable" => $request->is_tds_applicable  ? 1 : 0,
            "tds_section" => $request->tds_section ? $request->tds_section : 0, //// tds checkbox is active
            "tds_constitution" => $request->tds_constitution, // // tds checkbox is active
            "tds_amount" => $request->tds_amount ? $request->tds_amount : 0,  // tds checkbox is active
            "net_payment" => $request->net_payment,
            "net_payment_in_word" => $request->net_payment_in_word,
            "is_active" =>  $request->is_active ? ($request->is_active == 0 ? 0 : 1) : 1,
            "remark" => $request->remark,
            "narration" => $request->narration,
            "level" => (int)$new_level,
            'audit_remark' => $request->audit_remark,
            'external_audit_remark' => $request->external_audit_remark,
            'updated_at' => $request->updated_at,
            'updated_by' => $request->updated_by
        );
        // return $result;

        $extensions = ['jpg', 'pdf', 'jpeg', 'png'];

        if (!$request->bill_type) {
            return response()->json(['status' => 0, 'message' => "Bill Type Required !!!", 'data' => null]);
        } elseif (!$request->assign_to) {
            return response()->json(['status' => 0, 'message' => "Assign To Field Required !!!", 'data' => null]);
        } elseif (!$request->vendor_bill_no) {
            return response()->json(['status' => 0, 'message' => "Vendor Bill No Required !!!", 'data' => null]);
        } elseif (!$request->bill_date) {
            return response()->json(['status' => 0, 'message' => "Bill date Required !!!", 'data' => null]);
        }

        // Vendor Bill No. should accept maximum length of 26 values
        if (strlen($request->vendor_bill_no) >= 26) {
            return response()->json(['status' => 0, 'message' => "Vendor Bill No. should accept maximum length of 26 values!!!", 'data' => null]);
        }

        if ($request->vendor_bill_no != "NA") {
            $check = DB::table('tai_bc_bill_checking_transactions')->where('id', '!=', $id)->where('vendor_bill_no', $request->vendor_bill_no)->first();
            if ($check) {
                return response()->json(['status' => 0, 'message' => "Vendor Bill No. Already Exists !!!", 'data' => null]);
            }
        }

        if (!$request->bill_type) {
            return response()->json(['status' => 0, 'message' => "Bill Type Required !!!", 'data' => null]);
        }

        // create array 
        // $result = array(
        //     'bill_type'    => $request->bill_type,
        //     'assign_to'    => $request->assign_to,
        //     'vendor_bill_no' => $request->vendor_bill_no,
        //     'bill_date' => $request->bill_date,
        //     'received_date'        => date('Y-m-d'), //current date
        //     "debit_advance" => $request->debit_advance,
        //     "taxable_amount" => $request->taxable_amount ? $request->taxable_amount : 0,
        //     "tcs" => $request->tcs ? $request->tcs : 0,
        //     "round_off" => $request->round_off,
        //     "bill_amount" => $request->bill_amount,
        //     "tds_percentage" => $request->tds_percentage ? $request->tds_percentage : 0, // tds checkbox is active
        //     "is_igst_applicable" => $request->igst_amount  ? 1 : 0,
        //     "is_tds_applicable" => $request->is_tds_applicable  ? 1 : 0,
        //     "tds_section" => $request->tds_section ? $request->tds_section : 0, //// tds checkbox is active
        //     "tds_constitution" => $request->tds_constitution, // // tds checkbox is active
        //     "tds_amount" => $request->tds_amount ? $request->tds_amount : 0,  // tds checkbox is active
        //     "net_payment" => $request->net_payment,
        //     "net_payment_in_word" => $request->net_payment_in_word,
        //     "is_active" => $request->is_active,
        //     "remark" => $request->remark,
        //     'audit_remark' => $request->audit_remark,
        //     'external_audit_remark' => $request->external_audit_remark,
        //     'updated_at' => $request->updated_at,
        //     'updated_by' => $request->updated_by
        // );
        // bill_type

        // 
        // check gst & IGST
        if ($request->igst_amount) {
            // Bill Amount = Taxable Amount + IGST + Round Off + TCS
            $result['bill_amount'] = $request->taxable_amount + $request->igst_amount + $request->round_off + $request->tcs;
            $result['igst_amount'] = $request->igst_amount;
        }
        if ($request->gst_amount) {
            // Bill Amount = Taxable Amount + GST + Round Off + TCS
            $result['bill_amount'] = $request->taxable_amount + $request->gst_amount + $request->round_off + $request->tcs;
            $result['gst_amount'] = $request->gst_amount;
        }

        // get fixed 2 decimnal number 
        $bill_amount_arr = explode('.', $result['bill_amount']);
        if (count($bill_amount_arr) > 1) {
            $bill_amount_arr[1] = $bill_amount_arr[1] ? substr($bill_amount_arr[1], 0, 2) : '';
            $result['bill_amount'] = implode('.', $bill_amount_arr);
        } else {
            $result['bill_amount'] = $result['bill_amount'];
        }

        // dd($result['bill_amount']);

        if (round($request->bill_amount) != round($result['bill_amount'])) {
            return response()->json(['status' => 0, 'message' => "Bill Amount Is Invalid !!!", 'data' => NULL]);
        }


        // check debit advance
        if ($request->debit_advance >= $result['bill_amount']) {
            return response()->json(['status' => 0, 'message' => "Debit Advance Should Not be Greater Than Bill Amount !!!", 'data' => NULL]);
        }

        // net payment
        // Net Payment = Bill Amount - Debit Advance - TDS Amount 
        if ($request->net_payment) {
            $request->tds_amount ? $request->tds_amount : $request->tds_amount = 0;
            $result['net_payment'] = round($result['bill_amount'] - $request->debit_advance - $request->tds_amount);
        }
        if ($request->net_payment != $result['net_payment']) {
            return response()->json(['status' => 0, 'message' => "Net Payment Amount Is Invalid !!!", 'data' => NULL]);
        }



        // TDS AMOUNT = bill amount * (TDS% /100)
        if ($request->tds_amount > 0) {
            $result['tds_amount'] = ceil($request->taxable_amount * ($request->tds_percentage / 100));
            if ($result['tds_amount'] != $request->tds_amount) {
                return response()->json(['status' => 0, 'message' => "TDS Amount Is Invalid !!!", 'data' => NULL]);
            }
        }

        $data = DB::table('tai_bc_bill_checking_transactions')->where('id', $id)->update($result);

        if ($request->status) {
            DB::table('tai_bc_bill_checking_approval_history')->insert([
                'employee_id' => $request->approver_id,
                'bc_id' => $id,
                'bill_type_id' => $request->bill_type,
                'level' => $request->level + 1,
                'status' => $request->status,
                'created_at' => date('Y-m-d H:i:s'),
                'created_by' => $request->approver_id
            ]);
        }

        $this->insertPaymentDetails($id, $request->source ? $request->source : null);

        if ($data) {

            $this->insertPaymentDetails($id, $request->source ? $request->source : null); // insert payment details

            $attachment_ids = [];

            // insert attachments
            if ($request->attachment) {
                // if(is_array($request->attachment) &&
                if (is_array($request->attachment) && count($request->attachment) >= 0) {
                    foreach ($request->attachment as $f) {
                        // upload file
                        $fileName = time() . $f->getClientOriginalName();

                        // $base=  storage_path('BillChecking/Bill_Invoice/');
                        $base =  storage_path() . '/app/Attachment/BillChecking/Bill_Invoice/';

                        $filePath = $base . "/" . $fileName;

                        $f->move($base, $fileName);

                        $file_url = 'storage/app/Attachment/' . 'BillChecking/Bill_Invoice/' . $fileName;

                        $res = DB::table('tai_attachments')->insertGetId([
                            'ref_id' => $id,
                            'name' => $f->getClientOriginalName(),
                            'path' => $file_url,
                            'type' => 'BILL_CHECK',
                            'is_active' => 1,
                            'created_at' => now(),
                            'created_by' => $request->created_by ? $request->created_by : 1
                        ]);

                        $attachment_ids[] = $res;
                    }
                }
            }

            $table = 'tai_bc_bill_checking_transactions';
            $result = (array) DB::table('tai_bc_bill_checking_transactions')->where('id', $id)->first();
            $history_id = CommonController::generateHistory("UPDATE", $table, $id);
            foreach ($attachment_ids as $id) {
                $this->createAttachmentHistory($id, $history_id);
            }

            $check_bill = DB::table('tai_bc_bill_checking_assign_person_history')
                ->where('tai_bc_bill_checking_transactions_id', $result['id'])
                ->orderby('created_at', 'desc')
                ->select('id', 'tai_bc_bill_checking_transactions_id', 'assign_to', 'days_count', 'created_at')
                ->first();

            // assign person days count
            if ($check_bill == null) {
                DB::table('tai_bc_bill_checking_assign_person_history')->insert(['tai_bc_bill_checking_transactions_id' => $result['id'], 'assign_to' => $result['assign_to'], 'created_at' => now()]);
            } elseif ($result['assign_to'] != $check_bill->assign_to) {
                $current_date = date('Y-m-d H:i:s');
                $date = date_create(date('Y-m-d'));
                $created_at = date_create(date('Y-m-d', strtotime($check_bill->created_at)));
                $datediff = date_diff($date, $created_at);

                DB::table('tai_bc_bill_checking_assign_person_history')->where('id', $check_bill->id)->update(['updated_at' => now(), 'days_count' => $datediff->days]);

                DB::table('tai_bc_bill_checking_assign_person_history')->insert(['tai_bc_bill_checking_transactions_id' => $result['id'], 'assign_to' => $result['assign_to']]);
            }

            // return response()->json(['status' => 1, 'message' => "Bill Update Successfull !!!", 'data' => $result]);
            return response()->json(['status' => 1, 'message' => "Bill Update Successfull !!!", 'data' => $result]);
        } else {
            return response()->json(['status' => 0, 'message' => "Failed To Update Bill !!!", 'data' => NULL]);
        }
    }

    // upload Invoice file 
    public function uploadAttachment($attachment)
    {
        if ($attachment) {
            $fileName = time() . $attachment->getClientOriginalName();

            $file = 'storage/app/Attachment/BillChecking/Bill_Invoice/';
            $path = storage_path('app/Attachment/BillChecking/Bill_Invoice/');

            // $base = 'http://10.2.11.2/connectus-dummy/storage/app/car-booking/start_km/' . $fileName;
            $base = $file . $fileName;

            $attachment->move($path, $fileName);
            return $base;
        }
    }

    // create attachment history
    public function createAttachmentHistory($id, $history_id)
    {

        $data = DB::table('tai_attachments')->where('id', $id)->get();

        foreach ($data as $key => $row) {
            unset($row->id);
            $row->history_id = $history_id;
            $row->tai_attachments_id = $id;

            DB::table('tai_attachments_history')->insert([
                'history_id' => $history_id, 'tai_attachments_id' => $id, 'ref_id' => $row->ref_id,
                'name' => $row->name, 'path' => $row->path, 'type' => $row->type, 'shown_to_all' => $row->shown_to_all,
                'show_to_customer' => $row->show_to_customer, 'remark' => $row->remark, 'is_active' => $row->is_active,
                'created_at' => $row->created_at, 'created_by' => $row->created_by
            ]);
        }
    }

    public function getBillCheckHistoryById($id, Request $request)
    {
        $token_data = JWTAuth::parseToken()->authenticate(); //added by PUNAM 30-05-2023
        $tenantId = $token_data->tenant_id;

        $data = DB::table('tai_bc_bill_checking_transactions_history as b')
            ->leftJoin('tai_employee_master as e', 'e.id', 'b.assign_to')
            ->leftJoin('tai_department_master as d', 'd.id', 'b.department_name')
            ->leftJoin('tai_bc_bill_type_master as bill', 'bill.id', 'b.bill_type')
            ->leftJoin('tai_bc_vendor_master as v', 'v.id', 'b.vendor_name')
            // ->where('b.id',$id)
            ->leftJoin('tai_employee_master as emp', 'emp.id', '=', 'b.updated_by')
            ->leftJoin('tai_employee_master as em', 'em.id', '=', 'b.created_by')
            ->leftJoin('tai_bc_section_master as section', 'section.id', '=', 'b.tds_section')
            ->leftJoin('tai_bc_section_constitution_mapping as map', 'map.id', '=', 'b.tds_constitution')
            ->where('b.main_id', $id)
            ->select(
                'b.*',
                DB::raw("CONCAT(e.first_name,' ',e.last_name) AS assign_to"),
                'd.department as department_name',
                'v.vendor_name as vendor_name',
                'bill.bill_type as bill_type',
                'section.section_name as tds_section',
                'map.constitution_name as tds_constitution',
                DB::raw("CONCAT(em.first_name,' ',em.last_name) AS created_by"),
                DB::raw("CONCAT(emp.first_name,' ',emp.last_name) AS updated_by")
            )
            ->orderby('b.id', 'desc')
            ->get();

        foreach ($data as $key => $row) {
            $row->attachments = DB::table('tai_attachments_history')->where('history_id', $row->id)->select('name', 'path', 'type')->where('is_active', 1)->get();

            foreach ($row->attachments as $a) {
                $path_name = explode('/public', $request->url())[0];
                $a->path = $path_name . $a->path;
            }
        }

        $temp = [];


        if (count($data) > 0) {
            foreach ($data as $d) {
                $temp[] = (array)$d;
            }

            if (count($temp) >= 0) {

                foreach ($temp as $key => $value) {
                    // dd('1');
                    $temp[$key]['changes'] = [];
                    if ($key >= 1) {
                        $prevKey = $key - 1;

                        $tempPrevArray = $temp[$prevKey];
                        $tempCurrentArray = $temp[$key];

                        if (array_key_exists('changes', $tempPrevArray)) {
                            unset($tempPrevArray['changes']);
                        }
                        if (array_key_exists('changes', $tempCurrentArray)) {
                            unset($tempCurrentArray['changes']);
                        }
                        $diff = (array_diff_assoc($tempPrevArray, $tempCurrentArray));
                        unset($diff['id']);
                        unset($diff['operation']);

                        $temp[$key]['changes'] = array_keys($diff);
                    }
                }
            }




            // dd($temp);
            return response()->json(['status' => 1, 'message' => "Record found !!!", 'data' => $temp]);
        } else {
            return response()->json(['status' => 0, 'message' => "No Record found !!!", 'data' => null]);
        }
    }

    // // get bill checking transaction history
    // public function getBillCheckHistory(Request $request)
    // {
    //     $data = DB::table('tai_bc_bill_checking_transactions_history as b')
    //             ->leftJoin('tai_employee_master as e','e.id','b.assign_to')
    //             ->leftJoin('tai_department_master as d','d.id','b.department_name')
    //             ->leftJoin('tai_bc_bill_type_master as bill','bill.id','b.bill_type')
    //             ->leftJoin('tai_bc_vendor_master as v','v.id','b.vendor_name')
    //             // ->where('b.id',$id)
    //             ->select('b.*',DB::raw("CONCAT(e.first_name,' ',e.last_name) AS assign_to"),'d.department as department_name','v.vendor_name as vendor_id_name ',
    //             'bill.bill_type as bill_type_name',
    //                 DB::raw("CONCAT(em.first_name,' ',em.last_name) AS created_by"),DB::raw("CONCAT(emp.first_name,' ',emp.last_name) AS updated_by"))
    //             ->leftJoin('tai_employee_master as emp','emp.id','=','b.updated_by')
    //             ->leftJoin('tai_employee_master as em','em.id','=','b.created_by')
    //             ->get();

    //     return response()->json(['status' => 1, 'message'=>"Found Data", 'data' => $data]);
    // }

    public function getBillCheckHistory(Request $request)
    {
        $token_data = JWTAuth::parseToken()->authenticate(); //added by PUNAM 30-05-2023
        $tenantId = $token_data->tenant_id;

        $data = DB::table('tai_bc_bill_checking_transactions_history as b')
            ->leftJoin('tai_employee_master as e', 'e.id', 'b.assign_to')
            ->leftJoin('tai_department_master as d', 'd.id', 'b.department_name')
            ->leftJoin('tai_bc_bill_type_master as bill', 'bill.id', 'b.bill_type')
            ->leftJoin('tai_bc_vendor_master as v', 'v.id', 'b.vendor_name')
            // ->where('b.id',$id)
            ->select(
                'b.*',
                DB::raw("CONCAT(e.first_name,' ',e.last_name) AS assign_to"),
                'd.department as department_name',
                'v.vendor_name as vendor_id_name',
                'bill.bill_type as bill_type_name',
                DB::raw("CONCAT(em.first_name,' ',em.last_name) AS created_by"),
                DB::raw("CONCAT(emp.first_name,' ',emp.last_name) AS updated_by")
            )
            ->leftJoin('tai_employee_master as emp', 'emp.id', '=', 'b.updated_by')
            ->leftJoin('tai_employee_master as em', 'em.id', '=', 'b.created_by')
            ->get();

        $temp = [];

        if (count($data) > 0) {
            foreach ($data as $d) {
                $temp[] = (array)$d;
            }

            if (count($temp) >= 2) {

                foreach ($temp as $key => $value) {
                    $temp[$key]['changes'] = [];
                    if ($key >= 1) {
                        $prevKey = $key - 1;

                        $tempPrevArray = $temp[$prevKey];
                        $tempCurrentArray = $temp[$key];

                        if (array_key_exists('changes', $tempPrevArray)) {
                            unset($tempPrevArray['changes']);
                        }
                        if (array_key_exists('changes', $tempCurrentArray)) {
                            unset($tempCurrentArray['changes']);
                        }
                        $diff = (array_diff_assoc($tempPrevArray, $tempCurrentArray));
                        unset($diff['id']);
                        unset($diff['operation']);

                        $temp[$key]['changes'] = array_keys($diff);
                    }
                }
            }

            return response()->json(['status' => 1, 'message' => "Record found !!!", 'data' => $temp]);
        } else {
            return response()->json(['status' => 0, 'message' => "No Record found !!!", 'data' => null]);
        }
    }

    public function getPaymentDetailsHistory(Request $request, $id)
    {
        $token_data = JWTAuth::parseToken()->authenticate(); //added by PUNAM 30-05-2023
        $tenantId = $token_data->tenant_id;

        $token_data = JWTAuth::parseToken()->authenticate(); //added by PUNAM 30-05-2023
        $tenantId = $token_data->tenant_id;

        $data = DB::table('tai_bc_payment_details_history as pd')
            // ->leftJoin('tai_bc_bill_checking_transactions as b', 'b.id','=','pd.tai_bc_bill_checking_transactions_id')
            // ->leftJoin('tai_employee_master as e','e.id','b.assign_to')
            // ->leftJoin('tai_department_master as d','d.id','b.department_name')
            ->leftJoin('tai_bc_bill_type_master as bill', 'bill.id', 'pd.bill_type_id')
            ->leftJoin('tai_bc_vendor_master as v', 'v.id', 'pd.vendor_id')
            ->leftJoin('tai_employee_master as emp', 'emp.id', '=', 'pd.updated_by')
            ->leftJoin('tai_employee_master as em', 'em.id', '=', 'pd.created_by')
            ->leftJoin('tai_bc_status_conventions as con', 'con.id', '=', 'pd.payment_status')
            ->where('pd.tai_bc_bill_checking_transactions_id', $id)
            ->select(
                'pd.*',
                'v.vendor_name as vendor_id_name',
                'bill.bill_type as bill_type_id_name',
                'con.convention_name as payment_status_name',
                DB::raw("CONCAT(em.first_name,' ',em.last_name) AS created_by"),
                DB::raw("CONCAT(emp.first_name,' ',emp.last_name) AS updated_by"),
                'pd.ip_address',
                'pd.user_agent'
            )
            ->get();

        $temp = [];

        if (count($data) > 0) {
            foreach ($data as $d) {
                $temp[] = (array)$d;
            }

            if (count($temp) >= 0) {

                foreach ($temp as $key => $value) {
                    // dd($temp, $key);
                    $temp[$key]['changes'] = [];
                    if ($key >= 1) {
                        $prevKey = $key - 1;

                        $tempPrevArray = $temp[$prevKey];
                        $tempCurrentArray = $temp[$key];

                        if (array_key_exists('changes', $tempPrevArray)) {
                            unset($tempPrevArray['changes']);
                        }
                        if (array_key_exists('changes', $tempCurrentArray)) {
                            unset($tempCurrentArray['changes']);
                        }
                        $diff = (array_diff_assoc($tempPrevArray, $tempCurrentArray));
                        unset($diff['id']);
                        unset($diff['operation']);

                        $temp[$key]['changes'] = array_keys($diff);
                    }
                }
            }

            // dd($temp);
            return response()->json(['status' => 1, 'message' => "Record found !!!", 'data' => $temp]);
        } else {
            return response()->json(['status' => 0, 'message' => "No Record found !!!", 'data' => null]);
        }
    }

    // insert payment details
    public function insertPaymentDetails($id, $source = NULL)
    {
        $token_data = JWTAuth::parseToken()->authenticate(); //added by PUNAM 30-05-2023
        $tenantId = $token_data->tenant_id;

        // dd($id);
        $transaction_data = DB::table('tai_bc_bill_checking_transactions')->where('id', $id)->first();

        $bill_detail_data = DB::table('tai_bc_payment_details')
            ->select('tai_bc_bill_checking_transactions_id', DB::raw('SUM(amount_to_be_paid) as amount_to_be_paid'))
            ->where('tai_bc_bill_checking_transactions_id', $id)
            ->groupBy('tai_bc_bill_checking_transactions_id')
            ->where('is_active', 1)
            ->first();
        // dd($bill_detail_data);

        $old_amount = $bill_detail_data ? $bill_detail_data->amount_to_be_paid : 0;

        $new_amount = 0;
        if ($transaction_data->net_payment != NULL) {
            $new_amount = $transaction_data->net_payment;
        }

        if ($old_amount >= 1) {
            // dd(2);

            $new_amount = $transaction_data->net_payment;
            if ($transaction_data->net_payment == NULL) {
                $new_amount = 0;
            }
            // dd($new_amount,$old_amount);
            // if ($new_amount != $old_amount)
            // {
            //     // dd('hi');
            //     $amt_to_be_difference = 0;

            //     $bill_date_data = $this->checkBillDate($transaction_data->vendor_name, $transaction_data->bill_date, $transaction_data->received_date);

            //     $pay_status=[16,17];
            //     $updatePayments=0;
            //     // dd($new_amount < $old_amount);
            //     if ($new_amount < $old_amount) {

            //         $amt_to_be_difference = $old_amount - $new_amount;
            //         $perct = $amt_to_be_difference / $old_amount * 100;
            //         $query='UPDATE `tai_bc_payment_details` SET `amount_to_be_paid`= round(amount_to_be_paid-(amount_to_be_paid*' . $perct . '/100)) 
            //                 WHERE `tai_bc_bill_checking_transactions_id` = ' . $transaction_data->id . ' AND `is_active`=1 AND 
            //                 `payment_status` IN ('.implode(',',$pay_status).')';
            //         $updatePayments=DB::select(DB::raw($query));
            //     }

            //     if ($new_amount > $old_amount) {

            //         $amt_to_be_difference = $new_amount - $old_amount;
            //         $perct =  $amt_to_be_difference / $old_amount * 100;
            //         $query='UPDATE `tai_bc_payment_details` SET `amount_to_be_paid`= round(amount_to_be_paid+(amount_to_be_paid*' . $perct . '/100)) 
            //                 WHERE `tai_bc_bill_checking_transactions_id` = ' . $transaction_data->id . 
            //                 ' AND `is_active`=1 AND `payment_status` IN ('.implode(',',$pay_status).')';
            //         $updatePayments=DB::select(DB::raw($query));  
            //         // dd($new_amount , $old_amount, $amt_to_be_difference, $perct);                                      
            //     }

            //     $arrayName = array(
            //         'payment_date'    => $bill_date_data['result']->payment_date,
            //         'bill_type_id'       => $transaction_data->bill_type,
            //         'vendor_id'       => $transaction_data->vendor_name,
            //         'payment_status'  => 16,   // HOLD TILL AUDIT ID
            //         'is_active'       => 1,
            //         'tai_bc_bill_checking_transactions_id' => $transaction_data->id,
            //         'remark'          => 'Auto entry due to change in amount.',
            //         'created_at' => $transaction_data->created_at,
            //         'created_by' => $transaction_data->created_by
            //     );

            //     $bill_detail_data = DB::table('tai_bc_payment_details')
            //                         ->select('tai_bc_bill_checking_transactions_id', DB::raw('SUM(amount_to_be_paid) as amount_to_be_paid'))
            //                         ->where('tai_bc_bill_checking_transactions_id', $id)
            //                         ->groupBy('tai_bc_bill_checking_transactions_id')
            //                         ->where('is_active', 1)
            //                         ->first();

            //     $new_old_amount =  $new_amount - $bill_detail_data->amount_to_be_paid;

            //     $arrayName['amount_to_be_paid']  = $new_old_amount;
            //     // if ($new_old_amount != 0) {
            //     //   $id = DB::table('tai_bc_payment_details')->insertGetId($arrayName);
            //     // }
            //     $slab = 9900000; 

            //     if ($old_amount < $new_amount) {
            //         $new_diff_amt = $new_amount - $old_amount;
            //         $loopInsert = $this->checkSlabAmount(round($new_diff_amt), $slab);
            //         // dd($loopInsert);
            //         if ($loopInsert) {
            //           for ($i = 1; $i <= $loopInsert; $i++) {
            //             $amt = 0;
            //             if ($i == $loopInsert) {
            //               $amt = $new_diff_amt - ($slab * ($i - 1));
            //             } else {
            //               $amt = $slab;
            //             }        
            //             $arrayName['amount_to_be_paid'] = round($amt);
            //             // dd($arrayName);
            //             $id = DB::table('tai_bc_payment_details')->insertGetId($arrayName);
            //           }
            //        }
            //     } else {
            //       $new_diff_amt = $old_amount - $new_amount;
            //       $bill_payment_data = DB::table('tai_bc_payment_details')
            //                             ->where('tai_bc_bill_checking_transactions_id', $id)
            //                             ->where('is_active', 1)
            //                             ->get();
            //       $arr = array('updated_by' => 40, 'updated_at' => date('Y-m-d H:i:s'), 'remark' => 'Auto update due to change in amount.');
            //       $loopInsert = $this->checkSlabAmount(round($new_diff_amt), $slab);
            //     //   dd($loopInsert);
            //       for ($i = 0; $i < $loopInsert; $i++) {
            //         if ($new_diff_amt >= $bill_payment_data[$i]->amount_to_be_paid) {
            //           $new_diff_amt = $new_diff_amt - $bill_payment_data[$i]->amount_to_be_paid;
            //           $arr['amount_to_be_paid'] = 0;
            //         } else {
            //           $new_diff_amt = $bill_payment_data[$i]->amount_to_be_paid - $new_diff_amt;
            //           $arr['amount_to_be_paid'] = $new_diff_amt;
            //         }
            //         // dd($arr);
            //         DB::table('tai_bc_payment_details')->where('id', $bill_payment_data[$i]->id)->update($arr);
            //       }
            //     }
            // }

            if ($new_amount != $old_amount) {
                // dd('hi');
                $amt_to_be_difference = 0;

                $bill_date_data = $this->checkBillDate($transaction_data->vendor_name, $transaction_data->bill_date, $transaction_data->received_date);

                $arrayName = array(
                    'payment_date'    => $bill_date_data['result']->payment_date,
                    'bill_type_id'       => $transaction_data->bill_type,
                    'vendor_id'       => $transaction_data->vendor_name,
                    'payment_status'  => 16,   // HOLD TILL AUDIT ID
                    'is_active'       => 1,
                    'tai_bc_bill_checking_transactions_id' => $transaction_data->id,
                    'remark'          => 'Auto entry due to change in amount.',
                    'created_at' => $transaction_data->created_at,
                    'created_by' => $transaction_data->created_by
                );
                $new_diff_amt = 0;
                $slab = 9900000;

                if ($old_amount < $new_amount) {
                    $new_diff_amt = $new_amount - $old_amount;

                    $loopInsert = $this->checkSlabAmount(round($new_diff_amt), $slab);

                    if ($loopInsert) {
                        for ($i = 1; $i <= $loopInsert; $i++) {
                            $amt = 0;
                            if ($i == $loopInsert) {
                                // $amt=$transaction_data->net_amount - ($slab * ($i-1));
                                $amt = $new_diff_amt - ($slab * ($i - 1));
                            } else {
                                $amt = $slab;
                            }

                            $arrayName['amount_to_be_paid'] = round($amt);
                            $id = DB::table('tai_bc_payment_details')->insertGetId($arrayName);
                            // General::create_history($id, 'ADD', 'bc_bill_payment_detail');
                        }
                    }
                } else {

                    $new_diff_amt = $old_amount - $new_amount;
                    $bill_payment_data = DB::table('tai_bc_payment_details')
                        ->where('tai_bc_bill_checking_transactions_id', $id)
                        ->where('is_active', 1)
                        ->get();
                    $arr = array('updated_by' => 40, 'updated_at' => date('Y-m-d H:i:s'), 'remark' => 'Auto update due to change in amount.');
                    $loopInsert = $this->checkSlabAmount(round($new_diff_amt), $slab);

                    for ($i = 0; $i < $loopInsert; $i++) {
                        if ($new_diff_amt >= $bill_payment_data[$i]->amount_to_be_paid) {
                            $new_diff_amt = $new_diff_amt - $bill_payment_data[$i]->amount_to_be_paid;
                            $arr['amount_to_be_paid'] = 0;
                        } else {
                            $new_diff_amt = $bill_payment_data[$i]->amount_to_be_paid - $new_diff_amt;
                            $arr['amount_to_be_paid'] = $new_diff_amt;
                        }

                        DB::table('tai_bc_payment_details')->where('id', $bill_payment_data[$i]->id)->update($arr);
                    }
                }
            }
            // dd('bye');
        } else {

            // dd(1);
            // new entry for bill detail
            // do not consider --> paid mark else hold till audit

            $consider_in_payment = DB::table('tai_bc_vendor_master')->where('id', $transaction_data->vendor_name)->first();

            //Generate seperate txt file for petty cash bill type
            // added consider_in_payment 2 for petty cash
            if ($consider_in_payment->consider_in_payment == 'YES' || $consider_in_payment->consider_in_payment == 'NO' || $consider_in_payment->consider_in_payment == 'PETTY_CASH') {
                // dd('array');
                // $payment_status = DB::table('tai_bc_status_conventions')->where('used_for','Payment_Status')->where('id',16)->first();
                $arrayName = array(
                    'payment_date'    => $transaction_data->payment_date,
                    'amount_to_be_paid'  => round($transaction_data->net_payment),
                    'bill_type_id'       => $transaction_data->bill_type,
                    'vendor_id'       => $transaction_data->vendor_name,
                    'payment_status'  => 16,  // HOLD TILL AUDIT

                    'is_active'       => 1,
                    'tai_bc_bill_checking_transactions_id' => $transaction_data->id,
                    'remark'          => 'Do not consider in payment vendor.',
                    'created_at' => $transaction_data->created_at,
                    'updated_at' => $transaction_data->updated_at,
                    'created_by' => $transaction_data->created_by,
                    'updated_by' => $transaction_data->updated_by
                );
            } else {
                // dd('b');
                // $payment_status = DB::table('tai_bc_status_conventions')->where('used_for','Payment_Status')->where('convention_name','Paid')->first()->id;
                $arrayName = array(
                    'payment_date'    => $transaction_data->payment_date,
                    'amount_to_be_paid'  => round($transaction_data->net_payment),
                    'bill_type_id'       => $transaction_data->bill_type,
                    'vendor_id'       => $transaction_data->vendor_name,
                    'payment_status'  => 15, //paid
                    'is_active'       => 1,
                    'tai_bc_bill_checking_transactions_id' => $transaction_data->id,
                    'remark'          => 'Do not consider in payment vendor.',
                    'created_at' => $transaction_data->created_at,
                    'updated_at' => $transaction_data->updated_at,
                    'created_by' => $transaction_data->created_by,
                    'updated_by' => $transaction_data->updated_by
                );
            }

            // amount <=3 crore amount insert multiple records
            $slab = 9900000; //DB::table('cms_settings')->where('id', 20)->first()->content;
            $loopInsert = $this->checkSlabAmount(round($transaction_data->net_payment), $slab);
            // dd($loopInsert);
            if ($loopInsert) {
                for ($i = 1; $i <= $loopInsert; $i++) {
                    $amt = 0;
                    if ($i == $loopInsert) {
                        $amt = $transaction_data->net_payment - ($slab * ($i - 1));
                    } else {
                        $amt = $slab;
                    }
                    $arrayName['amount_to_be_paid'] = round($amt);
                    $id = DB::table('tai_bc_payment_details')->insertGetId($arrayName);
                    $table = 'tai_bc_payment_details';
                    CommonController::generateHistory("UPDATE", $table, $id);
                }
            }
        }
    }


    // Description : check bill date for set templates
    public function checkBillDate($vendor_master_id, $bill_date, $recieved_date)
    {
        $token_data = JWTAuth::parseToken()->authenticate(); //added by PUNAM 30-05-2023
        $tenantId = $token_data->tenant_id;

        $result = DB::table('tai_bc_vendor_master as v')
            // 'v.payment_template', 'p.template_name', 'p.min_days', 'p.bill_type', 'p.payment_type', 'p.bill_type')
            ->join('tai_bc_payment_template as p', 'v.payment_template', 'p.id')
            ->where('v.id', $vendor_master_id)
            ->select('v.*', 'p.*',) //'p.template_name','p.payment_type','p.bill_type','p.min_days','p.bill_day')
            ->first();
        // dd($vendor_master_id, $result, $check);

        if ($result) {

            if ($result->bill_type) {
                $check = DB::table('tai_bc_status_conventions')->where('id', $result->bill_type)->first();

                //Code for getting Bill Date
                if ($check  && $check->id == 9) {
                    $bill_date = date("Y-m-d", strtotime("$bill_date + $result->min_days day"));
                    $check_payment_type = DB::table('tai_bc_status_conventions')->where('id', $result->payment_type)->first();

                    if ($check_payment_type->id == 11) {
                        $payment_date = $this->getPaymentDate($bill_date, $result->bill_day);
                        $result->payment_date = $payment_date;
                    } elseif ($check_payment_type->id == 12) {
                        $payment_date = $this->getWeeklyPaymentDate($bill_date, $result->payment_weekly);
                        $result->payment_date = $payment_date;
                    }
                }

                //Code for getting Received Date
                if ($check  && $check->id == 10) {
                    //  Monthly
                    // payment date same as recived date. as per discuss with priyanka madam
                    if ($result->min_days == '-1') {
                        $bill_date = date("Y-m-d", strtotime("$recieved_date $result->min_days days"));
                    } else {
                        $bill_date = date("Y-m-d", strtotime("$recieved_date + $result->min_days days"));
                    }
                    $check_payment_type = DB::table('tai_bc_status_conventions')->where('id', $result->payment_type)->first();
                    if ($check_payment_type->id == 11) {
                        $payment_date = $this->getPaymentDate($bill_date, $result->bill_day);
                        $result->payment_date = $payment_date;
                    } else if ($check_payment_type->id == 12) {
                        $payment_date = $this->getWeeklyPaymentDate($bill_date, $result->payment_weekly);
                        $result->payment_date = $payment_date;
                    }
                }
            }
        }
        $data['result'] = $result;
        return $data;
    }

    // Description : pass week number and return week number name
    public function getWeek($week_no)
    {
        switch ($week_no) {
            case 1:
                return 'sunday';
                break;
            case 2:
                return "monday";
                break;
            case 3:
                return "tuesday";
                break;
            case 4:
                return "wednesday";
                break;
            case 5:
                return "thursday";
                break;
            case 6:
                return "friday";
                break;
            case 7:
                return "saturday";
                break;
        }
    }

    // Description : Monthly get payment date function
    public function getPaymentDate($date, $bill_day)
    {
        // dd($date, $bill_day);
        $bill_day = str_replace(" ", "", $bill_day);
        $explode_day = explode(',', $bill_day);
        $payment_date = '';

        $time = strtotime($date);
        $month = date("m", $time);
        $year = date("Y", $time);
        // dd($month, $year);
        $date_arr = array();

        for ($i = 0; $i < count($explode_day); $i++) {
            $date_from_day = $year . "-" . $month . "-" . $explode_day[$i];
            // dd($date_from_day);
            if ($time < strtotime($date_from_day)) {
                $date_arr[$i] = $date_from_day;
            } else {
                $date_arr[$i] = date('Y-m-d', strtotime('+1 month', strtotime($date_from_day)));
            }
        }
        // dd($date_arr);

        usort($date_arr, function ($a, $b) {
            $dateTimestamp1 = strtotime($a);
            $dateTimestamp2 = strtotime($b);
            return $dateTimestamp1 < $dateTimestamp2 ? -1 : 1;
        });

        return $date_arr[0];
    }

    public function getWeeklyPaymentDate($bill_date, $bill_day)
    {
        $explode_day = explode(',', $bill_day);
        $week_day = '';

        $date_arr = array();

        for ($i = 0; $i < count($explode_day); $i++) {
            $date_arr[$i] = date('Y-m-d', strtotime('next ' . self::getWeek($explode_day[$i]), strtotime($bill_date)));
        }

        usort($date_arr, function ($a, $b) {
            $dateTimestamp1 = strtotime($a);
            $dateTimestamp2 = strtotime($b);
            return $dateTimestamp1 < $dateTimestamp2 ? -1 : 1;
        });

        return $date_arr[0];
    }

    public function checkSlabAmount($amount, $slab)
    {
        $cnt = ceil($amount / $slab);
        return $cnt;
    }


    // Get Payment details
    public function getPaymentDetails($id, $user_id)
    {
        $token_data = JWTAuth::parseToken()->authenticate(); //added by PUNAM 30-05-2023
        $tenantId = $token_data->tenant_id;
        // $access[] = null;
        // dd('hi');
        $data = DB::table('tai_bc_payment_details as pd')
            ->leftJoin('tai_bc_bill_type_master as btm', 'pd.bill_type_id', '=', 'btm.id')
            ->leftJoin('tai_bc_vendor_master as vm', 'pd.vendor_id', '=', 'vm.id')
            ->leftJoin('tai_bc_status_conventions as sc', 'pd.payment_status', '=', 'sc.id')
            ->where('tai_bc_bill_checking_transactions_id', $id)
            ->where('pd.is_active', 1)
            ->select('pd.*', 'btm.bill_type as bill_type_name', 'vm.vendor_name', 'sc.convention_name as payment_status_name')
            ->get();

        $dropdown = DB::select("SELECT `id`, `convention_name` FROM `tai_bc_status_conventions` WHERE `used_for` = 'Payment_Status' AND `is_active` = 1 ");

        $check_user = DB::table('tai_employee_master')->where('is_active', 1)->where('id', $user_id)->first();
        if ($check_user && $check_user != null) {
            $access['Paid_Entry_Change'] = $this->checkAuthorityAccess($user_id, 'Bill_Checking', 'Paid_Entry_Change');
            $access['Amount_To_Be_Paid'] = $this->checkAuthorityAccess($user_id, 'Bill_Checking', 'Amount_To_Be_Paid');
            // $access['Auto_Update_Payment'] = $this->checkAuthorityAccess($user_id, 'Bill_Checking', 'Auto_Update_Payment' );
            // $access['Past_Financial_Year_Bill_Date'] = $this->checkAuthorityAccess($user_id, 'Bill_Checking', 'Past_Financial_Year_Bill_Date' );
            $access['Payment_Date'] = $this->checkAuthorityAccess($user_id, 'Bill_Checking', 'Payment_Date');
            $access['Payment_Status_Release'] = $this->checkAuthorityAccess($user_id, 'Bill_Checking', 'Payment_Status_Release');
            $access['Update_Payment_Details'] = $this->checkAuthorityAccess($user_id, 'Bill_Checking', 'Update_Payment_Details');

            if ($access['Paid_Entry_Change'] == true) {
                // status id Paid
                // $dropdown = DB::select("SELECT `id`, `convention_name` FROM `tai_bc_status_conventions` WHERE `used_for` = 'Payment_Status' 
                //                 AND `convention_name` != 'Hold' AND `convention_name` != 'Hold till Audit' AND `convention_name` != 'Release' AND `is_active` = 1 ");
                $dropdown = DB::select("SELECT `id`, `convention_name` FROM `tai_bc_status_conventions` WHERE `used_for` = 'Payment_Status' 
                                AND `id` = 15 AND `is_active` = 1 ");
            }

            if ($access['Payment_Status_Release'] == true) {
                // status is release
                // $dropdown = DB::select("SELECT `id`, `convention_name` FROM `tai_bc_status_conventions` WHERE `used_for` = 'Payment_Status' 
                //                 AND `convention_name` != 'Hold' AND `convention_name` != 'Hold till Audit' AND `convention_name` != 'Paid' AND `is_active` = 1 ");
                $dropdown = DB::select("SELECT `id`, `convention_name` FROM `tai_bc_status_conventions` WHERE `used_for` = 'Payment_Status' 
                                AND `id` != '17' AND `is_active` = 1 ");
            }
        } else {
            return response()->json(['status' => 1, 'message' => "Found Data", 'data' => $data, 'access' => null, 'dropdown' => $dropdown]);
        }

        return response()->json(['status' => 1, 'message' => "Found Data", 'data' => $data, 'access' => $access, 'dropdown' => $dropdown]);
    }

    // update payment details
    public function updatePaymentDetails(Request $request)
    {
        $token_data = JWTAuth::parseToken()->authenticate(); //added by PUNAM 30-05-2023
        $tenantId = $token_data->tenant_id;

        // dd('hi');
        if ($request->id) {
            if ($request->amount_to_be_paid < 0) {
                return response()->json(['status' => 0, 'message' => "Please Enter Valid Amount", 'data' => null]);
            }

            $data = DB::table('tai_bc_payment_details')->where('id', $request->id)->where('is_active', 1)->first();

            if ($data) {
                if ($request->payment_status) {
                    $check_status = DB::table('tai_bc_status_conventions') // ->where('used_for','Payment_Status')
                        ->where('id', $data->payment_status)->where('is_active', 1)
                        ->first();
                    // return $check_status;
                    // get paid status
                    // if($check_status->convention_name == 'Paid')
                    // {
                    //     return response()->json(['status' => 0, 'message'=>"Can't change payment status", 'data' => null]);
                    // }else
                    // if($check_status->convention_name == 'Confirmation Pending')
                    // {
                    //     return response()->json(['status' => 0, 'message'=>"Can't change payment status", 'data' => null]);
                    // }
                    // else
                    if (!$request->payment_date) {
                        return response()->json(['status' => 0, 'message' => "Date Not Found", 'data' => null]);
                    } elseif (!$request->amount_to_be_paid) {
                        $query = DB::table('tai_bc_payment_details')->where('is_active', 1)->where('id', $request->id)->update(['payment_date' => $request->payment_date]);
                    } else {

                        // get hold and audit status
                        $check_s = DB::table('tai_bc_status_conventions')->where('id', $request->payment_status)->first();
                        if ($check_s->id) {
                            $result = $request->toArray();
                            $result = array(
                                'tai_bc_bill_checking_transactions_id' => $data->tai_bc_bill_checking_transactions_id,
                                'bill_type_id'    => $data->bill_type_id,
                                'vendor_id'       => $data->vendor_id,
                                'payment_date'    => $request->payment_date,
                                'payment_status'  => $request->payment_status,
                                'is_active'       => 1,
                                'remark'          => $request->remark,
                                'created_at'    => now(),
                                'updated_at'    => now(),
                                'created_by'    => $request->created_by,
                                'updated_by'    => $request->updated_by
                            );
                            // dd($result);

                            if ($request->amount_to_be_paid) {

                                $total_amt = $data->amount_to_be_paid;


                                if (is_numeric($total_amt)) {
                                    // dd($request->amount_to_be_paid, $total_amt, $request->amount_to_be_paid >= $total_amt);
                                    if ($request->amount_to_be_paid >= $total_amt) {
                                        // dd((int)$request->amount_to_be_paid , (int)$total_amt);
                                        if ((int)$request->amount_to_be_paid == (int)$total_amt) {
                                            // dd('tru');
                                            $result['amount_to_be_paid'] = $request->amount_to_be_paid;
                                            // dd($result);
                                            $query = DB::table('tai_bc_payment_details')->where('id', $request->id)->update($result);
                                            if ($query) {
                                                // create history
                                                $table = 'tai_bc_payment_details';
                                                $result = (array) DB::table('tai_bc_payment_details')->where('id', $request->id)->first();
                                                CommonController::generateHistory("UPDATE", $table, $request->id);

                                                return response()->json(['status' => 1, 'message' => "Data Updated Successfully", 'data' => $request->id]);
                                            } else {
                                                return response()->json(['status' => 0, 'message' => "Data Not Updated", 'data' => null]);
                                            }
                                        } else {
                                            return response()->json(['status' => 1, 'message' => "Amount Exceeded", 'data' => $request->id]);
                                        }
                                    } else {

                                        // dd($request->amount_to_be_paid ,$total_amt,'bye');
                                        $remaining_amount = $total_amt - $request->amount_to_be_paid;
                                        if ($remaining_amount) {
                                            // get old data of same id 
                                            $old_data = DB::table('tai_bc_payment_details')->where('id', $request->id)->first();

                                            $result['amount_to_be_paid'] = $request->amount_to_be_paid;
                                            $query = DB::table('tai_bc_payment_details')->where('id', $request->id)->update($result);

                                            // create history
                                            $table = 'tai_bc_payment_details';
                                            $result_data = (array) DB::table('tai_bc_payment_details')->where('id', $request->id)->first();

                                            CommonController::generateHistory("UPDATE", $table, $request->id);


                                            // dd($result);
                                            if ($query) {
                                                $insert_result = array(
                                                    'tai_bc_bill_checking_transactions_id' => $data->tai_bc_bill_checking_transactions_id,
                                                    'bill_type_id'    => $data->bill_type_id,
                                                    'vendor_id'       => $data->vendor_id,
                                                    'payment_date'    => $old_data->payment_date,
                                                    'payment_status'  => $old_data->payment_status,
                                                    'is_active'       => 1,
                                                    'remark'          => $old_data->remark,
                                                    'created_at'    => now(),
                                                    'updated_at'    => now(),
                                                    'created_by'    => $request->created_by,
                                                    'updated_by'    => $request->updated_by
                                                );

                                                $insert_result['amount_to_be_paid'] = $remaining_amount;
                                                // dd($insert_result);
                                                $insert_query = DB::table('tai_bc_payment_details')->insertGetId($insert_result);


                                                // create history
                                                $table = 'tai_bc_payment_details';
                                                $result = (array) DB::table('tai_bc_payment_details')->where('id', $insert_query)->first();
                                                CommonController::generateHistory("INSERT", $table, $insert_query);
                                            }
                                            return response()->json(['status' => 1, 'message' => "New Amount Added", 'data' => $request->id]);
                                        } else {
                                            return response()->json(['status' => 0, 'message' => "Invalid Amount", 'data' => null]);
                                        }
                                    }
                                } else {
                                    return response()->json(['status' => 0, 'message' => "Invalid Amount", 'data' => null]);
                                }
                            } else {
                                return response()->json(['status' => 0, 'message' => "Amount Not Found", 'data' => null]);
                            }
                        } else {
                            return response()->json(['status' => 0, 'message' => "Invalid Payment Status", 'data' => null]);
                        }
                    }
                }
            } else {
                return response()->json(['status' => 0, 'message' => "Data Not Found", 'data' => Null]);
            }
        }
    }

    // payment details History
    public function paymentHistory($id)
    {
        $token_data = JWTAuth::parseToken()->authenticate(); //added by PUNAM 30-05-2023
        $tenantId = $token_data->tenant_id;

        $result = DB::table('tai_bc_payment_details_history as pd')
            ->leftJoin('tai_bc_bill_type_master as btm', 'pd.bill_type_id', '=', 'btm.id')
            ->leftJoin('tai_bc_vendor_master as vm', 'pd.vendor_id', '=', 'vm.id')
            ->leftJoin('tai_bc_status_conventions as sc', 'pd.payment_status', '=', 'sc.id')
            ->where('tai_bc_bill_checking_transactions_id', $id)
            ->leftJoin('tai_employee_master as emp', 'emp.id', '=', 'b.updated_by')
            ->leftJoin('tai_employee_master as em', 'em.id', '=', 'b.created_by')
            ->where('pd.is_active', 1)
            ->select(
                'pd.*',
                'btm.bill_type as bill_type_name',
                'vm.vendor_name',
                'sc.convention_name as payment_status_name',
                DB::raw("CONCAT(em.first_name,' ',em.last_name) AS created_by"),
                DB::raw("CONCAT(emp.first_name,' ',emp.last_name) AS updated_by")
            )
            ->get();

        return response()->json(['status' => 1, 'message' => 'Found Data', 'data' => $result]);
    }

    // Authoruity for update Bill Checking transaction_data
    public function checkAuthorityAccess($auth_id = null, $module_name, $auth_name = null)
    {
        $query = DB::table('tai_module_setting')->where('setting_name', $auth_name)
            ->where('field_value', 'Y')
            ->where('setting_type', 'U')
            ->where('module_name', $module_name)
            ->where('is_active', '1')->first();
        if ($query) {
            $check = DB::table('tai_module_setting_details')->where('setting_id', $query->id)
                ->where('user_id', $auth_id)
                ->where('is_active', '1')
                ->where('setting_value', 'Y')->first();
            if ($check) {
                return true;
            } else {
                return false;
            }
        } else {
            return false;
        }
        return false;
    }


    // Filter data
    public function filterBillCheckingData(Request $request)
    {

        $token_data = JWTAuth::parseToken()->authenticate(); //added by PUNAM 30-05-2023
        $tenantId = $token_data->tenant_id;

        $bc_id = $request->id;
        $bill_no = $request->bill_no;
        $bill_type = $request->bill_type;
        $vendor_name = $request->vendor_name;
        $payment_details_status = $request->payment_details_status;
        $bill_status = $request->bill_status;
        $assign_to = $request->assign_to;
        $bill_date = $request->bill_date;
        $received_date = $request->received_date;
        $payment_date = $request->payment_date;
        $created_by = $request->created_by;
        // dd($request->all());

        $query = DB::table('tai_bc_bill_checking_transactions as b')
            ->leftJoin('tai_employee_master as e', 'e.id', 'b.assign_to')
            ->leftJoin('tai_department_master as d', 'd.id', 'b.department_name')
            ->leftJoin('tai_bc_bill_type_master as bill', 'bill.id', 'b.bill_type')
            ->leftJoin('tai_bc_vendor_master as v', 'v.id', 'b.vendor_name')
            ->select(
                'b.*',
                DB::raw("CONCAT(e.first_name,' ',e.last_name) AS assign_to_name"),
                'd.department as department_name',
                'v.vendor_name as vendor_id_name',
                'bill.bill_type as bill_type_name',
                'b.created_by as creator',
                DB::raw("CONCAT(em.first_name,' ',em.last_name) AS created_by"),
                DB::raw("CONCAT(emp.first_name,' ',emp.last_name) AS updated_by")
            )
            ->leftJoin('tai_employee_master as emp', 'emp.id', '=', 'b.updated_by')
            ->leftJoin('tai_employee_master as em', 'em.id', '=', 'b.created_by');

        if ($bc_id) {
            $query = $query->where('b.bc_id', $bc_id);
        }
        if ($bill_no) {
            $query = $query->where('b.bill_no', $bill_no);
        }
        if ($bill_type) {
            $query = $query->where('b.bill_type', $bill_type);
        }
        if ($vendor_name) {
            $query = $query->where('b.vendor_name', $vendor_name);
        }
        if ($bill_status) {
            $query = $query->where('b.is_active', $bill_status);
        }
        if ($assign_to) {
            $query = $query->where('b.assign_to', $assign_to);
        }
        if ($bill_date) {
            $query = $query->where('b.bill_date', $bill_date);
        }
        if ($received_date) {
            $query = $query->where('b.received_date', $received_date);
        }
        if ($created_by) {
            $query = $query->where('b.created_by', $created_by);
        }

        if ($payment_details_status) {
            $query = $query->leftJoin('tai_bc_payment_details as p', 'b.id', 'p.tai_bc_bill_checking_transactions_id')
                ->where('p.payment_status', $payment_details_status);
            // ->where('b.received_date',$received_date);
        }

        $query = $query->get();

        foreach ($query as $r) {
            // bill status from payment details
            if ($r->is_active == 0) {
                $r->payment_status = 'Cancelled';
            } else {
                $payment_status = DB::table('tai_bc_payment_details')->where('tai_bc_bill_checking_transactions_id', $r->id)->where('payment_status', '!=', 15)->get();
                if (count($payment_status) >= 0) {
                    $r->payment_status = 'Unsolved';
                } else {
                    $r->payment_status = 'Solved';
                }
            }
        }
        return $query;
        if (count($query) > 0) {
            return response()->json(['status' => 1, 'message' => 'Found Data', 'data' => $query]);
        } else {
            return response()->json(['status' => 0, 'message' => 'Data Not Found !', 'data' => $query]);
        }
    }

    // delete bill checking attachment
    public function deleteAttachment($id)
    {
        $query = DB::table('tai_attachments')->where('id', $id)->update(['is_active' => 0, 'updated_at' => now()]);
        if ($query) {
            return response()->json(['status' => 1, 'message' => 'Your file is deleted']);
        } else {
            return response()->json(['status' => 0, 'message' => 'Failed to delete']);
        }
    }

    public function getApprovalAuthority(Request $request)
    {
        $token_data = JWTAuth::parseToken()->authenticate(); //added by PUNAM 30-05-2023
        $tenantId = $token_data->tenant_id;

        $data = $request->toArray();
        // dd($data);
        foreach ($data['data'] as $d) {

            for ($i = 0; $i < count($d['employee_id']); $i++) {
                // dd($d);
                $is_required = in_array($d['employee_id'][$i], $d['is_required']) ? 1 : 0;
                //  $is_required_number = in_array($d['employee_id'][$i], $d['is_required_number']) ? count($d['is_required_number']) : 0;
                $table_insert = DB::table('tai_bc_bill_multi_approval_authority')->insertGetId([
                    'bil_type_id' => $d['bil_type_id'],
                    'employee_id' => $d['employee_id'][$i],
                    'amount' =>  $d['amount'],
                    'is_required' => $is_required,
                    'is_required_number' => $d['is_required_number'],
                    'bil_approval_level' => $d['bil_approval_level']
                ]);
            }
        }
        return response()->json(['status' => 1, 'message' => 'Data Insert Successfully', 'data' => null]);
    }
}
