<?php
// live
namespace App\Http\Controllers;

use Illuminate\Http\Request;
use DB;

use JWTAuth;
use Tymon\JWTAuth\Exceptions\JWTException;
use Symfony\Component\HttpFoundation\Response;
use Illuminate\Support\Facades\Validator;

class CityController extends Controller
{
    // test with token
    // //added by PUNAM 15-05-2023
    public function getAllCity(Request $request, $type = null, $sorted = null)
    {
        $token_data = JWTAuth::parseToken()->authenticate();
        $result = DB::table('tai_city_master as ci')
            ->leftJoin('tai_state_master as s', 's.id', '=', 'ci.state_id')
            ->leftJoin('tai_country_master as c', 'c.id', '=', 'ci.country_id')
            ->leftJoin('tai_employee_master as e', 'e.id', '=', 'ci.updated_by')
            ->leftJoin('tai_employee_master as ec', 'ec.id', '=', 'ci.created_by')
            ->select('ci.*', 'c.country', 's.state', DB::raw("CONCAT(e.first_name,' ',e.last_name) AS updated_by"), DB::raw("CONCAT(ec.first_name,' ',ec.last_name) AS created_by"))
            ->where('ci.tenant_id', $token_data->tenant_id);
            if ($sorted != null) {
                $result = $result->orderby('ci.city')
                    ->orderByDesc('ci.is_active')
                    ->orderBy('ci.id', 'DESC')->get();
            } else {
                $result = $result->orderByDesc('ci.is_active')
                    ->orderBy('ci.id', 'DESC');
                if ($type != null) {
                    $result = $result->where('ci.is_active', 1)->get();
                } else {
                    $result = $result->get();
                }
            }
        if (count($result) > 0) {
            return response()->json(['status' => 1, 'message' => "Record found !!!", 'data' => $result]);
        } else {
            return response()->json(['status' => 0, 'message' => "No Record found !!!", 'data' => null]);
        }
    }

    public function createCity(Request $request)
    {
        $token_data = JWTAuth::parseToken()->authenticate(); //added by PUNAM 15-05-2023

        if (!$request->country_id && !$request->state_id && !$request->city) {
            return response()->json(['status' => 0, 'message' => "Required Country and State !!!", 'data' => null]);
        }

        $check = DB::table('tai_city_master')
            ->where('country_id', $request->country_id)
            ->where('state_id', $request->state_id)
            ->where('city', $request->city)
            ->first();

        if ($check) {
            return response()->json(['status' => 0, 'message' => "City Already Exists !!!", 'data' => null]);
        } else {
            $data = $request->toArray();
            $data['tenant_id'] = $token_data->tenant_id; //added by PUNAM 15-05-2023
            $data['created_by'] = $token_data->id; //added by PUNAM 15-05-2023

            $result = DB::table('tai_city_master')->insertGetId($data);
            if ($result > 0) {
                return response()->json(['status' => 1, 'message' => "City Inserted !!!", 'data' => null]);
            } else {
                return response()->json(['status' => 0, 'message' => "City Not Inserted !!!", 'data' => null]);
            }
        }
    }

    // update city data
    public function updateCity(Request $request, $id)
    {
        $token_data = JWTAuth::parseToken()->authenticate(); //added by PUNAM 15-05-2023

        $data = $request->toArray();
        $data['tenant_id'] = $token_data->tenant_id; //added by PUNAM 15-05-2023
        $data['updated_by'] = $token_data->id; //added by PUNAM 15-05-2023

        $check = DB::table('tai_city_master')
            ->where('city', $data['city'])
            ->where('country_id', $data['country_id'])
            ->where('state_id', $data['state_id'])
            ->where('id', '!=', $id)
            ->where('tenant_id', $token_data->tenant_id)
            ->first();

        if ($check) {
            return response()->json(['status' => 0, 'message' => "City Already Exists !!!", 'data' => null]);
        }
        $result = DB::table('tai_city_master')->where('id', $id)->where('tenant_id', $token_data->tenant_id)->update($data);
        if ($result) {
            return response()->json(['status' => 1, 'message' => "Record Updated !!!", 'data' => null]);
        } else {
            return response()->json(['status' => 0, 'message' => "Record Not Updated !!!", 'data' => null]);
        }
    }


    // delete city data
    public function deleteCity($id)
    {
        $token_data = JWTAuth::parseToken()->authenticate(); //added by PUNAM 15-05-2023

        $result = DB::table('tai_city_master')->where('id', $id)
            ->where('tenant_id', $token_data->tenant_id)
            ->update(['is_active' => 0, 'updated_by' => $token_data->id, 'updated_at' => date('Y-m-d H:i:s')]);

        if ($result) {
            return response()->json(['status' => 1, 'message' => "Record Deleted !!!", 'data' => null]);
        } else {
            return response()->json(['status' => 0, 'message' => "Record Not Deleted !!!", 'data' => null]);
        }
    }

    public function getCityById($id)
    {
        $token_data = JWTAuth::parseToken()->authenticate(); //added by PUNAM 15-05-2023

        $result = DB::table('tai_city_master')->where('id', $id)->where('tenant_id', $token_data->tenant_id)->first();
        if ($result) {
            return response()->json(['status' => 1, 'message' => "Record Deleted !!!", 'data' => $result]);
        } else {
            return response()->json(['status' => 0, 'message' => "No Record Deleted !!!", 'data' => null]);
        }
    }
}
