<?php
// live
namespace App\Http\Controllers;

use Illuminate\Http\Request;
use DB;
use JWTAuth;
use Tymon\JWTAuth\Exceptions\JWTException;

class CountryController extends Controller
{
    public function getAllCountry(Request $request, $type = null, $sorted = null)
    {
        $token_data = JWTAuth::parseToken()->authenticate(); //added by PUNAM 18-05-2023

        $result = DB::table('tai_country_master as c')
            ->leftJoin('tai_employee_master as e', 'e.id', '=', 'c.updated_by')
            ->leftJoin('tai_employee_master as ec', 'ec.id', '=', 'c.created_by')
            ->select('c.*', DB::raw("CONCAT(e.first_name,' ',e.last_name) AS updated_by"), DB::raw("CONCAT(ec.first_name,' ',ec.last_name) AS created_by"))
            // ->where('c.tenant_id', $tenantId);
            ->where('c.tenant_id', $token_data->tenant_id);
            
        if ($sorted != null) {
            $result = $result->orderby('c.country')
                ->orderByDesc('c.is_active')
                ->orderBy('c.id', 'DESC')->get();
        } else {
            $result = $result->orderByDesc('c.is_active')
                ->orderBy('c.id', 'DESC');
            if ($type != null) {
                $result = $result->where('c.is_active', 1)->get();
            } else {
                $result = $result->get();
            }
        }

        if (count($result) > 0) {
            return response()->json(['status' => 1, 'message' => "Record found !!!", 'data' => $result]);
        } else {
            return response()->json(['status' => 0, 'message' => "No Record found !!!", 'data' => null]);
        }
    }

    public function createCountry(Request $request)
    {
        $token_data = JWTAuth::parseToken()->authenticate(); //added by PUNAM 18-05-2023

        if (!$request->country) {
            return response()->json(['status' => 0, 'message' => "Required Country !!!", 'data' => null]);
        }

        $check = DB::table('tai_country_master')->where('country', $request->country)->first();
        if ($check) {
            return response()->json(['status' => 0, 'message' => "Country Already Exists !!!", 'data' => null]);
        }

        $data = $request->toArray();
        $data['tenant_id'] = $token_data->tenant_id; //added by PUNAM 18-05-2023

        $result = DB::table('tai_country_master')->insert($data);
        if ($result > 0) {
            return response()->json(['status' => 1, 'message' => "Record Inserted !!!", 'data' => null]);
        } else {
            return response()->json(['status' => 0, 'message' => "Record Not Inserted !!!", 'data' => null]);
        }
    }


    public function updatedCountry(Request $request, $id)
    {
        $token_data = JWTAuth::parseToken()->authenticate(); //added by PUNAM 18-05-2023

        $data = $request->toArray();

        $check = DB::table('tai_country_master')->where('tenant_id', $token_data->tenant_id)->where('country', $request->country)
            ->first();
        if ($check && $check->id != $id) {
            return response()->json(['status' => 0, 'message' => "Record Already Exists !!!", 'data' => null]);
        }

        $prevCheck = DB::table('tai_country_master')->where('tenant_id', $token_data->tenant_id)->where('id', $id)->first();

        $result = DB::table('tai_country_master')->where('id', $id)->update($data);
        if ($result) {

            if ($prevCheck->is_active != $data['is_active']) {
                DB::table('tai_state_master')->where('country_id', $id)->update([
                    'is_active' => $data['is_active'],
                    'updated_by' => $data['updated_by'],
                    'updated_at' => $data['updated_at']
                ]);

                DB::table('tai_city_master')->where('country_id', $id)->update([
                    'is_active' => $data['is_active'],
                    'updated_by' => $data['updated_by'],
                    'updated_at' => $data['updated_at']
                ]);
            }

            return response()->json(['status' => 1, 'message' => "Record Updated !!!", 'data' => null]);
        } else {
            return response()->json(['status' => 0, 'message' => "Record Not Updated !!!", 'data' => null]);
        }
    }

    public function deleteCountry($id)
    {
        $token_data = JWTAuth::parseToken()->authenticate(); //added by PUNAM 18-05-2023

        $result = DB::table('tai_country_master')->where('tenant_id', $token_data->tenant_id)->where('id', $id)->update(['is_active' => 0]);
        if ($result) {
            return response()->json(['status' => 1, 'message' => "Record Deleted !!!", 'data' => null]);
        } else {
            return response()->json(['status' => 0, 'message' => "Record Not Deleted !!!", 'data' => null]);
        }
    }

    public function getCountryById($id)
    {
        $token_data = JWTAuth::parseToken()->authenticate(); //added by PUNAM 18-05-2023

        $result = DB::table('tai_country_master')->where('tenant_id', $token_data->tenant_id)->where('id', $id)->first();
        if ($result) {
            return response()->json(['status' => 1, 'message' => "Record Found !!!", 'data' => $result]);
        } else {
            return response()->json(['status' => 0, 'message' => "No Record Found !!!", 'data' => null]);
        }
    }
}
