<?php

namespace App\Http\Controllers;

use App\Models\Tenant;
use Illuminate\Http\Request;
use DB;
use JWTAuth;

class CustomerMappingController extends Controller
{

	public function getAllCustomerMapping(Request $request)
	{
		$token_data = JWTAuth::parseToken()->authenticate(); //Date:30-05-2023 PUNAM SHINDE
        $tenantId = $token_data->tenant_id; //Date:30-05-2023 PUNAM SHINDE

		$data = DB::table('tai_customer_mapping as c')
			->leftJoin('tai_customer_master as cm', 'cm.id', 'c.customer_id')
			->leftJoin('tai_customer_type_master as ctm', 'ctm.id', 'c.customer_type_id')
			->leftJoin('tai_query_type_master as q', 'q.id', 'c.query_type_id')
			->leftJoin('tai_template_master as t', 't.id', 'c.template_id')
			->leftJoin('tai_dynamic_form as dy', 'dy.id', 'c.dynamic_form_id')
			->leftJoin('tai_project_master as p', 'p.id', 'c.project_id')
			->leftJoin('tai_project_modules as pm', 'pm.id', 'c.module_id')
			->leftJoin('tai_project_sub_modules as psm', 'psm.id', 'c.submodule_id')
			->leftJoin('tai_department_master as d', 'd.id', 'c.department_id')
			->leftJoin('tai_employee_master as e', 'e.id', '=', 'c.updated_by')
			->leftJoin('tai_employee_master as em', 'em.id', '=', 'c.created_by')
			->where('c.tenant_id', $tenantId)
			->select(
				'c.*',
				'cm.name as customer_name',
				'ctm.type_name as customer_type_name',
				'q.query_type_name',
				't.template_name',
				'dy.template_name as dynamic_form_name',
				'p.project_name',
				'pm.module_name',
				'psm.sub_module_name',
				'd.department as department_name',
				DB::RAW('(CASE WHEN c.approach = "HLT" THEN "Having Less Ticket" 
													WHEN c.approach = "SP" THEN  "Single Person"
													WHEN c.approach = "RW" THEN  "Ratio Wise"
													WHEN c.approach = "RR" THEN  "Deparment Round Robin"
													ELSE c.approach END) as approach'),
				DB::raw("CONCAT(e.first_name,' ',e.last_name) AS updated_by"),
				DB::raw("CONCAT(em.first_name,' ',em.last_name) AS created_by")
			)
			->orderBy('c.id', 'DESC')
			->orderBy('c.is_active', 'DESC')
			->get();
		if ($data) {
			return response()->json(['status' => 1, 'message' => "Record Found", 'data' => $data]);
		} else {
			return response()->json(['status' => 0, 'message' => "Record Not Found", 'data' => NULL]);
		}
	}

	public function createCustomerMapping(Request $request)
	{
		$token_data = JWTAuth::parseToken()->authenticate(); //Date:30-05-2023 PUNAM SHINDE
        $tenantId = $token_data->tenant_id; //Date:30-05-2023 PUNAM SHINDE

		$data = $request->toArray();
		$data['tenant_id'] = $tenantId;

		if (!$request->tenant_id) {
			return  response()->json(['status' => 0, 'message' => "Unable to find tenant id", 'data' => NULL]);
		}

		if (!$request->customer_type_id) {
			return  response()->json(['status' => 0, 'message' => "Required Customer Type", 'data' => NULL]);
		}
		if (!$request->query_type_id) {
			return  response()->json(['status' => 0, 'message' => "Required Query Type", 'data' => NULL]);
		}
		$check = DB::table('tai_customer_mapping')->where('customer_type_id', $request->customer_type_id)
			->where('query_type_id', $request->query_type_id)
			->where('tenant_id', $request->tenant_id)
			->first();

		if ($check) {
			return  response()->json(['status' => 0, 'message' => "Selected Query Type Is Already Exists !!!", 'data' => NULL]);
		}

		if (!$request->approach) {
			return  response()->json(['status' => 0, 'message' => "Required Approach", 'data' => NULL]);
		}

		if (!$request->department_id) {
			return  response()->json(['status' => 0, 'message' => "Required Department", 'data' => NULL]);
		}

		if ($request->approach == 'RW')  //RATIO WISE
		{
			if (array_sum($data["ratio"]) > 100) {
				return  response()->json(['status' => 0, 'message' => "Ratio Must be less than or equal to 100", 'data' => NULL]);
			}
			$policy = [];
			for ($i = 0; $i < count($data['user_id']); $i++) {
				if ($data['ratio'][$i]) {
					if ($data['ratio'][$i] > 0) {
						$policy[] = $data['user_id'][$i] . ':' . $data['ratio'][$i];
					}
				}
			}
			$data['user_policy'] = implode(',', $policy);
		} else if ($request->approach == 'RR')  //ROUND ROBIN
		{
			$data['user_policy'] = implode(',', $data['user_id']);
		} else if ($request->approach == 'HLT') //HAVING LESS TICKET
		{
			$data['user_policy'] = (is_array($data['user_id'])) ? implode(',', $data['user_id']) : $data['user_id'];
		} else if ($request->approach == 'SP') //SINGLE PERSON
		{
			$data['user_policy'] = $data['user_id'][0];
		} else {
			return  response()->json(['status' => 0, 'message' => "Required Approach", 'data' => NULL]);
		}

		unset($data['user']);
		unset($data['user_id[]']);
		unset($data['ratio[]']);

		$temp = [];
		$temp['tenant_id'] = $request->tenant_id ? $request->tenant_id : 1;
		$temp['customer_type_id'] = $request->customer_type_id;
		$temp['query_type_id'] = $request->query_type_id;
		$temp['template_id'] = $request->template_id;
		$temp['dynamic_form_id'] = $request->dynamic_form_id;
		$temp['department_id'] = $request->department_id;
		$temp['approach'] = $request->approach;
		$temp['user_policy'] = $data['user_policy'];
		$temp['priority'] = $request->priority;
		$temp['is_active'] = 1;
		$temp['confirmation_required'] = $request->confirmation_required;
		$temp['created_at'] = $request->created_at;
		$temp['created_by'] = $request->created_by;

		$insert = DB::table('tai_customer_mapping')->insertGetId($temp);
		if ($insert) {
			$this->userPolicy($insert, $request->toArray(), "INSERTION");
			return  response()->json(['status' => 1, 'message' => "Record Inserted", 'data' => NULL]);
		} else {
			return  response()->json(['status' => 0, 'message' => "Record Not Inserted", 'data' => NULL]);
		}
	}

	public function insertAll()
	{
		$data = DB::table('tai_customer_mapping')->where('is_active', 1)->get();
		foreach ($data as $d) {
			$this->userPolicy($d->id, (array)$d, "INSERTION");
		}
	}

	// insert Customer mapping users
	public function userPolicy($id, $data, $operationType)
	{
		if ($id && $data) {
			//ADDED BY PARTH
			//MAKE PREVIOUS DATA AS 0 AND ADD NEW ENTRIES
			// if ($data['approach'] == 'RW') {
			// 	$RW = explode(',', $data['user_policy']);
			// 	foreach ($RW as $d) {
			// 		$array = explode(':', $d);
			// 		$data['user_id'][] = $array[0];
			// 		$data['ratio'][] = $array[1];
			// 	}
			// } else {
			// 	$data['user_id'] = explode(',', $data['user_policy']);
			// }

			if ($operationType == "UPDATION") {
				DB::table('tai_customer_user_mapping')->where('tai_customer_mapping_id', $id)->where('is_active', 1)->update(['is_active' => 0, 'updated_at' => now(), 'updated_by' => $data['updated_by']]);
				$data['created_by'] = $data['updated_by'];
			}

			if ($data['approach'] == 'RW')  //Ration Wise
			{
				for ($i = 0; $i < count($data['user_id']); $i++) {
					if ($data['ratio'][$i]) {
						if ($data['ratio'][$i] > 0) {
							$res = DB::table('tai_customer_user_mapping')->insert([
								'tai_customer_mapping_id' => $id,
								'user_id' => $data['user_id'][$i],
								'percentage' => $data['ratio'][$i],   //Added by parth
								'created_at' => now(),
								'created_by' => $data['created_by']
							]);
						}
					}
				}
			} else if ($data['approach'] == 'RR') // Round Robin
			{
				$orderBy = 1;
				foreach ($data['user_id'] as $key => $val) {
					$res = DB::table('tai_customer_user_mapping')->insert([
						'tai_customer_mapping_id' => $id,
						'user_id' => $val,
						'order_by' => $orderBy,		//Added by parth															
						'created_at' => now(),
						'created_by' => $data['created_by']
					]);
					$orderBy++;
				}
			} else if ($data['approach'] == 'HLT') // Having less ticket
			{
				foreach ($data['user_id'] as $key => $val) {
					$res = DB::table('tai_customer_user_mapping')->insert([
						'tai_customer_mapping_id' => $id,
						'user_id' => $val,
						'created_at' => now(),
						'created_by' => $data['created_by']
					]);
				}
			} else if ($data['approach'] == 'SP') // Single Person
			{
				$res = DB::table('tai_customer_user_mapping')->insert([
					'tai_customer_mapping_id' => $id,
					'user_id' => $data['user_id'][0],
					'created_at' => now(),
					'created_by' => $data['created_by']
				]);
			}
		}
	}

	public function updateCustomerMapping(Request $request, $id)
	{
		$token_data = JWTAuth::parseToken()->authenticate(); //Date:30-05-2023 PUNAM SHINDE
        $tenantId = $token_data->tenant_id; //Date:30-05-2023 PUNAM SHINDE

		$data = $request->toArray();
		$data['tenant_id'] = $tenantId; //Date:30-05-2023

		if (!$data['tenant_id']) {
			return  response()->json(['status' => 0, 'message' => "Unable to find tenant id", 'data' => NULL]);
		}

		if (!$request->customer_type_id) {
			return  response()->json(['status' => 0, 'message' => "Required Customer Type", 'data' => NULL]);
		}
		if (!$request->query_type_id) {
			return  response()->json(['status' => 0, 'message' => "Required Query Type", 'data' => NULL]);
		}
		if (!$request->approach) {
			return  response()->json(['status' => 0, 'message' => "Required Approach", 'data' => NULL]);
		}
		if (!$request->department_id) {
			return  response()->json(['status' => 0, 'message' => "Required Department", 'data' => NULL]);
		}

		if ($request->approach == 'RW')  //RATIO WISE
		{
			if (array_sum($data["ratio"]) > 100) {
				return  response()->json(['status' => 0, 'message' => "Ratio Must be less than or equal to 100", 'data' => NULL]);
			}
			$policy = [];
			for ($i = 0; $i < count($data['user_id']); $i++) {
				$policy[] = $data['user_id'][$i] . ':' . $data['ratio'][$i];
			}
			$data['user_policy'] = implode(',', $policy);
		} else if ($request->approach == 'RR')  //ROUND ROBIN
		{
			$data['user_policy'] = implode(',', $data['user_id']);
		} else if ($request->approach == 'HLT') //HAVING LESS TICKET
		{
			$data['user_policy'] = (is_array($data['user_id'])) ? implode(',', $data['user_id']) : $data['user_id'];
		} else if ($request->approach == 'SP') //SINGLE PERSON
		{
			$data['user_policy'] = (is_array($data['user_id'])) ? implode(',', $data['user_id']) : $data['user_id'];
		} else {
			return  response()->json(['status' => 0, 'message' => "Required Approach", 'data' => NULL]);
		}

		unset($data['user']);
		unset($data['user_id']);
		unset($data['ratio']);

		$temp['tenant_id'] = $request->tenant_id;
		$temp['customer_type_id'] = $request->customer_type_id;
		$temp['query_type_id'] = $request->query_type_id;
		$temp['template_id'] = $request->template_id;
		$temp['dynamic_form_id'] = $request->dynamic_form_id;
		$temp['department_id'] = $request->department_id;
		$temp['approach'] = $request->approach;
		$temp['user_policy'] = $data['user_policy'];
		$temp['priority'] = $request->priority;
		$temp['is_active'] = $request->is_active;
		$temp['confirmation_required'] = $request->confirmation_required;
		// $temp['created_at']=$request->created_at;
		// $temp['created_by']=$request->created_by;
		$temp['updated_at'] = now();
		$temp['updated_by'] = $request->updated_by;

		$update = DB::table('tai_customer_mapping')->where('id', $id)->update($temp);

		if ($update > 0) {

			$this->userPolicy($id, $request->toArray(), "UPDATION");

			return  response()->json(['status' => 1, 'message' => "Record Updated", 'data' => NULL]);
		} else {
			return  response()->json(['status' => 0, 'message' => "Record Not Updated", 'data' => NULL]);
		}
	}


	public function getQueryTypeCustomerMapping(Request $request, $user_id, $query_type_id = NULL)
	{
		$token_data = JWTAuth::parseToken()->authenticate(); //Date:30-05-2023 PUNAM SHINDE
        $tenant_id = $token_data->tenant_id; //Date:30-05-2023 PUNAM SHINDE

		$tenant = DB::table('tai_tenant_master')->where('id', $tenant_id)->first();

		$user = DB::table('tai_employee_master as e')
			->leftJoin('tai_customer_master as c', 'c.id', 'e.customer_id')
			->where('e.id', $user_id)
			->select('e.*', 'c.customer_type_id')
			->first();
		
		$result = "";

		$query = DB::table('tai_customer_mapping')->where('tenant_id', $tenant_id);

		if ($query_type_id) {
			$query->where('query_type_id', $query_type_id);
		} else {
			// DATE:10-08-2023 PUNAM SHINDE
			// if ($user->account_for == "CUSTOMER") {
			// 	$query->where('customer_type_id', $user->customer_type_id);
			// } else {
				$query->where('is_active', 1);
			// }
		}
		$result = $query->get();

		if (count($result)) {
			foreach ($result as $r) {

				if (!$r->dynamic_form_id) {
					$queryForm = DB::table('tai_query_type_master')->where('id', $r->query_type_id)->first();
					if ($queryForm && $queryForm->form_id) {
						$r->dynamic_form_id = $queryForm->form_id;
					}
				}
				if ($r->dynamic_form_id) {
					$r->dynamic_form = json_decode(DB::table('tai_dynamic_form')->where('id', $r->dynamic_form_id)->first()->data);
				}
			}
		}




		// if($user->account_for=="CUSTOMER"){
		// 	$query=DB::table('tai_customer_mapping');

		// 		if($query_type_id){		
		// 			$query->where('query_type_id',$query_type_id);
		// 		}else{
		// 			$query->where('is_default',1);
		// 		}

		// 		$result=$query->where('is_active',1)->first();
		// }else{

		// }

		// $assign_to_user_id="";

		// if($result)
		// {
		// 	$data['query_type_id']=$result->query_type_id;  

		// 	if($result->dynamic_form_id)
		// 	{
		// 		$data['dynamic_form_id']=$result->dynamic_form_id;
		// 	}else{
		// 		$queryType=DB::table('tai_query_type_master as q')
		// 						->leftJoin('tai_dynamic_form as d','d.id','=','q.form_id')
		// 						->where('q.id',$result->query_type_id)
		// 						->where('d.is_active',1)
		// 						->first();
		// 		$data['dynamic_form_id']=$queryType->form_id;
		// 	}
		// 	$data['mapping_id']=$result->id;
		// 	$data['confirmation_required']=$result->confirmation_required;

		// }else{
		// 	$queryType=DB::table('tai_query_type_master as q')
		// 							->leftJoin('tai_dynamic_form as d','d.id','=','q.form_id')
		// 							->where('q.id',$query_type_id)
		// 							->where('d.is_active',1)
		// 							->first();
		// 	if($queryType){						
		// 		$data['query_type_id']=$queryType->id;  
		// 		$data['dynamic_form_id']=$queryType->form_id;
		// 		$data['mapping_id']=null;
		// 		$data['confirmation_required']="";
		// 	}
		// }


		if ($result) {
			return  response()->json(['status' => 1, 'message' => "Record Found", 'data' => $result]);
		} else {
			return  response()->json(['status' => 0, 'message' => "Record Not Found", 'data' => NULL]);
		}
	}

	public function getCustomerMappingById($id)
	{
		$token_data = JWTAuth::parseToken()->authenticate(); //Date:30-05-2023 PUNAM SHINDE
        $tenantId = $token_data->tenant_id; //Date:30-05-2023 PUNAM SHINDE

		$data = DB::table('tai_customer_mapping')->where('id', $id)->first();


		if ($data) {

			if ($data->approach == 'SP') {
				// $user_policy = DB::table('tai_customer_user_mapping')->where('tai_customer_mapping_id', $id)->where('is_active', 1)->first();
				// $data->user_policy = $user_policy->user_id;

				// updated by punam 11-04-2023
				$user_policy = DB::table('tai_customer_user_mapping as m')
					->join('tai_employee_master as e', 'e.id', '=', 'm.user_id')
					->where('m.tai_customer_mapping_id', $id)
					->where('m.is_active', 1)
					->where('e.is_active', 1)
					->select('m.user_id', DB::raw('concat(e.first_name, " ", e.last_name) as user_name'))
					->first();
				$data->user_policy = $user_policy;
			} else if ($data->approach == 'HLT') {
				// $user_policy = DB::table('tai_customer_user_mapping')->where('tai_customer_mapping_id', $id)->where('is_active', 1)->orderBy('id')->pluck('user_id');
				// $data->user_policy = implode(',', $user_policy);

				// updated by punam 11-04-2023
				$user_policy = DB::table('tai_customer_user_mapping as m')
					->leftJoin('tai_employee_master as e', 'e.id', '=', 'm.user_id')
					->where('m.tai_customer_mapping_id', $id)
					->where('m.is_active', 1)
					->where('e.is_active', 1)
					->orderBy('m.id')
					->select('m.user_id', DB::raw('concat(e.first_name, " ", e.last_name) as user_name'))
					->get();

				// dd($user_policy);
				$data->user_policy = $user_policy;
			} else if ($data->approach == 'RR') {
				// $user_policy = DB::table('tai_customer_user_mapping')->where('tai_customer_mapping_id', $id)->where('is_active', 1)->orderBy('order_by', 'ASC')->pluck('user_id');
				// $data->user_policy = implode(',', $user_policy);

				// updated by punam 11-04-2023
				$user_policy = DB::table('tai_customer_user_mapping as m')
					->join('tai_employee_master as e', 'e.id', '=', 'm.user_id')
					->where('m.tai_customer_mapping_id', $id)
					->where('m.is_active', 1)
					->where('e.is_active', 1)
					->orderBy('m.order_by', 'ASC')
					->select('m.user_id', DB::raw('concat(e.first_name, " ", e.last_name) as user_name'))
					->get();

				$data->user_policy = $user_policy;
				
			} else if ($data->approach == 'RW') {
				$user_policy = DB::table('tai_customer_user_mapping')->where('tai_customer_mapping_id', $id)->where('is_active', 1)->orderBy('order_by', 'ASC')
					->select('user_id', 'percentage')->get();

				$returnData = [];
				$returnData2 = [];

				foreach ($user_policy as $u) {
					$val = (int)$u->user_id . ":" . (int)$u->percentage;
					$returnData[] = $val;

					$emp = DB::table('tai_employee_master')->where('id', $u->user_id)->first();

					$returnData2[] = array(
						'user_id' => $u->user_id,
						"user_name" => $emp->first_name . " " . $emp->last_name . " (" . $u->user_id . ")",
						"ratio" => $u->percentage ? $u->percentage : 0
					);
				}
				$data->user_policy = $returnData;
				$data->user_policy2 = $returnData2;
			}

			// dd($data);


			// dd($data);

			// $policy = [];
			// // dd($data->user_policy);
			// foreach ($data->user_policy as $d) {

			// 	$val = explode(':', $d);
			// 	if (count($val) > 1) {
			// 		array_push($policy, (int)$val[0]);
			// 	} else {
			// 		array_push($policy, (int)$d);
			// 	}
			// }

			// if ($data->approach != "RW") {
			// 	$data->user_policy = $policy;
			// }

			// // dd($data);
			// $data->user_policy_label = DB::table('tai_employee_master')
			// 	->select('id as value', DB::raw('concat(first_name, " ", last_name) as label'))
			// 	->whereIn('id', $policy)->get();
			// $data->priority = explode(",", $data->priority);

			return response()->json(['status' => 1, 'message' => "Record Found", 'data' => $data]);
		} else {
			return  response()->json(['status' => 0, 'message' => "Record Not Found", 'data' => NULL]);
		}
	}

	// public $priority = ['Low', 'Medium', 'High', 'Very High'];

	// public function getCustomerMappingById(Request $request,$id)
	// {
	// 	$data = DB::table('tai_customer_mapping')->where('id',$id)->first();
	// 	if($data){
	// 		$data->user_policy = explode(",",$data->user_policy);
	// 		$user_priority_array = explode(",",$data->priority);
	// 		// dd($data);
	// 		$size = sizeof($user_priority_array);

	// 		$res = '';
	// 		for ($i = 0; $i < $size; $i++){
	// 			$temp = $user_priority_array[$i];

	// 			$temp = str_replace(" ", "", $temp);
	// 			if($i == ($size-1)){
	// 				$res .= $this->priority[$temp];
	// 			}else{
	// 				$res .= $this->priority[$temp] . ',';
	// 			}
	// 		}
	// 		$data->priority = ($res);
	// 		return response()->json(['status'=>1,'message'=>"Record Found",'data'=>$data]);
	// 	}else{
	// 		return  response()->json(['status'=>0,'message'=>"Record Not Found",'data'=>NULL]);
	// 	}
	// }

	public function getCustomerMappingDynamicForm(Request $request, $user_id, $query_type_id)
	{
	}

	public function getQuertyTypeCustomerMappingOld(Request $request, $user_id, $query_type_id = NULL)
	{
		$tenant = DB::table('tai_tenant_master')->where('id', $tenant_id)->first();

		$query = DB::table('tai_customer_mapping');
		if ($query_type_id) {
			$query->where('query_type_id', $query_type_id);
		} else {
			$query->where('is_default', 1);
		}
		$cust = $query->where('is_active', 1)->first();
		$assign_to_user_id = "";

		if ($cust) {
			//QUERY TYPE
			if ($cust->query_type_id) {
				$dynamicForm = DB::table('tai_query_type_master as q')
					->leftJoin('tai_dynamic_form as d', 'd.id', '=', 'q.form_id')
					->where('q.id', $cust->query_type_id)
					->where('d.is_active', 1)
					->first();
			} else if ($cust->dynamic_form_id) {
				$dynamicForm = DB::table('tai_dynamic_form as d')->where('d.id', $cust->dynamic_form_id)->first();
			}


			//SINGLE PESON
			if ($cust->approach == "SP") {
				$assign_to_user_id = $cust->user_policy;
			}
			//Round Robin
			if ($cust->approach == "RR") {
				$assigned_to = explode(',', $cust->user_policy);
				$last_entry = DB::table('tai_ticket_master')->whereIn('assign_to_user_id', $assigned_to)
					->orderBy('id', 'DESC')
					->limit(1)
					->first();
				$nextId = 0;
				foreach ($assigned_to  as $key => $value) {
					if ($last_entry->assign_to_user_id == $value) {
						$nextKey = $key + 1;
						if ($nextKey >= count($assigned_to)) {
							$nextKey = 0;
						}
						break;
					}
				}
				$assign_to_user_id = $assigned_to[$nextKey];
			}
			//Having Less Ticket
			if ($cust->approach == "HLT" && $cust->user_policy) {
				$query = "SELECT assign_to_user_id,count(id) as less_ticket FROM tai_ticket_master where assign_to_user_id in ($cust->user_policy) GROUP BY assign_to_user_id order by less_ticket limit 1";
				$lessAssignedUser = DB::SELECT(DB::RAW($query));
				$assign_to_user_id = $lessAssignedUser[0]->assign_to_user_id;
			}

			$data['query_type_id'] = $cust->query_type_id;
			$data['priority'] = $cust->priority;
			$data['assign_to_department_id'] = $cust->department_id;
			$data['template_id'] = $cust->template_id;
			$data['dynamic_form_id'] = $cust->dynamic_form_id;
			$data['project_id'] = $cust->project_id;
			$data['module_id'] = $cust->module_id;
			$data['submodule_id'] = $cust->submodule_id;
			$data['assign_to_user_id'] = $assign_to_user_id;

			return  response()->json(['status' => 1, 'message' => "Record Found", 'data' => $data]);
		} else {
			return  response()->json(['status' => 0, 'message' => "Record Not Found", 'data' => NULL]);
		}
	}

	public function customerMapping(Request $request)
	{
		//SP= SINGLE PERSON
		//RR= DEPT ROUND ROBIN
		//RWT= Ratio-wise Ticket
		//LT = Having Less Ticket

		$tenant = DB::table('tai_tenant_master')->where('id', $request->tenant_id)->first();

		$cust = DB::table('tai_customer_mapping')->where('type_id', $tenant->id)->where('is_active', 1)->first();

		if ($cust) {
			$priority = $cust->priority;

			//QUERY TYPE
			if ($cust->query_type_id) {
				$dynamicForm = DB::table('tai_query_type_master as q')
					->leftJoin('tai_dynamic_form as d', 'd.id', '=', 'q.form_id')
					->where('q.id', $cust->query_type_id)
					->where('d.is_active', 1)
					->first();
			}

			//SINGLE PESON
			if ($cust->approach == "SP") {
				$assign_to_user_id = $cust->user_policy;
			}

			//Round Robin
			if ($cust->approach == "RR") {
				$assigned_to = explode(',', $cust->user_policy);
				$last_entry = DB::table('tai_ticket_master')->whereIn('assign_to_user_id', $assigned_to)
					->orderBy('id', 'DESC')
					->limit(1)
					->first();
				$nextId = 0;
				foreach ($assigned_to  as $key => $value) {
					if ($last_entry->assign_to_user_id == $value) {
						$nextKey = $key + 1;
						if ($nextKey >= count($assigned_to)) {
							$nextKey = 0;
						}
						break;
					}
				}
				$assign_to_user_id = $assigned_to[$nextKey];
			}

			//Having Less Ticket

			if ($cust->approach == "HLT") {
				$query = "SELECT assign_to_user_id,count(id) as less_ticket FROM tai_ticket_master where assign_to_user_id in ($cust->user_policy) GROUP BY assign_to_user_id order by less_ticket limit 1";
				$lessAssignedUser = DB::SELECT(DB::RAW($query));

				$assign_to_user_id = $lessAssignedUser[0]->assign_to_user_id;
			}


			echo "Assign To dept $assign_to_department_id";
			echo "<br>";
			echo "Assign To user $assign_to_user_id";
		}
	}


	public function test(Request $request, $user_id = null)
	{
		$token_data = JWTAuth::parseToken()->authenticate(); //Date:30-05-2023 PUNAM SHINDE
        $tenantId = $token_data->tenant_id; //Date:30-05-2023 PUNAM SHINDE

		//SP= SINGLE PERSON
		//RR= DEPT ROUND ROBIN
		//RWT= Ratio-wise Ticket
		//LT = Having Less Ticket

		$tenantId = 1;
		$userId = 42;

		$employee = DB::table('tai_employee_master')->where('id', $userId)->first();
		// dd($employee);


		$tenant = DB::table('tai_tenant_master')->where('id', $request->tenant_id)->first();

		$cust = DB::table('tai_customer_mapping')->where('type_id', $tenant->id)->where('is_active', 1)->first();

		if ($cust) {
			$priority = $cust->priority;

			//QUERY TYPE
			if ($cust->query_type_id) {
				$dynamicForm = DB::table('tai_query_type_master as q')
					->leftJoin('tai_dynamic_form as d', 'd.id', '=', 'q.form_id')
					->where('q.id', $cust->query_type_id)
					->where('d.is_active', 1)
					->first();
			}

			//SINGLE PESON
			if ($cust->approach == "SP") {
				$assign_to_user_id = $cust->user_policy;
			}

			//Round Robin
			if ($cust->approach == "RR") {
				$assigned_to = explode(',', $cust->user_policy);
				$last_entry = DB::table('tai_ticket_master')->whereIn('assign_to_user_id', $assigned_to)
					->orderBy('id', 'DESC')
					->limit(1)
					->first();
				$nextId = 0;
				foreach ($assigned_to  as $key => $value) {
					if ($last_entry->assign_to_user_id == $value) {
						$nextKey = $key + 1;
						if ($nextKey >= count($assigned_to)) {
							$nextKey = 0;
						}
						break;
					}
				}
				$assign_to_user_id = $assigned_to[$nextKey];
			}

			//Having Less Ticket

			if ($cust->approach == "HLT") {
				$query = "SELECT assign_to_user_id,count(id) as less_ticket FROM tai_ticket_master where assign_to_user_id in ($cust->user_policy) GROUP BY assign_to_user_id order by less_ticket limit 1";
				$lessAssignedUser = DB::SELECT(DB::RAW($query));

				$assign_to_user_id = $lessAssignedUser[0]->assign_to_user_id;
			}


			echo "Assign To dept $assign_to_department_id";
			echo "<br>";
			echo "Assign To user $assign_to_user_id";
		}
	}

	public function getPriorityDropdown()
	{
		$token_data = JWTAuth::parseToken()->authenticate(); //Date:30-05-2023 PUNAM SHINDE
        $tenantId = $token_data->tenant_id; //Date:30-05-2023 PUNAM SHINDE

		$data = array(array('label' => 'Low', 'id' => 0), array('label' => 'Medium', 'id' => 1), array('label' => 'High', 'id' => 2), array('label' => 'Very High', 'id' => 3));
		return  response()->json(['status' => 1, 'message' => "Record Found", 'data' => $data]);
	}
}
