<?php

namespace App\Http\Controllers;

use App\Models\Department;
use Illuminate\Http\Request;
use DB;
use JWTAuth;

class DepartmentMappingController extends Controller
{
    public function getAllDepartmentMapping(Request $request)
    {
        $token_data = JWTAuth::parseToken()->authenticate(); //Date:30-05-2023 PUNAM SHINDE
        $tenantId = $token_data->tenant_id; //Date:30-05-2023 PUNAM SHINDE

        $result = DB::table('tai_employee_department_mapping as ed')
            ->leftJoin('tai_employee_master as eu', 'eu.id', '=', 'ed.updated_by')
            ->leftJoin('tai_employee_master as e', 'e.id', '=', 'ed.employee_id')
            ->leftJoin('tai_department_master as d', 'd.id', '=', 'ed.department_id')
            ->select(
                'ed.*',
                DB::raw("CONCAT(eu.first_name,' ',eu.last_name) AS updated_by"),
                                     DB::raw("CONCAT(e.first_name,' ',e.last_name) AS employee_name"),
                                     'd.department'
                            )
                    // ->where('ed.is_active',1)     
                    ->orderBy('ed.is_active', 'DESC')  
            ->where('ed.tenant_id', $tenantId)
                    ->get();

        if (count($result) > 0) {
            return response()->json(['status' => 1, 'message' => "Record found !!!", 'data' => $result]);
        } else {
            return response()->json(['status' => 0, 'message' => "No Record found !!!", 'data' => null]);
        }        
    }
    
    public function createDepartmentMapping(Request $request)
    {
        $token_data = JWTAuth::parseToken()->authenticate(); //Date:30-05-2023 PUNAM SHINDE
        $tenantId = $token_data->tenant_id; //Date:30-05-2023 PUNAM SHINDE

        if ($request->employee_id && count($request->department_id)) {

            DB::table('tai_employee_department_mapping')->where([
                'tenant_id' => $request->tenant_id,
                'employee_id' => $request->employee_id
            ])
                ->update(['is_active' => 0, 'updated_at' => now(), 'is_default' => 0]);


            for ($i = 0; $i < count($request->department_id); $i++) {
                DB::table('tai_employee_department_mapping')->insert([
                    'tenant_id' => $request->tenant_id,
                    'employee_id' => $request->employee_id,
                    'department_id' => $request->department_id[0],
                    'ticket_show_type' => $request->ticket_show_type[0],
                    'is_default' => $request->is_default
                                                                        ]);
                }
            return response()->json(['status' => 1, 'message' => "Employee and Department Mapping Created !!!", 'data' => null]);
        } else {
            return response()->json(['status' => 0, 'message' => "Record Already Exists !!!", 'data' => null]);
        }
    }

    public function getDepartmentMappingByEmployeeId(Request $request, $employee_id)
    {
        $token_data = JWTAuth::parseToken()->authenticate(); //Date:30-05-2023 PUNAM SHINDE
        $tenantId = $token_data->tenant_id; //Date:30-05-2023 PUNAM SHINDE

        $data = DB::table('tai_employee_department_mapping as ed')
            ->leftJoin('tai_employee_master as eu', 'eu.id', '=', 'ed.updated_by')
            ->leftJoin('tai_employee_master as e', 'e.id', '=', 'ed.employee_id')
            ->leftJoin('tai_department_master as d', 'd.id', '=', 'ed.department_id')
            ->select(
                'ed.*',
                DB::raw("CONCAT(eu.first_name,' ',eu.last_name) AS updated_by"),
                                     DB::raw("CONCAT(e.first_name,' ',e.last_name) AS employee_name"),
                                     'd.department'
                            )
            ->where('ed.is_active', 1)
            ->where('ed.employee_id', $employee_id)
                    ->get();

        if (count($data) > 0) {
            return response()->json(['status' => 1, 'message' => "Record found !!!", 'data' => $data]);
        } else {
            return response()->json(['status' => 0, 'message' => "No Record found !!!", 'data' => null]);
        }         
    }

    public function updateDepartmentMapping(Request $request, $id)
    {
        $token_data = JWTAuth::parseToken()->authenticate(); //Date:30-05-2023 PUNAM SHINDE
        $tenantId = $token_data->tenant_id; //Date:30-05-2023 PUNAM SHINDE
        
        $data = $request->toArray();
        $result = DB::table('tai_employee_department_mapping')->where('id', $id)->update($data);
        if ($result) {
            return response()->json(['status' => 1, 'message' => "Record Updated !!!", 'data' => null]);
        } else {
            return response()->json(['status' => 0, 'message' => "Record Not Updated !!!", 'data' => null]);
        }         
    }

    public function deleteDepartmentMapping($id)
    {
        $result = DB::table('tai_employee_department_mapping')->where('id', $id)->update(['is_active' => 0]);
        if ($result) {
            return response()->json(['status' => 1, 'message' => "Record Deleted !!!", 'data' => null]);
        } else {
            return response()->json(['status' => 0, 'message' => "Record Not Deleted !!!", 'data' => null]);
        }        
    }
}
