<?php
// live
namespace App\Http\Controllers;

use App\Models\Department;
use Illuminate\Http\Request;
use DB;
use JWTAuth;

class DesignationController extends Controller
{
    public function getAllDesignation(Request $request, $tenantId = null)
    {
        $token_data = JWTAuth::parseToken()->authenticate(); //added by PUNAM 18-05-2023

        $result = DB::table('tai_designation_master as d')
            ->leftJoin('tai_employee_master as e', 'e.id', '=', 'd.updated_by')
            ->leftJoin('tai_employee_master as ec', 'ec.id', '=', 'd.created_by')
            ->select('d.*', DB::raw("CONCAT(e.first_name,' ',e.last_name) AS updated_by"), DB::raw("CONCAT(ec.first_name,' ',ec.last_name) AS created_by"))
            // ->where('d.tenant_id', $tenantId)
            ->where('d.tenant_id', $token_data->tenant_id)
            ->orderBy('d.is_active', 'DESC')
            ->orderBy('d.id', 'DESC')
            ->get();

        if (count($result) > 0) {
            return response()->json(['status' => 1, 'message' => "Record found !!!", 'data' => $result]);
        } else {
            return response()->json(['status' => 0, 'message' => "No Record found !!!", 'data' => null]);
        }
    }

    public function createDesignation(Request $request)
    {
        $token_data = JWTAuth::parseToken()->authenticate(); //added by PUNAM 18-05-2023

        if (!$request->designation) {
            return response()->json(['status' => 0, 'message' => "Invalid Data !!!", 'data' => null]);
        } elseif (preg_match('/^[a-zA-Z\s]+$/', $request->designation)) {
            $data = $request->toArray();
            $data['tenant_id'] = $token_data->tenant_id;

            $check = DB::table('tai_designation_master')
                ->where('tenant_id', $token_data->tenant_id)
                ->where('designation', $request->designation)
                ->get();

            if (count($check) == 0) {

                $result = DB::table('tai_designation_master')->insert($data);

                if ($result > 0) {
                    return response()->json(['status' => 1, 'message' => "Record Inserted !!!", 'data' => null]);
                } else {
                    return response()->json(['status' => 0, 'message' => "Record Not Inserted !!!", 'data' => null]);
                }
            } else {
                return response()->json(['status' => 0, 'message' => "Record Already Exists !!!", 'data' => null]);
            }
        } else {
            return response()->json(['status' => 0, 'message' => "Please input alphabet characters only !!!", 'data' => null]);
        }
    }

    public function updateDesignation(Request $request, $id)
    {
        $token_data = JWTAuth::parseToken()->authenticate(); //added by PUNAM 18-05-2023

        if (!$request->designation) {
            return response()->json(['status' => 0, 'message' => "Invalid Data !!!", 'data' => null]);
        } elseif (preg_match('/^[a-zA-Z\s]+$/', $request->designation)) {

            $data = $request->toArray();

            $check = DB::table('tai_designation_master')
                ->where('tenant_id', $token_data->tenant_id)
                ->where('id', '<>', $id)
                ->where('designation', $request->designation)
                ->get();

            if (count($check) == 0) {
                $result = DB::table('tai_designation_master')->where('id', $id)->update($data);
                if ($result) {
                    return response()->json(['status' => 1, 'message' => "Record Updated !!!", 'data' => null]);
                } else {
                    return response()->json(['status' => 0, 'message' => "Record Not Updated !!!", 'data' => null]);
                }
            } else {
                return response()->json(['status' => 0, 'message' => "Record Already Exists !!!", 'data' => null]);
            }
        } else {
            return response()->json(['status' => 0, 'message' => "Please input alphabet characters only !!!", 'data' => null]);
        }
    }

    public function deleteDesignation($id)
    {
        $token_data = JWTAuth::parseToken()->authenticate(); //added by PUNAM 18-05-2023

        $result = DB::table('tai_designation_master')->where('tenant_id', $token_data->tenant_id)->where('id', $id)->update(['is_active' => 0]);
        if ($result) {
            return response()->json(['status' => 1, 'message' => "Record Deleted !!!", 'data' => null]);
        } else {
            return response()->json(['status' => 0, 'message' => "Record Not Deleted !!!", 'data' => null]);
        }
    }

    public function getDesignationById($id)
    {
        $token_data = JWTAuth::parseToken()->authenticate(); //added by PUNAM 18-05-2023

        $result = DB::table('tai_designation_master')->where('id', $id)->first();
        if ($result) {
            return response()->json(['status' => 1, 'message' => "Record Found !!!", 'data' => $result]);
        } else {
            return response()->json(['status' => 0, 'message' => "No Record Found !!!", 'data' => null]);
        }
    }

    public function getdesignatedDropdowns($ticketId = null)
    {
        $token_data = JWTAuth::parseToken()->authenticate(); //added by PUNAM 18-05-2023

        $data['BA'] = DB::table('tai_employee_master as em')
            ->select('em.*')
            ->Leftjoin('tai_designation_master as tdm', 'em.designation_id', 'tdm.id')
            ->whereIn('tdm.designation', ['Business Analyst'])
            ->where('em.is_active', 1)
            ->get();
        $data['DEV'] = DB::table('tai_employee_master as em')
            ->select('em.*')
            ->Leftjoin('tai_designation_master as tdm', 'em.designation_id', 'tdm.id')
            ->whereIn('tdm.designation', ['Developer', 'developer', 'Jr Developer', 'Sr Developer'])
            ->where('em.is_active', 1)
            ->get();
        $data['TESTER'] = DB::table('tai_employee_master as em')
            ->select('em.*')
            ->Leftjoin('tai_designation_master as tdm', 'em.designation_id', 'tdm.id')
            ->whereIn('tdm.designation', ['Automation Tester'])
            ->where('em.is_active', 1)
            ->get();

        if ($ticketId) 
        {
            // $data['ALL'] = DB::table('tai_employee_master as em')
            //     ->select('em.*')
            //     ->Leftjoin('tai_owner_mapping_master as owmm', 'em.id', 'owmm.user_id')
            //     ->where('owmm.ticket_id', $ticketId)
            //     ->distinct('em.id')
            //     ->get();

            $data['ALL']=DB::table("tai_owner_mapping_master as o")
                            ->leftJoin('tai_employee_master as e','e.id','o.user_id')
                            ->select('e.*', 'o.type')
                            ->where('o.is_active', 1)
                            ->where('o.ticket_id',$ticketId)
                            // ->where('o.ticket_id',$token_data->tenant_id)
                            ->groupBy('o.user_id')                        
                            ->get();
                            
        }
        return response()->json(['status' => 1, 'message' => "Record Found !!!", 'data' => $data]);
    }
}
