<?php

namespace App\Http\Controllers;

use App\Models\Employee;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\Hash;
use DB;
use JWTAuth;

class DynamicFormDropdownMasterController extends Controller
{

    public function getAllDynamicFormDropdown(Request $request)
    {
        $token_data = JWTAuth::parseToken()->authenticate(); //Date:30-05-2023 PUNAM SHINDE
        $tenantId = $token_data->tenant_id; //Date:30-05-2023 PUNAM SHINDE

        $result = DB::table('tai_dynamic_form_dropdown_master as c')
            ->leftJoin('tai_employee_master as e', 'e.id', '=', 'c.updated_by')
            ->select('c.*', DB::raw("CONCAT(e.first_name,' ',e.last_name) AS updated_by"))
            ->where('c.tenant_id', $tenantId)
            ->orderBy('c.is_active', 'ASC')
            ->orderBy('c.id', 'DESC')
            ->get();
        if (count($result) > 0) {
            return response()->json(['status' => 1, 'message' => "Record found !!!", 'data' => $result]);
        } else {
            return response()->json(['status' => 0, 'message' => "No Record found !!!", 'data' => null]);
        }
    }


    public function getAllDynamicFormDropdownData(Request $request, $dropdownId)
    {
        $token_data = JWTAuth::parseToken()->authenticate(); //Date:30-05-2023 PUNAM SHINDE
        $tenantId = $token_data->tenant_id; //Date:30-05-2023 PUNAM SHINDE

        $result = DB::table('tai_dynamic_form_dropdown_data as c')
            ->leftJoin('tai_employee_master as e', 'e.id', '=', 'c.updated_by')
            ->select('c.*', DB::raw("CONCAT(e.first_name,' ',e.last_name) AS updated_by"))
            ->where('c.tenant_id', $tenantId)
            ->where('c.dropdown_id', $dropdownId)
            ->where('c.is_active', 1)
            ->orderBy('c.is_active', 'ASC')
            ->orderBy('c.id', 'DESC')
            ->get();
        if (count($result) > 0) {
            return response()->json(['status' => 1, 'message' => "Record found !!!", 'data' => $result]);
        } else {
            return response()->json(['status' => 0, 'message' => "No Record found !!!", 'data' => null]);
        }
    }

    public function createDropdown(Request $request)
    {

        $token_data = JWTAuth::parseToken()->authenticate(); //Date:30-05-2023 PUNAM SHINDE
        $tenantId = $token_data->tenant_id; //Date:30-05-2023 PUNAM SHINDE

        if ($request->dropdown_name && $request->tenant_id) {
            $check = DB::table('tai_dynamic_form_dropdown_master')->where('dropdown_name', $request->dropdown_name)
                ->where('tenant_id', $tenantId)
                ->first();
            if ($check) {
                return response()->json(['status' => 0, 'message' => "Dropdown already exists !!!", 'data' => null]);
            }
        } else {
            return response()->json(['status' => 0, 'message' => "Dropdown Name or Tenant Name is missing !!!", 'data' => null]);
        }

        $data = [
            'tenant_id' => $request->tenant_id,
            'dropdown_name' => $request->dropdown_name,
            'created_by' => $request->created_by,
            'is_active' => $request->is_active ? $request->is_active : 1,
        ];

        if ($request->dynamic_dropdown_name == "FETCHING") {
            $data["api_source"] = $request->api_source;
            $data["value_source"] = $request->value_source;
            $data["label_source"] = $request->label_source;
            $data["api_type"] = $request->api_type;
            $data["api_key_data"] = $request->api_key_data;
        }


        $insert = DB::table('tai_dynamic_form_dropdown_master')->insertGetId($data);

        if ($insert > 0) {
            if ($request->dynamic_dropdown_name != "FETCHING") {
                for ($i = 0; $i < count($request->dropdown_values); $i++) {
                    DB::table('tai_dynamic_form_dropdown_data')->insert([
                        'tenant_id' => $request->tenant_id,
                        'dropdown_id' => $insert,
                        'label' => $request->dropdown_values[$i],
                        'value' => $request->dropdown_values[$i],
                        'is_active' => $request->is_active ? $request->is_active : 1,
                        'created_by' => $request->created_by
                    ]);
                } //for
            } //if
        } //if
        return response()->json(['status' => 1, 'message' => "Dropdown created !!!", 'data' => null]);
    }


    public function getDropdownByName(Request $request)
    {
        $data = $request->toArray();

        $master = DB::table('tai_dynamic_form_dropdown_master')->where('id', $request->dropdownId)->first();

        if ($master->api_source) {
            $curl = curl_init();
            $requestArray = array(
                CURLOPT_URL => $master->api_source,
                CURLOPT_RETURNTRANSFER => true,
                CURLOPT_ENCODING => '',
                CURLOPT_MAXREDIRS => 10,
                CURLOPT_TIMEOUT => 0,
                CURLOPT_FOLLOWLOCATION => true,
                CURLOPT_HTTP_VERSION => CURL_HTTP_VERSION_1_1,
                CURLOPT_CUSTOMREQUEST => $master->api_type,
            );
            if ($request->key && $request->value) {
                $postData[$request->key] = $request->value;
                $requestArray[CURLOPT_POSTFIELDS] = $postData;
            }
            curl_setopt_array($curl, $requestArray);

            $response = curl_exec($curl);
            curl_close($curl);
            $response = json_decode($response, true);

            $keyData = $master->api_key_data ? $response[$master->api_key_data] : $response;
            $dropdown = [];
            foreach ($keyData as $k) {
                $temp = [];
                $temp["id"] = $master->value_source ? $k[$master->value_source] : $k["value"];
                $temp["label"] = $master->label_source ? $k[$master->label_source] : $k["label"];
                $dropdown[] = $temp;
            }
            $data['dropdown'] = $dropdown;
        } else {
            $dropdown = DB::table('tai_dynamic_form_dropdown_data')->where('dropdown_id', $master->dropdownId)->where('is_active', 1)->get();
            $data['dropdown'] = $dropdown;
        }
        return response()->json(['status' => 1, 'message' => "Dropdown Data Found !!!", 'data' => $data]);
    }

    public function getDropdownById(Request $request, $dropdownId, $itemId = null)
    {
        $token_data = JWTAuth::parseToken()->authenticate(); //Date:30-05-2023 PUNAM SHINDE
        $tenantId = $token_data->tenant_id; //Date:30-05-2023 PUNAM SHINDE

        if ($dropdownId) {
            $master = DB::table('tai_dynamic_form_dropdown_master')->where('id', $dropdownId)->first();
            if ($master) {
                $data['master'] = $master;
                if ($master->api_source) {
                    $curl = curl_init();
                    $requestArray = array(
                        CURLOPT_URL => $master->api_source,
                        CURLOPT_RETURNTRANSFER => true,
                        CURLOPT_ENCODING => '',
                        CURLOPT_MAXREDIRS => 10,
                        CURLOPT_TIMEOUT => 0,
                        CURLOPT_FOLLOWLOCATION => true,
                        CURLOPT_HTTP_VERSION => CURL_HTTP_VERSION_1_1,
                        CURLOPT_CUSTOMREQUEST => $master->api_type,
                    );
                    curl_setopt_array($curl, $requestArray);
                    $response = curl_exec($curl);
                    curl_close($curl);
                    $response = json_decode($response, true);

                    $keyData = $master->api_key_data ? $response[$master->api_key_data] : $response;
                    $dropdown = [];

                    foreach ($keyData as $k) {
                        $temp = [];
                        $temp["id"] = $master->value_source ? $k[$master->value_source] : $k["value"];
                        $temp["label"] = $master->label_source ? $k[$master->label_source] : $k["label"];

                        $dropdown[] = $temp;
                    }
                    $data['dropdown'] = $dropdown;
                } else {
                    $dropdown = DB::table('tai_dynamic_form_dropdown_data')->where('dropdown_id', $dropdownId)->where('is_active', 1)->get();
                    $data['dropdown'] = $dropdown;
                }

                return response()->json(['status' => 1, 'message' => "Dropdown Data Found !!!", 'data' => $data]);
            } else {
                return response()->json(['status' => 0, 'message' => "Dropdown Data Not Found !!!", 'data' => NULL]);
            }
        }
    }


    public function getDropdownByIdOLD(Request $request, $dropdownId, $itemId = null)
    {
        // $connectUs = [16, 17, 18];
        // if (!in_array($dropdownId, $connectUs)) 
        // {
        if ($dropdownId) {
            $master = DB::table('tai_dynamic_form_dropdown_master')->where('id', $dropdownId)->first();
            if ($master) {
                $data['master'] = $master;
                if ($master->api_source) {
                    $curl = curl_init();
                    curl_setopt_array($curl, array(
                        CURLOPT_URL => $master->api_source,
                        CURLOPT_RETURNTRANSFER => true,
                        CURLOPT_ENCODING => '',
                        CURLOPT_MAXREDIRS => 10,
                        CURLOPT_TIMEOUT => 0,
                        CURLOPT_FOLLOWLOCATION => true,
                        CURLOPT_HTTP_VERSION => CURL_HTTP_VERSION_1_1,
                        CURLOPT_CUSTOMREQUEST => 'GET',
                    ));
                    $response = curl_exec($curl);
                    curl_close($curl);
                    $response = json_decode($response, true);


                    $keyData = $master->api_key_data ? $response[$master->api_key_data] : $response;
                    $dropdown = [];
                    foreach ($keyData as $k) {
                        $temp = [];
                        $temp["id"] = $master->value_source ? $k[$master->value_source] : $k;
                        $temp["label"] = $master->label_source ? $k[$master->label_source] : $k;
                        $dropdown[] = $temp;
                    }
                    $data['dropdown'] = $dropdown;
                } else {
                    $dropdown = DB::table('tai_dynamic_form_dropdown_data')->where('dropdown_id', $dropdownId)->where('is_active', 1)->get();
                    $data['dropdown'] = $dropdown;
                }

                return response()->json(['status' => 1, 'message' => "Dropdown Data Found !!!", 'data' => $data]);
            } else {
                return response()->json(['status' => 1, 'message' => "Dropdown Data Found !!!", 'data' => $data]);
            }
        } else {
            return response()->json(['status' => 1, 'message' => "Dropdown Id Is Missing !!!", 'data' => null]);
        }

        // } else {
        //     $master = DB::table('tai_dynamic_form_dropdown_master')->where('id', $dropdownId)->first();
        //     $postData['item'] = ($itemId) ? $itemId : null;
        //     $postData['master'] = $master->dropdown_name;
        //     $curl = curl_init();
        //     curl_setopt_array($curl, array(
        //         CURLOPT_URL => 'http://114.29.232.154/connectus/api/ticketing/dropdownData',
        //         CURLOPT_RETURNTRANSFER => true,
        //         CURLOPT_ENCODING => '',
        //         CURLOPT_MAXREDIRS => 10,
        //         CURLOPT_TIMEOUT => 0,
        //         CURLOPT_FOLLOWLOCATION => true,
        //         CURLOPT_HTTP_VERSION => CURL_HTTP_VERSION_1_1,
        //         CURLOPT_CUSTOMREQUEST => 'POST',
        //         CURLOPT_POSTFIELDS => $postData
        //     ));
        //     $response = curl_exec($curl);
        //     curl_close($curl);
        //     $response = json_decode($response, true);
        //     $data['master'] = $master;
        //     $data['dropdown'] = $response;
        //     return response()->json(['status' => 1, 'message' => "Dropdown created !!!", 'data' => $data]);
        // }
    }

    public function updateDropdown(Request $request, $id)
    {
        $token_data = JWTAuth::parseToken()->authenticate(); //Date:30-05-2023 PUNAM SHINDE
        $tenantId = $token_data->tenant_id; //Date:30-05-2023 PUNAM SHINDE

        if (!$request->dropdown_id || !$request->tenant_id || !$request->is_active) {
            return response()->json(['status' => 1, 'message' => "Param Missing !!!", 'data' => null]);
        }
        $check = DB::table('tai_dynamic_form_dropdown_master')->where('id', $id)
            ->update([
                'dropdown_name' => $request->dropdown_name,
                'updated_at' => $request->updated_at,
                'updated_by' => $request->updated_by,
                'is_active' => $request->is_active
            ]);
        // for ($i = 0; $i < count($request->value); $i++) {
        //     if ($request->dropdown_id[$i]) {

        if($request->dropdown_values && count($request->dropdown_values))
        {
            for ($i = 0; $i < count($request->dropdown_values); $i++) {
                if ($request->dropdown_id[$i] && $request->dropdown_id[$i] != NULL && $request->dropdown_id[$i] != "null") {
                    DB::table('tai_dynamic_form_dropdown_data')->where('id', $request->dropdown_id[$i])
                        ->update([
                            'label' => $request->dropdown_values[$i],
                            'value' => $request->dropdown_values[$i],
                            'is_active' => $request->is_active,
                            'updated_by' => $request->updated_by,
                            'updated_at' => $request->updated_at
                        ]);
                } else {
                    DB::table('tai_dynamic_form_dropdown_data')->insert([
                        'tenant_id' => $request->tenant_id,
                        'dropdown_id' => $id,
                        'label' => $request->dropdown_values[$i],
                        'value' => $request->dropdown_values[$i],
                        'is_active' => 1,
                        'created_by' => $request->updated_by,
                        'created_at' => $request->updated_at
                    ]);
                }
            }
            return response()->json(['status' => 1, 'message' => "Dropdown updated !!!", 'data' => null]);
        }
        return response()->json(['status' => 1, 'message' => "Dropdown updated !!!", 'data' => null]);
        
    }


    public function getAllDropdown(Request $request)
    {
        $token_data = JWTAuth::parseToken()->authenticate(); //Date:30-05-2023 PUNAM SHINDE
        $tenantId = $token_data->tenant_id; //Date:30-05-2023 PUNAM SHINDE

        $master = DB::table('tai_dynamic_form_dropdown_master')->where('tenant_id', $tenantId)
            ->where('is_active', 1)
            ->get();
        if ($master) {
            return response()->json(['status' => 1, 'message' => "Data Found !!!", 'data' => $master]);
        } else {
            return response()->json(['status' => 1, 'message' => "Data  Not Found !!!", 'data' => NULL]);
        }
    }
}
