<?php

namespace App\Http\Controllers;

use App\Models\Employee;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\Hash;
use DB;
use Illuminate\Support\Facades\Storage;

use JWTAuth;
use Tymon\JWTAuth\Exceptions\JWTException;
use Symfony\Component\HttpFoundation\Response;
use Illuminate\Support\Facades\Validator;

class EmployeeController extends Controller
{
    // Dummy API for testing purposes of JWT authentication Token
    public function getToken(Request $request)
    {

        // return [$request->all()];
        $credentials = $request->only('email_id', 'password');
        try {
            if (!$token = JWTAuth::attempt($credentials)) {
                return response()->json([
                    'success' => false,
                    'message' => 'Login credentials are invalid.',
                ], 400);
            }
        } catch (JWTException $e) {
            return response()->json([
                'success' => false,
                'message' => 'Could not create token.',
            ], 500);
        }
        //Token created, return with success response and jwt token
        return response()->json(['success' => true, 'token' => $token]);
    }

    public function afterToken(Request $request)
    {
        return JWTAuth::parseToken()->authenticate();
    }
    // dummy----------------------------------------------------------


    public function dataMerger(Request $request)
    {
        $emp = DB::table('tai_employee_master')->get();
        foreach ($emp as $e) {
            $a = DB::table('tai_employee_department_mapping')->where('employee_id', $e->id)->where('is_default', 1)->first();
            if ($a) {
                DB::table('tai_ticket_master')->where('created_by', $e->id)
                    ->whereNull('from_department_id')
                    ->update([
                        'user_id' => $e->id,
                        'from_department_id' => $a->department_id
                    ]);
            }
        }
    }

    public function login(Request $request)
    {
        $response = ['status' => 0, 'message' => null, 'data' => NULL];

        try {
            if ($request->email_id && $request->password) {
                $query = DB::table('tai_employee_master as e')
                    ->leftJoin('tai_department_master as d', 'd.id', '=', 'e.department_id')
                    ->leftJoin('tai_role_master as r', 'r.id', '=', 'e.role_id')
                    ->leftJoin('tai_designation_master as dg', 'dg.id', '=', 'e.designation_id')
                    ->where('e.is_active', 1);

                if (filter_var($request->email_id, FILTER_VALIDATE_EMAIL)) {
                    $query->where('e.email_id', $request->email_id);

                    $response = ['status' => 0, 'message' => "Email Id does not match !!!", 'data' => NULL, 'type' => 'USERNAME'];
                } else if (is_numeric($request->email_id)) {
                    $query->where('e.contact_no', $request->email_id);

                    $response = ['status' => 0, 'message' => "Contact Number Does not match !!!", 'data' => NULL, 'type' => 'USERNAME'];
                    // dd($response);
                } else {
                    $query->where(DB::raw('e.`user_name`'), 'Like', $request->email_id);

                    $response = ['status' => 0, 'message' => "User does not exist !!!", 'data' => NULL, 'type' => 'USERNAME'];
                }

                $multiple = $query->select('e.*', 'd.department as department_name', 'r.role as role_name', 'dg.designation as designation')->get();

                if (count($multiple) > 1) {
                    $response = ['status' => 0, 'message' => "Multiple Account with this Email. Login with USERNAME!!!"];
                    return response()->json($response);
                } else {
                    $check = $query->select('e.*', 'd.department as department_name', 'r.role as role_name', 'dg.designation as designation')->first();
                    if ($check) {
                        if ($check->is_active != 1) {
                            $response = ['status' => 0, 'message' => "User is deactivated !!!", 'data' => NULL];
                            return response()->json($response);
                        } else {

                            // $credentials = ['email_id' => $check->email_id, 'user_name'=>$check->user_name, 'password' => $request->password];
                            $credentials = ['user_name' => $check->user_name, 'password' => $request->password];

                            if (Hash::check($request->password, $check->password)) {
                                if ($check->tenant_id) {
                                    $data = (array)$check;
                                    $checkMenu = DB::table('tai_menu_access')->where('role_id', $check->role_id)->get();
                                    if (count($checkMenu)) {
                                        $department_id = DB::table('tai_employee_department_mapping')->where('employee_id', $check->id)
                                            ->where('tenant_id', $check->tenant_id)
                                            ->where('is_active', 1)
                                            ->pluck('department_id')
                                            ->toArray();

                                        if (count($department_id)) {
                                            $data['department_id'] = $department_id;

                                            // dd($data);

                                            // JWT TOken 
                                            if (!$token = JWTAuth::attempt($credentials)) {
                                                return response()->json([
                                                    'success' => false,
                                                    'message' => 'Login credentials are invalid.',

                                                ], 400);
                                            } else {
                                                return response()->json([
                                                    'status' => 1,
                                                    'message' => "Record found !!!",
                                                    'data' => $data,
                                                    'token' => $token
                                                ]);
                                            }
                                        } else {
                                            return response()->json(['status' => 0, 'message' => "No Department has been mapped !!!", 'data' => NULL]);
                                        }
                                    } else {
                                        return response()->json(['status' => 0, 'message' => "No Menu has been assigned for your role !!!", 'data' => NULL]);
                                    }
                                } else {
                                    return response()->json(['status' => 0, 'message' => "Company Details Not Found !!!", 'data' => NULL]);
                                }
                            } else {
                                return response()->json(['status' => 0, 'message' => "Password does not match !!!", 'data' => NULL, 'type' => 'PASSWORD']);
                            }
                        }
                    } else {
                        return response()->json($response);
                    }
                }
            } else {
                return response()->json(['status' => 0, 'message' => "Please fill all fields !!!", 'data' => NULL]);
            }
        } catch (JWTException $e) {
            return response()->json([
                'success' => false,
                'message' => 'Could not create token.',
            ], 500);
        }
        return response()->json(['success' => true, 'token' => $token]);
    }

    public function getUserWithMultipleDepartment(Request $request, $department_id = null)
    {
        $token_data = JWTAuth::parseToken()->authenticate();
        $tenantId = $token_data->tenant_id;
        $customerId = $token_data->customer_id;

        $query = DB::table('tai_employee_master as e')
            ->leftJoin('tai_employee_master as ej', 'ej.id', '=', 'e.updated_by')
            ->leftJoin('tai_employee_master as ec', 'ec.id', '=', 'e.created_by')
            ->leftJoin('tai_customer_master as c', 'c.id', '=', 'e.customer_id')
            ->leftjoin('tai_role_master as tr', 'e.role_id', 'tr.id')
            ->leftjoin('tai_designation_master as td', 'e.designation_id', 'td.id')
            ->leftjoin('tai_department_master as d', 'e.department_id', 'd.id')
            ->leftjoin('tai_country_master as tcm', 'e.country_id', 'tcm.id')
            ->leftjoin('tai_city_master as tc', 'e.city_id', 'tc.id')
            ->leftjoin('tai_state_master as ts', 'e.state_id', 'ts.id');

        $query->select(
            'e.*',
            'tcm.country',
            'tc.city',
            'ts.state',
            'tr.role',
            'd.department',
            'td.designation',
            DB::raw("CONCAT(ej.first_name,' ',ej.last_name) AS updated_by"),
            DB::raw("CONCAT(ec.first_name,' ',ec.last_name) AS created_by"),
            'c.name as customer'
        );
        $result = $query->orderBy('e.id')->get();

        foreach ($result as $r) {
            $departments = DB::table('tai_employee_department_mapping')->where('employee_id', $r->id)->where('is_active', 1)->pluck('department_id')->toArray();

            $r->multiple_department_id = $departments;
        }


        if (count($result) > 0) {
            return response()->json(['status' => 1, 'message' => "Record found !!!", 'data' => $result]);
        } else {
            return response()->json(['status' => 0, 'message' => "No Record found !!!", 'data' => null]);
        }
    }


    public function getData(Request $request, $ids = null)
    {
        $token_data = JWTAuth::parseToken()->authenticate();
        $tenantId = $token_data->tenant_id;
        $customerId = $token_data->customer_id;

        // $result = DB::table('tai_employee_master')->where('is_active', 1)->get();
        $query = DB::table('tai_employee_master as e')
            ->leftJoin('tai_employee_master as ej', 'ej.id', '=', 'e.updated_by')
            ->leftJoin('tai_employee_master as ec', 'ec.id', '=', 'e.created_by')
            ->leftJoin('tai_customer_master as c', 'c.id', '=', 'e.customer_id')
            ->leftjoin('tai_role_master as tr', 'e.role_id', 'tr.id')
            ->leftjoin('tai_designation_master as td', 'e.designation_id', 'td.id')
            ->leftjoin('tai_department_master as d', 'e.department_id', 'd.id')
            ->leftjoin('tai_country_master as tcm', 'e.country_id', 'tcm.id')
            ->leftjoin('tai_city_master as tc', 'e.city_id', 'tc.id')
            ->leftjoin('tai_state_master as ts', 'e.state_id', 'ts.id');
        $query->select(
            'e.*',
            'tcm.country',
            'tc.city',
            'ts.state',
            'tr.role',
            'd.department',
            'td.designation',
            DB::raw("CONCAT(ej.first_name,' ',ej.last_name) AS updated_by"),
            DB::raw("CONCAT(ec.first_name,' ',ec.last_name) AS created_by"),
            'c.name as customer',
        );


        $result = $query->orderByDesc('e.id')->get();
        if (count($result) > 0) {
            foreach ($result as $r) {
                $ticket_show_type = DB::table('tai_employee_department_mapping as m')
                    ->leftJoin('tai_department_master as d', 'd.id', '=', 'm.department_id')
                    ->where('m.employee_id', $r->id)
                    ->select("d.department", "m.ticket_show_type")
                    ->where('m.is_active', 1)
                    ->get();
                // $q="SELECT d.department,ticket_show_type FROM tai_employee_department_mapping  AS m 
                //                         LEFT JOIN tai_department_master AS d ON d.id =m.department_id
                //                         WHERE employee_id=$r->id AND ticket_show_type IS NOT NULL GROUP BY ticket_show_type";
                // $departmentData=DB::select(DB::raw($q))->toArray();
                // dd($departmentData);
                // dd($r->ticket_show_type);

                $passing_auth = DB::table('tai_employee_department_mapping')
                    ->select('department_id')
                    ->where('employee_id', $r->id)
                    ->where('is_active', 1)
                    ->where('ticket_passing_authority', 1)
                    ->get();

                $r->passing_auth = $passing_auth;

                $r->ticket_show_type = "";
                $r->all_department = "";
                if (count($ticket_show_type)) {
                    foreach ($ticket_show_type as $x) {
                        $r->ticket_show_type .= $x->ticket_show_type . ",";
                        $r->all_department .= $x->department . ",";
                    }
                    $r->ticket_show_type = rtrim($r->ticket_show_type, ',');
                    $r->all_department = rtrim($r->all_department, ',');
                }
            }
            return response()->json(['status' => 1, 'message' => "Record found !!!", 'data' => $result]);
        } else {
            return response()->json(['status' => 0, 'message' => "No Record found !!!", 'data' => null]);
        }
    }


    public function getExcept(Request $request, $ids = null)
    {
        // $result = DB::table('tai_employee_master')->where('is_active', 1)->get();

        $query = DB::table('tai_employee_master as e')
            ->leftJoin('tai_employee_master as ej', 'ej.id', '=', 'e.updated_by')
            ->leftJoin('tai_customer_master as c', 'c.id', '=', 'e.customer_id')
            ->leftjoin('tai_role_master as tr', 'e.role_id', 'tr.id')
            ->leftjoin('tai_designation_master as td', 'e.designation_id', 'td.id')
            ->leftjoin('tai_employee_department_mapping as tedm', 'e.id', 'tedm.employee_id')
            ->leftjoin('tai_department_master as tdm', 'tedm.department_id', 'tdm.id');

        // country, city, state, role, designation, department
        if ($request->header('X-Type')) {
            $query->select(
                'e.id',
                'e.tenant_id',
                'e.first_name',
                'e.middle_name',
                'e.last_name',
                'e.email_id',
                'e.contact_no',
                'e.role_id',
                'e.department_id',
                'e.designation_id',
                'tr.role',
                'td.designation'
            );
        } else {
            $query->select(
                'e.*',
                'tcm.country',
                'tc.city',
                'ts.state',
                'tr.role',
                'tdm.department',
                //'tetd.ticket_show_type','tetd.ticket_passing_authority', 'tetd.is_default', 
                'td.designation',
                DB::raw("CONCAT(ej.first_name,' ',ej.last_name) AS updated_by"),
                'c.name as customer'
            )
                ->leftjoin('tai_country_master as tcm', 'e.country_id', 'tcm.id')
                ->leftjoin('tai_city_master as tc', 'e.city_id', 'tc.id')
                ->leftjoin('tai_state_master as ts', 'e.state_id', 'ts.id')
                //->leftjoin('tai_employee_department_mapping as tetd', 'e.id', 'tetd.employee_id')
            ;
        }

        if ($ids != "") {
            $ids = explode(',', $ids);
            $query->whereNotIn('e.id', $ids);
        }
        $result = $query->where(function ($q) {
            $q->where('tedm.is_active', 1);
            $q->where('tedm.is_default', 1);
        })
            ->orderBy('e.id')
            ->get();
        if (count($result) > 0) {
            return response()->json(['status' => 1, 'message' => "Record found !!!", 'data' => $result]);
        } else {
            return response()->json(['status' => 0, 'message' => "No Record found !!!", 'data' => null]);
        }
    }

    public function getById($id)
    {
        $token_data = JWTAuth::parseToken()->authenticate();
        $tenantId = $token_data->tenant_id;
        $customerId = $token_data->customer_id != 0 ? [$token_data->customer_id, 0] : [0];

        $result = DB::table('tai_employee_master as e')
            ->leftJoin('tai_role_master as r', 'r.id', '=', 'e.role_id')
            ->leftJoin('tai_designation_master as d', 'd.id', '=', 'e.designation_id')
            ->where('e.id', $id)
            ->where('e.tenant_id', $tenantId)
            // ->whereIn('e.customer_id', $customerId)
            ->select('e.*', 'd.designation as designation_name', 'r.role as role_name')
            ->first();
        if ($result) {
            if ($result->profile_picture == null) {
                $result->profile_picture = "storage/app/Defaults/profile.png";
            }
            $department = DB::table('tai_employee_department_mapping as dm')
                ->leftJoin('tai_department_master as d', 'd.id', 'dm.department_id')
                ->where('dm.employee_id', $id)
                ->where('dm.is_active', 1)
                ->select('dm.*', 'd.department as department_name')
                ->get();
            $result->department = $department;
            return response()->json(['status' => 1, 'message' => "Record Found !!!", 'data' => $result]);
        } else {
            return response()->json(['status' => 0, 'message' => "No Record Found !!!", 'data' => null]);
        }
    }

    public function updateAccountDetails(Request $request, $id)
    {
        $token_data = JWTAuth::parseToken()->authenticate();
        $tenantId = $token_data->tenant_id;
        $customerId = $token_data->customer_id != 0 ? [$token_data->customer_id, 0] : [0];

        // dd($request->all());
        if ($id) {
            $array = [
                'first_name' => $request->first_name,
                'last_name' => $request->last_name,
                'email_id' => $request->email_id,
                'contact_no' => $request->contact_no,
                'whats_app_contact_no' => $request->whats_app_contact_no,
                'updated_by' => $request->updated_by,
                'updated_at' => now(),
                'preferred_communication' => $request->preferred_communication ? $request->preferred_communication : "EMAIL",
            ];
            if ($request->file('profile_picture')) {
                $path = "Profile";
                $fileName = $request->profile_picture->getClientOriginalName();
                $isUploaded = $request->profile_picture->storeAs($path, $fileName);
                $array['profile_picture'] = "storage/app/" . $path . "/" . $fileName;
            }

            $update = DB::table('tai_employee_master')->where('id', $id)
                ->update($array);
            if ($update) {
                $emp_data = DB::table('tai_employee_master')->where('id', $id)->first();
                return response()->json(['status' => 1, 'message' => "Profile Updated !!!", 'data' => $emp_data]);
            } else {
                return response()->json(['status' => 0, 'message' => "Profile Not Updated !!!", 'data' => null]);
            }
        } else {
            return response()->json(['status' => 0, 'message' => "No Id found !!!", 'data' => null]);
        }
    }

    public function updatePasswordDetails(Request $request, $id)
    {
        $token_data = JWTAuth::parseToken()->authenticate();
        $tenantId = $token_data->tenant_id;
        $customerId = $token_data->customer_id != 0 ? [$token_data->customer_id, 0] : [0];

        // dd($request->all(), $id);
        if ($id) {

            $emp_data = DB::table('tai_employee_master')->where('id', $id)->first();
            // dd($emp_data);
            if ($request->password && $request->confirm_password) {
                if (Hash::check($request->password, $emp_data->password)) {

                    return response()->json(['status' => 0, 'message' => "You have been entered old password. Please enter new password !!!", 'data' => NULL]);
                } else {

                    if ($request->password == $request->confirm_password) {
                        $update = DB::table('tai_employee_master')->where('id', $id)
                            ->update([
                                'password' => Hash::make($request->password),
                                'updated_by' => $request->updated_by,
                                'updated_at' => now(),
                            ]);
                        if ($update) {
                            return response()->json(['status' => 1, 'message' => "Password Updated !!!", 'data' => null]);
                        } else {
                            return response()->json(['status' => 0, 'message' => "Password Not Updated !!!", 'data' => null]);
                        }
                    } else {
                        return response()->json(['status' => 0, 'message' => "New password does not match !!!", 'data' => null]);
                    }
                }
            } else {
                return response()->json(['status' => 0, 'message' => "Please enter new password !!!", 'data' => null]);
            }
        } else {
            return response()->json(['status' => 0, 'message' => "No Id found !!!", 'data' => null]);
        }
    }

    public function postData(Request $request)
    {
        $token_data = JWTAuth::parseToken()->authenticate();
        $tenantId = $token_data->tenant_id;
        $customerId = $token_data->customer_id != 0 ? [$token_data->customer_id, 0] : [0];

        if (!$request->tenant_id) {
            return response()->json(['status' => 0, 'message' => "Unable to get tenant !!!", 'data' => null]);
        }

        if (!$request->account_for) {
            return response()->json(['status' => 0, 'message' => "Unable to get account for !!!", 'data' => null]);
        }

        if (!$request->first_name || !$request->middle_name || !$request->last_name) {
            return response()->json(['status' => 0, 'message' => "Required Full Name !!!", 'data' => null]);
        }

        if (count($request->department_id) == 0) {
            return response()->json(['status' => 0, 'message' => "Please Map Department for user !!!", 'data' => null]);
        }

        $check = DB::table('tai_employee_master')
            ->where('user_name', $request->user_name)
            ->first();
        if ($check) {
            return response()->json(['status' => 0, 'message' => "Username is aleady Exists", 'data' => null]);
        }


        $data = array(
            'tenant_id' => $tenantId,
            'account_for' => $request->account_for,
            'customer_id' => $request->customer_id,
            'first_name' => $request->first_name,
            'last_name' => $request->last_name,
            'middle_name' => $request->middle_name,
            'email_id' => $request->email_id,
            'whats_app_contact_no' => $request->whats_app_contact_no,
            'contact_no' => $request->contact_no,
            'role_id' => $request->role_id,
            'designation_id' => $request->designation_id,
            'gender' => $request->gender,
            'address' => $request->address,
            'city_id' => $request->city_id,
            'country_id' => $request->country_id,
            'state_id' => $request->state_id,
            'pincode' => $request->pincode,
            'user_name' => $request->user_name,
            'password' => Hash::make($request->password),
            'created_at' => $request->created_at,
            'created_by' => $request->created_by
        );
        if ($request->file('profile_picture')) {
            $path = "Profile";
            $fileName = $request->profile_picture->getClientOriginalName();
            $isUploaded = $request->profile_picture->storeAs($path, $fileName);
            $data['profile_picture'] = "storage/app/" . $path . "/" . $fileName;
        } else {
            $data['profile_picture'] = "storage/app/Defaults/profile.png";
        }
        $result = DB::table('tai_employee_master')->insertGetId($data);

        if ($result) {

            for ($i = 0; $i < count($request->department_id); $i++) {

                if ($request->is_default[$i] == 1 || $request->is_default[$i]) {
                    $defaultDepartment = $request->department_id[$i];
                }

                DB::table('tai_employee_department_mapping')->insert([
                    'tenant_id' => $tenantId,
                    'employee_id' => $result,
                    'department_id' => $request->department_id[$i],
                    'ticket_passing_authority' => $request->ticket_passing_authority[$i],
                    'ticket_show_type' => $request->ticket_show_type[$i],
                    'is_default' => array_key_exists($i, $request->is_default) ? $request->is_default[$i] : 0
                ]);
            }

            DB::table('tai_employee_master')->where('id', $result)->update(['department_id' => $defaultDepartment]);

            return response()->json(['status' => 1, 'message' => "Record Inserted !!!", 'data' => null]);
        } else {
            return response()->json(['status' => 0, 'message' => "Record Not Inserted !!!", 'data' => null]);
        }
    }

    public function putData(Request $request, $id)
    {
        $token_data = JWTAuth::parseToken()->authenticate();
        $tenantId = $token_data->tenant_id;
        $customerId = $token_data->customer_id != 0 ? [$token_data->customer_id, 0] : [0];

        $data = array(
            'account_for' => $request->account_for,
            'customer_id' => $request->customer_id,
            'first_name' => $request->first_name,
            'last_name' => $request->last_name,
            'middle_name' => $request->middle_name,
            'email_id' => $request->email_id,
            'whats_app_contact_no' => $request->whats_app_contact_no,
            'contact_no' => $request->contact_no,
            'role_id' => $request->role_id,
            'designation_id' => $request->designation_id,
            'gender' => $request->gender,
            'address' => $request->address,
            'city_id' => $request->city_id,
            'country_id' => $request->country_id,
            'state_id' => $request->state_id,
            'pincode' => $request->pincode,
            'user_name' => $request->user_name,
            'updated_at' => $request->updated_at,
            'updated_by' => $request->updated_by,
            'is_active' => $request->is_active
        );

        if ($request->password) {
            if (strlen($request->password) < 6 || strlen($request->password) > 20) {
                return response()->json(['status' => 0, 'message' => "Password should be 6 to 20 characters!!!", 'data' => $request->toArray()]);
            }

            if ($request->password != $request->confirm_password) {
                return response()->json([
                    'status' => 0, 'message' => "Confirm Password does not match!!!",
                    'data' => $request->toArray()
                ]);
            }

            $data['password'] = Hash::make($request->password);
        }

        if ($request->file('profile_picture')) {
            $path = "Profile";
            $fileName = $request->profile_picture->getClientOriginalName();
            $isUploaded = $request->profile_picture->storeAs($path, $fileName);
            $data['profile_picture'] = "storage/app/" . $path . "/" . $fileName;
        }

        $result = DB::table('tai_employee_master')->where('id', $id)->update($data);

        $oldData = DB::table('tai_employee_department_mapping')->where('employee_id', $id)->update(['is_active' => 0, 'updated_at' => now()]);

        $defaultDepartment = 0;

        for ($i = 0; $i < count($request->department_id); $i++) {
            if ($request->is_default[$i] == 1 || $request->is_default[$i]) {
                $defaultDepartment = $request->department_id[$i];
            }
            $check = DB::table('tai_employee_department_mapping')->insert([
                'tenant_id' => $request->tenant_id,
                'employee_id' => $id,
                'department_id' => $request->department_id[$i],
                'ticket_passing_authority' => $request->ticket_passing_authority[$i],
                'ticket_show_type' => $request->ticket_show_type[$i],
                'is_default' => $request->is_default[$i]
            ]);
        }
        DB::table('tai_employee_master')->where('id', $id)->update(['department_id' => $defaultDepartment]);

        if ($result) {
            return response()->json(['status' => 1, 'message' => "Record Updated !!!", 'data' => $request->toArray()]);
        } else {
            return response()->json(['status' => 0, 'message' => "Record Not Updated !!!", 'data' => $request->toArray()]);
        }
    }

    public function deleteData($id)
    {
        $result = DB::table('tai_employee_master')->where('id', $id)->update(['is_active' => 0]);
        if ($result) {
            return response()->json(['message' => "Record Deleted", 'status' => 1, 'data' => null]);
        } else {
            return response()->json(['message' => "Record Not Deleted", 'status' => 0, 'data' => null]);
        }
    }
}
