<?php

namespace App\Http\Controllers;

use Illuminate\Http\Request;
use DB;
use App\Library\OwnershipController;
use JWTAuth;

class ProjectsController extends Controller
{
    public function getAllProject(Request $request)
    {
        $token_data = JWTAuth::parseToken()->authenticate(); //Date:30-05-2023 PUNAM SHINDE
        $tenantId = $token_data->tenant_id; //Date:30-05-2023 PUNAM SHINDE

        $result = DB::table('tai_project_master as p')
            ->leftJoin('tai_employee_master as e', 'e.id', '=', 'p.created_by')
            ->leftJoin('tai_employee_master as emplo', 'p.updated_by',  '=', 'emplo.id')
            ->leftJoin('tai_employee_master as em', 'em.id', '=', 'p.ba_id')
            ->leftJoin('tai_employee_master as emp', 'emp.id', '=', 'p.dev_id')
            ->leftJoin('tai_employee_master as empl', 'empl.id', '=', 'p.tester_id')
            // ->where('p.is_active', 1)
            ->where('p.tenant_id', $tenantId)
            ->select(
                'p.*',
                DB::raw("CONCAT(e.first_name,' ',e.last_name) AS created_by"),
                DB::raw("CONCAT(em.first_name,' ',em.last_name) AS assigned_ba"),
                DB::raw("CONCAT(emp.first_name,' ',emp.last_name) AS assigned_dev"),
                DB::raw("CONCAT(empl.first_name,' ',empl.last_name) AS assigned_tester"),
                DB::raw("CONCAT(emplo.first_name,' ',emplo.last_name) AS updated_by")
            )
            ->orderBy('p.id', 'DESC')->get();

        foreach ($result as $row) {
            $reviewer = DB::table('tai_owner_mapping_master as map')
                ->leftJoin('tai_employee_master as e', 'e.id', 'map.user_id')
                ->where('map.project_id', $row->id)
                ->where('map.type', "REVIEWER")
                ->where('map.is_active', 1)
                ->where('map.is_active', 1)
                // ->select('map.user_id', DB::raw("CONCAT(e.first_name,' ',e.last_name) AS employee_name"))
                ->pluck(DB::raw("CONCAT(e.first_name,' ',e.last_name) AS employee_name"))->toArray();
            $row->projectReviewer = implode(', ', $reviewer);
        }

        if (count($result) > 0) {
            return response()->json(['status' => 1, 'message' => null, 'data' => $result]);
        } else {
            return response()->json(['status' => 0, 'message' => "No Record found !!!", 'data' => null]);
        }
    }

    public function createProject(Request $request)
    {
        $token_data = JWTAuth::parseToken()->authenticate(); //Date:30-05-2023 PUNAM SHINDE
        $tenantId = $token_data->tenant_id; //Date:30-05-2023 PUNAM SHINDE

        if (!$request->project_name || !$request->description) {
            return response()->json(['status' => 0, 'message' => "Invalid Data !!!", 'data' => null]);
        }

        // return [count($request->project_reviewer)];
        if (count($request->project_reviewer) == 0) {
            return response()->json(['status' => 0, 'message' => "Please enter project reviewer !!!", 'data' => null]);
        }

        $check = DB::table('tai_project_master')
            ->where('project_name', $request->project_name)
            ->get();

        if (count($check) == 0) {
            $data = $request->toArray();
            $data['tenant_id'] = $tenantId;
            $projectOwner = $data['project_owner'];
            $projectReviewer = $data['project_reviewer'];

            unset($data['project_owner']);
            unset($data['project_reviewer']);
            unset($data['logo']);

            $fileName = '';
            if ($request->file('logo')) {
                $file = $request->file('logo');
                $path = 'Attachment/project/';
                $fileName = $file->getClientOriginalName();

                $isUploaded = $file->storeAs($path, $fileName);
            }
            $data['logo'] = $fileName;
            $data['tenant_id'] = $tenantId;

            $projectId = DB::table('tai_project_master')->insertGetId($data);

            if ($projectId) {

                // if (!empty($data['reviewer_id'])) {
                //     foreach ($data['reviewer_id'] as $r) {
                //         DB::table('tai_project_reviewer')->insert([
                //             'tenant_id' => $data['tenant_id'],
                //             'project_id' => $projectId,
                //             'reviewer_id' => $r,
                //             'created_at' => $data['created_at'],
                //             'created_by' => $data['created_by'],
                //             'updated_at' => $data['updated_at'],
                //             'updated_by' => $data['updated_by']
                //         ]);
                //     }
                // }


                if ($projectOwner) {

                    // project owner
                    if (is_array($projectOwner)) {
                        $projectOwner = implode(',', $projectOwner);
                    }

                    $result = OwnershipController::mapping('Insert', 'PROJECT', $projectOwner, $projectId, '', '', '', $tenantId);
                }
                // project reviewer
                if (is_array($projectReviewer)) {
                    $projectReviewer = implode(',', $projectReviewer);
                }
                $result = OwnershipController::mapping('Insert', 'REVIEWER', $projectReviewer, $projectId, '', '', '', $tenantId);

                return response()->json(['status' => 1, 'message' => "Project has been created successfully !!!", 'data' => null]);
            } else {
                return response()->json(['status' => 0, 'message' => "Project failed to create !!!", 'data' => null]);
            }
        } else {
            return response()->json(['status' => 0, 'message' => "Project already successfully !!!", 'data' => null]);
        }
    }

    public function updateProject(Request $request, $id)
    {
        $token_data = JWTAuth::parseToken()->authenticate(); //Date:30-05-2023 PUNAM SHINDE
        $tenantId = $token_data->tenant_id; //Date:30-05-2023 PUNAM SHINDE4

        $data = $request->toArray();
        $check = DB::table('tai_project_master as p')
            ->where('p.id', $id)
            ->where('p.tenant_id', $tenantId)->first();

        $projectOwner = $data['project_owner'];
        $projectReviewer = $data['project_reviewer'];

        unset($data['logo']);
        unset($data['project_owner']);
        unset($data['project_reviewer']);

        $data['logo'] = $check->logo;
        if ($request->file('logo')) {
            if ($check->logo != $request->file('logo')->getClientOriginalName()) {
                $file = $request->file('logo');
                $path = 'Attachment/project/';
                $fileName = $file->getClientOriginalName();
                $isUploaded = $file->storeAs($path, $fileName);
                $data['logo'] = $fileName;
            }
        }
        $result = DB::table('tai_project_master')->where('id', $id)->update([
            "project_name" => $data['project_name'],
            "description" => $data['description'],
            "is_active" => $data['is_active'],
            "updated_by" => $data['updated_by'],
            "logo" => $data['logo'],
            "updated_at" => date('Y-m-d H:i:s')
        ]);

        if ($result) {
            if ($projectOwner) {
                if (is_array($projectOwner)) {
                    $projectOwner = implode(',', $projectOwner);
                }
                $result = OwnershipController::mapping('Update', 'PROJECT', $projectOwner, $id, '', '', '', $tenantId);
            }

            // project reviewer
            if (is_array($projectReviewer)) {
                $projectReviewer = implode(',', $projectReviewer);
            }
            $result = OwnershipController::mapping('Update', 'REVIEWER', $projectReviewer, $id, '', '', '', $tenantId);

            return response()->json(['status' => 1, 'message' => "Record Updated !!!", 'data' => null]);
        } else {
            return response()->json(['status' => 1, 'message' => "Record Updated !!!", 'data' => null]);
        }
    }


    public function deleteProject($id)
    {
        $result = DB::table('tai_project_master')->where('id', $id)->update(['is_active' => 0]);
        if ($result) {
            return response()->json(['status' => 1, 'message' => "Record Deleted !!!", 'data' => null]);
        } else {
            return response()->json(['status' => 0, 'message' => "Record Not Deleted !!!", 'data' => null]);
        }
    }

    public function getProjectById($id)
    {
        $token_data = JWTAuth::parseToken()->authenticate(); //Date:30-05-2023 PUNAM SHINDE
        $tenantId = $token_data->tenant_id; //Date:30-05-2023 PUNAM SHINDE

        $result = DB::table('tai_project_master')->where('id', $id)->first();
        if ($result) {
            // $owners = DB::table('tai_owner_mapping_master')
            //     ->where('project_id', $id)
            //     ->where('type', "PROJECT")
            //     ->where('is_active', 1)
            //     ->pluck('user_id')
            //     ->toArray();

            $owners = DB::table('tai_owner_mapping_master as map')
                ->leftJoin('tai_employee_master as e', 'e.id', 'map.user_id')
                ->where('map.project_id', $id)
                ->where('map.type', "PROJECT")
                ->where('map.is_active', 1)
                ->where('map.is_active', 1)
                ->select('map.user_id', DB::raw("CONCAT(e.first_name,' ',e.last_name) AS employee_name"))
                ->get()
                ->toArray();

            $result->projectOwners = $owners;


            $reviewer = DB::table('tai_owner_mapping_master as map')
                ->leftJoin('tai_employee_master as e', 'e.id', 'map.user_id')
                ->where('map.project_id', $id)
                ->where('map.type', "REVIEWER")
                ->where('map.is_active', 1)
                ->where('map.is_active', 1)
                ->select('map.user_id', DB::raw("CONCAT(e.first_name,' ',e.last_name) AS employee_name"))
                ->get()
                ->toArray();

            $result->projectReviewer = $reviewer;
            return response()->json(['status' => 1, 'message' => "Record Found !!!", 'data' => $result]);
        } else {
            return response()->json(['status' => 0, 'message' => "No Record Found !!!", 'data' => null]);
        }
    }

    public function getClientData($tenantId)
    {
        $token_data = JWTAuth::parseToken()->authenticate(); //Date:30-05-2023 PUNAM SHINDE
        $tenantId = $token_data->tenant_id; //Date:30-05-2023 PUNAM SHINDE

        $result = DB::table('tai_customer_master')
            ->select('id', 'name')
            ->where('tenant_id', $tenantId)
            ->where('is_active', 1)
            ->orderBy('id', 'DESC')
            ->get();
        if (count($result) > 0) {
            return response()->json(['status' => 1, 'message' => "Record found !!!", 'data' => $result]);
        } else {
            return response()->json(['status' => 0, 'message' => "No Record found !!!", 'data' => null]);
        }
    }

    // punam shinde 2022-10-19
    public function addProjectMapping(Request $request)
    {
        $token_data = JWTAuth::parseToken()->authenticate(); //Date:30-05-2023 PUNAM SHINDE
        $tenantId = $token_data->tenant_id; //Date:30-05-2023 PUNAM SHINDE

        $getData = $request->toArray();
        if ($getData) {
            $data['created_at'] = $getData['created_at'];
            $data['created_by'] = $getData['created_by'];
            $data['tenant_id'] = $getData['tenant_id'];
            $data['project_id'] = $getData['project'];

            if ($getData['ba_id']) {
                $data['user_type'] = "BA";
                $data['user_id'] = $getData['ba_id'];
                $query = DB::table('tai_project_owner_mapping_master')->insert($data);
            }
            if ($getData['dev_id']) {
                $data['user_type'] = "DEV";
                $data['user_id'] = $getData['dev_id'];
                $query = DB::table('tai_project_owner_mapping_master')->insert($data);
            }
            if ($getData['tester_id']) {
                $data['user_type'] = "TESTER";
                $data['user_id'] = $getData['tester_id'];
                $query = DB::table('tai_project_owner_mapping_master')->insert($data);
            }
            return response()->json(['status' => 1, 'message' => "Record Added !!!", 'data' => $query]);
            // $query = DB::table('tai_project_owner_mapping_master')->insertGetId($data); 
            // if($query){
            // return response()->json([ 'status'=> 1, 'message'=> "Record Added !!!",'data' => $query]);
            // }else{
            // return response()->json([ 'status'=> 0, 'message'=> "No Record Added!!!",'data' => null]);
            // }        
        } else {
            return response()->json(['status' => 0, 'message' => "No Data Found!!!", 'data' => null]);
        }
    }

    public function getProjectReviewers($project_id)
    {
        $token_data = JWTAuth::parseToken()->authenticate(); //Date:30-05-2023 PUNAM SHINDE
        $tenantId = $token_data->tenant_id; //Date:30-05-2023 PUNAM SHINDE

        if ($project_id) {
            $data = DB::table('tai_project_master as m')
                ->join('tai_owner_mapping_master as map', 'map.project_id', 'm.id')
                ->join('tai_employee_master as e', 'e.id', 'map.user_id')
                ->DISTINCT()
                ->where('map.is_active', '1')
                ->where('m.is_active', 1)
                ->where('m.tenant_id', $tenantId)
                ->where('m.id', $project_id)
                ->where('map.type', 'REVIEWER')
                ->select('map.user_id', 'm.id as project_id', DB::raw("CONCAT(e.first_name,' ',e.last_name) AS employee_name"))
                ->get();

            if ($data) {
                return response()->json(['status' => 1, 'message' => "Data Found!!!", 'data' => $data]);
            } else {
                return response()->json(['status' => 0, 'message' => "Data Not Found!!!", 'data' => null]);
            }
        } else {
            return response()->json(['status' => 0, 'message' => "Project id Not Found!!!", 'data' => null]);
        }
    }
}
