<?php

namespace App\Http\Controllers;

use App\Models\Department;
use Illuminate\Http\Request;
use DB;
use JWTAuth;

class RoleController extends Controller
{
    public function getAllRole(Request $request)
    {
        $token_data = JWTAuth::parseToken()->authenticate();
        $tenantId = $token_data->tenant_id;

        $result = DB::table('tai_role_master as r')
            ->leftJoin('tai_employee_master as e', 'e.id', '=', 'r.updated_by')
            ->leftJoin('tai_employee_master as ec', 'ec.id', '=', 'r.created_by')
            ->where('r.tenant_id', $tenantId)
            ->select('r.*', DB::raw("CONCAT(e.first_name,' ',e.last_name) AS updated_by"), DB::raw("CONCAT(ec.first_name,' ',ec.last_name) AS created_by"))
            ->orderBy('r.is_active', 'DESC')
            ->orderBy('r.id', 'DESC')
            ->get();
            
        if (count($result) > 0) {
            return response()->json(['status' => 1, 'message' => "Record found !!!", 'data' => $result]);
        } else {
            return response()->json(['status' => 0, 'message' => "No Record found !!!", 'data' => null]);
        }
    }

    public function createRole(Request $request)
    {
        $token_data = JWTAuth::parseToken()->authenticate();
        $tenantId = $token_data->tenant_id;

        if (!$request->role) {
            return response()->json(['status' => 0, 'message' => "Invalid Data !!!", 'data' => null]);
        }
        $data = $request->toArray();
        $data['tenant_id'] = $tenantId;

        $check = DB::table('tai_role_master')->where('role', $request->role)->first();

        if ($check) {
            return response()->json(['status' => 0, 'message' => "Record Already Exists !!!", 'data' => null]);
        } else {
            $result = DB::table('tai_role_master')->insertGetId($data);
            if ($result > 0) {
                return response()->json(['status' => 1, 'message' => "Record Inserted !!!", 'data' => $result]);
            } else {
                return response()->json(['status' => 0, 'message' => "Record Not Inserted !!!", 'data' => null]);
            }
        }
    }

    public function updateRole(Request $request, $id)
    {
        $token_data = JWTAuth::parseToken()->authenticate();
        $tenantId = $token_data->tenant_id;

        $data = $request->toArray();
        $result = DB::table('tai_role_master')->where('role', $request->role)->where('tenant_id', $tenantId)->where('id','!=',$id)->first();

        if ($result) {
            return response()->json(['status' => 0, 'message' => "Role already exist !!!", 'data' => null]);
        }
        else{
            $result = DB::table('tai_role_master')->where('id', $id)->update($data);
            if ($result) {
                return response()->json(['status' => 1, 'message' => "Record Updated !!!", 'data' => null]);
            } else {
                return response()->json(['status' => 0, 'message' => "Record Not Updated !!!", 'data' => null]);
            }
        }
    }

    public function deleteRole($id)
    {
        $token_data = JWTAuth::parseToken()->authenticate();
        $tenantId = $token_data->tenant_id;

        $result = DB::table('tai_role_master')->where('id', $id)->update(['is_active' => 0]);
        if ($result) {
            return response()->json(['status' => 1, 'message' => "Record Deleted !!!", 'data' => null]);
        } else {
            return response()->json(['status' => 0, 'message' => "Record Not Deleted !!!", 'data' => null]);
        }
    }

    public function getRoleById($id)
    {
        $token_data = JWTAuth::parseToken()->authenticate();
        $tenantId = $token_data->tenant_id;

        $result = DB::table('tai_role_master')->where('id', $id)->first();
        if ($result) {
            return response()->json(['status' => 1, 'message' => "Record Found !!!", 'data' => $result]);
        } else {
            return response()->json(['status' => 0, 'message' => "No Record Found !!!", 'data' => null]);
        }
    }
}
