<?php
// live
namespace App\Http\Controllers;

use App\Models\Department;
use Illuminate\Http\Request;
use DB;
use JWTAuth;

class StateController extends Controller
{
    public function getAllState(Request $request, $type = null, $sorted = null)
    {
        $token_data = JWTAuth::parseToken()->authenticate(); //added by PUNAM 18-05-2023

        $result = DB::table('tai_state_master as s')
            ->leftJoin('tai_country_master as c', 'c.id', '=', 's.country_id')
            ->leftJoin('tai_employee_master as e', 'e.id', '=', 's.updated_by')
            ->leftJoin('tai_employee_master as ec', 'ec.id', '=', 's.created_by')
            ->select('s.*', 'c.country', DB::raw("CONCAT(e.first_name,' ',e.last_name) AS updated_by"), DB::raw("CONCAT(ec.first_name,' ',ec.last_name) AS created_by"))
            // ->where('s.tenant_id', $tenantId);
            ->where('s.tenant_id', $token_data->tenant_id);

        if ($sorted != null) {
            $result = $result->orderby('s.state')
                ->orderByDesc('s.is_active')
                ->orderBy('s.id', 'DESC')->get();
        } else {
            $result = $result->orderByDesc('s.is_active')
                ->orderBy('s.id', 'DESC');
            if ($type != null) {
                $result = $result->where('s.is_active', 1)->get();
            } else {
                $result = $result->get();
            }
        }

        if (count($result) > 0) {
            return response()->json(['status' => 1, 'message' => "Record found !!!", 'data' => $result]);
        } else {
            return response()->json(['status' => 0, 'message' => "No Record found !!!", 'data' => null]);
        }
    }



    public function createState(Request $request)
    {
        $token_data = JWTAuth::parseToken()->authenticate(); //added by PUNAM 18-05-2023

        if (!$request->country_id && !$request->state) {
            return response()->json(['status' => 0, 'message' => "Required Country & State !!!", 'data' => null]);
        }
        $data = $request->toArray();
        $check = DB::table('tai_state_master')
            ->where('tenant_id', $token_data->tenant_id)
            ->where('country_id', $request->country_id)
            ->where('state', $request->state)
            ->get();

        if (count($check) == 0) {
            $result = DB::table('tai_state_master')->insert($data);
            if ($result > 0) {
                return response()->json(['status' => 1, 'message' => "Record Inserted !!!", 'data' => null]);
            } else {
                return response()->json(['status' => 0, 'message' => "Record Not Inserted !!!", 'data' => null]);
            }
        } else {
            return response()->json(['status' => 0, 'message' => "Record Already Exists !!!", 'data' => null]);
        }
    }

    public function updateState(Request $request, $id)
    {
        $token_data = JWTAuth::parseToken()->authenticate(); //added by PUNAM 18-05-2023

        $check = DB::table('tai_state_master')->where('tenant_id', $token_data->tenant_id)->where('country_id', $request->country_id)->where('state', $request->state)->first();
        if ($check && $check->id != $id) {
            return response()->json(['status' => 0, 'message' => "Record Already Exists !!!", 'data' => null]);
        }

        $data = $request->toArray();
        $data['updated_at'] = date('Y-m-d H:i:s');
        $prevCheck = DB::table('tai_state_master')->where('tenant_id', $token_data->tenant_id)->where('id', $id)->first();
        $result = DB::table('tai_state_master')->where('tenant_id', $token_data->tenant_id)->where('id', $id)->update($data);
        if ($result) {
            if ($prevCheck->is_active != $data['is_active']) {
                DB::table('tai_city_master')
                    ->where('tenant_id', $token_data->tenant_id)
                    ->where('state_id', $id)->update([
                    'is_active' => $data['is_active'],
                    'updated_by' => $data['updated_by'],
                    'updated_at' => $data['updated_at']
                ]);
            }

            return response()->json(['status' => 1, 'message' => "Record Updated !!!", 'data' => null]);
        } else {
            return response()->json(['status' => 0, 'message' => "Record Not Updated !!!", 'data' => null]);
        }
    }

    public function deleteState($id)
    {
        $token_data = JWTAuth::parseToken()->authenticate(); //added by PUNAM 18-05-2023

        $result = DB::table('tai_state_master')->where('tenant_id', $token_data->tenant_id)->where('id', $id)->update(['is_active' => 0]);
        if ($result) {
            return response()->json(['status' => 1, 'message' => "Record Deleted !!!", 'data' => null]);
        } else {
            return response()->json(['status' => 0, 'message' => "Record Not Deleted !!!", 'data' => null]);
        }
    }

    public function getStateById($id)
    {
        $token_data = JWTAuth::parseToken()->authenticate(); //added by PUNAM 18-05-2023

        $result = DB::table('tai_state_master')->where('tenant_id', $token_data->tenant_id)->where('id', $id)->first();
        if ($result) {
            return response()->json(['status' => 1, 'message' => "Record Found !!!", 'data' => $result]);
        } else {
            return response()->json(['status' => 0, 'message' => "No Record Found !!!", 'data' => null]);
        }
    }
}
