<?php

namespace App\Http\Controllers;

use App\Models\Department;
use Illuminate\Http\Request;
use DB;
use JWTAuth;

class StatusController extends Controller
{
    public function getAllStatus(Request $request)
    {
        $token_data = JWTAuth::parseToken()->authenticate();
        $tenantId = $token_data->tenant_id;

        $query = DB::table('tai_status_master as s')
        ->leftJoin('tai_employee_master as ec', 'ec.id', '=', 's.created_by')
        ->leftJoin('tai_employee_master as e', 'e.id', '=', 's.updated_by');

        if ($request->header('X-Type')) {
            $query->select('s.id', 's.status');
        } else {
            $query->where('s.tenant_id', $tenantId);
            $query->select('s.*', DB::raw("CONCAT(e.first_name,' ',e.last_name) AS updated_by"), DB::raw("CONCAT(ec.first_name,' ',ec.last_name) AS created_by"));
        }

        $result = $query->orderBy('s.is_active', 'DESC')->orderBy('s.id', 'DESC')
            ->get();
        if (count($result) > 0) {
            return response()->json(['status' => 1, 'message' => "Record found !!!", 'data' => $result]);
        } else {
            return response()->json(['status' => 0, 'message' => "No Record found !!!", 'data' => null]);
        }
    }

    public function createStatus(Request $request)
    {
        $token_data = JWTAuth::parseToken()->authenticate();
        $tenantId = $token_data->tenant_id;

        if (!$request->status) {
            return response()->json(['status' => 0, 'message' => "Invalid Data !!!", 'data' => null]);
        }
        $data = $request->toArray();
        $data['tenant_id'] = $tenantId;

        $check = DB::table('tai_status_master')
            ->where('status', $request->status)
            ->get();
        if (count($check) == 0) {
            $result = DB::table('tai_status_master')->insert($data);
            if ($result > 0) {
                return response()->json(['status' => 1, 'message' => "Record Inserted !!!", 'data' => null]);
            } else {
                return response()->json(['status' => 0, 'message' => "Record Not Inserted !!!", 'data' => null]);
            }
        } else {
            return response()->json(['status' => 0, 'message' => "Record Already Exists !!!", 'data' => null]);
        }
    }

    public function updateStatus(Request $request, $id)
    {
        $token_data = JWTAuth::parseToken()->authenticate();
        $tenantId = $token_data->tenant_id;

        $data = $request->toArray();
        $already = DB::table('tai_status_master')->where('status', $data['status'])->where('id', '!=', $id)->first();
        if (!$already) {
            $result = DB::table('tai_status_master')->where('tenant_id', $tenantId)->where('id', $id)->update($data);
            if ($result) {
                return response()->json(['status' => 1, 'message' => "Record Updated !!!", 'data' => null]);
            } else {
                return response()->json(['status' => 0, 'message' => "Record Not Updated !!!", 'data' => null]);
            }
        } else {
            return response()->json(['status' => 0, 'message' => "status already exist !!!", 'data' => null]);
        }
    }

    public function deletestatus($id)
    {
        $token_data = JWTAuth::parseToken()->authenticate();
        $tenantId = $token_data->tenant_id;

        $result = DB::table('tai_status_master')->where('tenant_id', $tenantId)->where('id', $id)->update(['is_active' => 0]);
        if ($result) {
            return response()->json(['status' => 1, 'message' => "Record Deleted !!!", 'data' => null]);
        } else {
            return response()->json(['status' => 0, 'message' => "Record Not Deleted !!!", 'data' => null]);
        }
    }

    public function getStatusById($id)
    {
        $token_data = JWTAuth::parseToken()->authenticate();
        $tenantId = $token_data->tenant_id;

        $result = DB::table('tai_status_master')->where('tenant_id', $tenantId)->where('id', $id)->first();
        if ($result) {
            return response()->json(['status' => 1, 'message' => "Record Found !!!", 'data' => $result]);
        } else {
            return response()->json(['status' => 0, 'message' => "No Record Found !!!", 'data' => null]);
        }
    }
}
