<?php

namespace App\Http\Controllers;

use App\Models\Department;
use Illuminate\Http\Request;
use DB;
use JWTAuth;

class TestingTypeMasterController extends Controller
{
    public function getAllTestingType(Request $request, $type = null, $sorted = null)
    {
        $token_data = JWTAuth::parseToken()->authenticate(); //added by PUNAM 26-05-2023
        $tenantId = $token_data->tenant_id;

        $result = DB::table('tai_testing_type_master as s')
            ->leftJoin('tai_employee_master as e', 'e.id', '=', 's.updated_by')
            ->leftJoin('tai_employee_master as ec', 'ec.id', '=', 's.created_by')
            ->select('s.*', DB::raw("CONCAT(e.first_name,' ',e.last_name) AS updated_by"), DB::raw("CONCAT(ec.first_name,' ',ec.last_name) AS created_by"))
            ->where('s.tenant_id', $tenantId);
        if ($sorted != null) {
            $result = $result->orderby('s.testing_type')
                ->orderByDesc('s.is_active')
                ->orderBy('s.id', 'DESC')->get();
        } else {
            $result = $result->orderByDesc('s.is_active')
                ->orderBy('s.id', 'DESC');
            if ($type != null) {
                $result = $result->where('s.is_active', 1)->get();
            } else {
                $result = $result->get();
            }
        }
        
        if (count($result) > 0) {
            return response()->json(['status' => 1, 'message' => "Record found !!!", 'data' => $result]);
        } else {
            return response()->json(['status' => 0, 'message' => "No Record found !!!", 'data' => null]);
        }
    }



    public function createTestingType(Request $request)
    {
        $token_data = JWTAuth::parseToken()->authenticate(); //added by PUNAM 26-05-2023
        $tenantId = $token_data->tenant_id;

        if (!$request->testing_type) {
            return response()->json(['status' => 0, 'message' => "Required Testing Type name !!!", 'data' => null]);
        }
        $data = $request->toArray();
        $check = DB::table('tai_testing_type_master')
            ->where('testing_type', $request->testing_type)
            ->get();

        if (count($check) == 0) {
            $result = DB::table('tai_testing_type_master')->insert($data);
            if ($result > 0) {
                return response()->json(['status' => 1, 'message' => "Record Inserted !!!", 'data' => null]);
            } else {
                return response()->json(['status' => 0, 'message' => "Record Not Inserted !!!", 'data' => null]);
            }
        } else {
            return response()->json(['status' => 0, 'message' => "Record Already Exists !!!", 'data' => null]);
        }
    }

    public function updateTestingType(Request $request, $id)
    {
        $token_data = JWTAuth::parseToken()->authenticate(); //added by PUNAM 26-05-2023
        $tenantId = $token_data->tenant_id;

        $check = DB::table('tai_testing_type_master')->where('testing_type', $request->testing_type)->first();
        if ($check && $check->id != $id) {
            return response()->json(['status' => 0, 'message' => "Record Already Exists !!!", 'data' => null]);
        }

        $data = $request->toArray();
        $data['updated_at'] = date('Y-m-d H:i:s');
        $prevCheck = DB::table('tai_testing_type_master')->where('tenant_id', $token_data->tenant_id)->where('id', $id)->first();
        $result = DB::table('tai_testing_type_master')->where('tenant_id', $token_data->tenant_id)->where('id', $id)->update($data);
        if ($result) {
            return response()->json(['status' => 1, 'message' => "Record Updated !!!", 'data' => null]);
        } else {
            return response()->json(['status' => 0, 'message' => "Record Not Updated !!!", 'data' => null]);
        }
    }

    public function deleteTestingType($id)
    {
        $token_data = JWTAuth::parseToken()->authenticate(); //added by PUNAM 26-05-2023
        $tenantId = $token_data->tenant_id;

        $result = DB::table('tai_testing_type_master')->where('tenant_id', $token_data->tenant_id)->where('id', $id)->update(['is_active' => 0]);
        if ($result) {
            return response()->json(['status' => 1, 'message' => "Record Deleted !!!", 'data' => null]);
        } else {
            return response()->json(['status' => 0, 'message' => "Record Not Deleted !!!", 'data' => null]);
        }
    }

    public function getTestingTypeById($id)
    {
        $token_data = JWTAuth::parseToken()->authenticate(); //added by PUNAM 26-05-2023
        $tenantId = $token_data->tenant_id;

        $result = DB::table('tai_testing_type_master')->where('tenant_id', $token_data->tenant_id)->where('id', $id)->first();
        if ($result) {
            return response()->json(['status' => 1, 'message' => "Record Found !!!", 'data' => $result]);
        } else {
            return response()->json(['status' => 0, 'message' => "No Record Found !!!", 'data' => null]);
        }
    }
}
