<?php

namespace App\Http\Controllers;

use Illuminate\Http\Request;
use DB;
use File;
use App\Library\CommonController;
use JWTAuth;


class VendorMasterController extends Controller
{

    public function getVendorMaster($user_id,  $type = null)
    {
        $token_data = JWTAuth::parseToken()->authenticate();
        $tenantId = $token_data->tenant_id;

        // dd($tenantId, $type);
        $data = DB::table('tai_bc_vendor_master as tvm')
            // ->select('tvm.*')
            ->leftJoin('tai_country_master as tc', 'tvm.country', 'tc.id')
            ->leftJoin('tai_state_master as ts', 'tvm.state', 'ts.id')
            ->leftJoin('tai_city_master as tcm', 'tvm.city', 'tcm.id')
            ->leftJoin('tai_bc_payment_template as pt', 'tvm.payment_template', 'pt.id')
            ->leftJoin('tai_employee_master as e', 'e.id', '=', 'tvm.updated_by')
            ->leftJoin('tai_employee_master as em', 'em.id', '=', 'tvm.created_by')
            ->where('tvm.tenant_id', $tenantId)

            ->select(
                'tvm.*',
                'tc.country as country_name',
                'ts.state as state_name',
                'tcm.city as city_name',
                'pt.template_name',
                DB::raw("CONCAT(em.first_name,' ',em.last_name) AS created_by_name"),
                DB::raw("CONCAT(e.first_name,' ',e.last_name) AS updated_by_name")
            );

        // // authority    
        // $access['Bank_Details'] = $this->checkAuthorityAccess($user_id, 'Bill_Checking','Bank_Details');      
        // $access['Update_ERP_Account_Name'] = $this->checkAuthorityAccess($user_id, 'Bill_Payment','Update_ERP_Account_Name');      

        if ($type != null) {
            $data = $data->where('tvm.is_active', 1)->get();
        } else {
            $data = $data->get();
        }
        return response()->json(['status' => 1, 'message' =>  "Found Data !!!", 'data' => $data]);
    }

    public function getVendorMasterById($id)
    {
        $token_data = JWTAuth::parseToken()->authenticate();
        $tenantId = $token_data->tenant_id;

        if ($id) {
            $data = DB::table('tai_bc_vendor_master as tvm')->where('tvm.id', $id)->where('tenant_id', $tenantId)->first();
            // $data = DB::table('tai_bc_vendor_master as tvm')
            //             // ->select('tvm.*')
            //             ->select('tvm.id','tvm.tenant_id','tvm.vendor_name','tvm.address','tc.country','ts.state','tcm.city', 'tvm.pincode', 'tvm.mobile_no',
            //                 'tvm.email', 'tvm.adhar_no', 'tvm.adhar_attachment', 'tvm.pan_no', 'tvm.pan_attachment', 'tvm.msme_no', 'tvm.msme_attachment',
            //                 'tvm.gst_no', 'tvm.gst_attachment', 'tvm.consider_in_payment', 'tvm.bank_name', 'tvm.account_no', 'tvm.ifsc_code', 'tvm.bank_passbook_attachment',
            //                 'tvm.cheque_attachment', 'pt.template_name', 'tvm.is_active', 'tvm.remark', 'tvm.created_at', 'tvm.created_by', 'tvm.updated_at', 'tvm.updated_by')
            //             ->leftJoin('tai_country_master as tc','tvm.country','tc.id')
            //             ->leftJoin('tai_state_master as ts','tvm.state','ts.id')
            //             ->leftJoin('tai_city_master as tcm','tvm.city','tcm.id')
            //             ->leftJoin('tai_bc_payment_template as pt','tvm.payment_template','pt.id')
            //             ->where('tvm.id',$id)
            //             ->first();

            return response()->json(['status' => 1, 'message' =>  "Found Data !!!", 'data' => $data]);
        } else {
            return response()->json(['status' => 0, 'message' =>  "ID Not Found !!!", 'data' => $data]);
        }
    }

    public function createVendorMaster(Request $request, $id = '')
    {
        $token_data = JWTAuth::parseToken()->authenticate();
        $tenantId = $token_data->tenant_id;

        // attachment extensions
        $extensions = ['jpg', 'pdf', 'jpeg', 'png'];

        $result = $request->input();
        // dd($result);

        // validation 
        if (!$request->vendor_name) {
            return response()->json(['status' => 0, 'message' => "Required Vendor Name !!!", 'data' => null]);
        }
        if (!$request->mobile_no) {
            return response()->json(['status' => 0, 'message' => "Required Mobile Number !!!", 'data' => null]);
        }
        if (!$request->email) {
            return response()->json(['status' => 0, 'message' => "Required Email Id !!!", 'data' => null]);
        }
        // if(!$request->adhar_no){ return response()->json([ 'status'=> 0, 'message'=> "Required Aadhar Number !!!", 'data' => null]); }
        if (!$request->pan_no) {
            return response()->json(['status' => 0, 'message' => "Required PAN Number !!!", 'data' => null]);
        }
        // if(!$request->gst_no){ return response()->json([ 'status'=> 0, 'message'=> "Required GST Number !!!", 'data' => null]); }


        // mobile validation
        // dd($this->validateMobileNo);
        if (!$request->payment_template) {
            return response()->json(['status' => 0, 'message' => "Payment Template Required !!!", 'data' => Null]);
        }
        if ($this->validateMobileNo($request->mobile_no) == 0) {
            return response()->json(['status' => 0, 'message' => "Invalid Mobile Number !!!", 'data' => Null]);
        }

        // email validation
        if (!preg_match("/^([a-z0-9\+_\-]+)(\.[a-z0-9\+_\-]+)*@([a-z0-9\-]+\.)+[a-z]{2,6}$/ix", $request->email)) {
            return response()->json(['status' => 0, 'message' => "Invalid Email Address !!!", 'data' => Null]);
        }

        // check aadhar number
        if ($request->adhar_no) {
            if (!preg_match("/^[2-9]{1}[0-9]{3}[0-9]{4}[0-9]{4}$/", $request->adhar_no)) {
                return response()->json(['status' => 0, 'message' => "Invalid Aadhar Number !!!", 'data' => Null]);
            }
        }


        // PAN NUmber Validatiob
        if ($request->pan_no != 'NA') {
            if (!preg_match("/^([a-zA-Z]){5}([0-9]){4}([a-zA-Z]){1}?$/", $request->pan_no)) {
                return response()->json(['status' => 0, 'message' => "Invalid PAN Number !!!", 'data' => Null]);
            }
        }

        // Pincode NUmber Validatiob
        // if (!preg_match("/^[1-9]{1}[0-9]{2}{0, 1}[0-9]{3}$/", $request->pan_no)) {
        //     return response()->json([ 'status'=> 0, 'message'=> "Invalid Pincode !!!",'data' => Null]);
        // }


        // GST Number validation
        if ($request->gst_no) {
            if (!preg_match("/^[0-9]{2}[A-Z]{5}[0-9]{4}[A-Z]{1}[1-9A-Z]{1}Z[0-9A-Z]{1}$/", $request->gst_no)) {
                return response()->json(['status' => 0, 'message' => "Invalid GST Number !!!", 'data' => Null]);
            }
        }


        // check vendor name 
        $check = DB::table('tai_bc_vendor_master');

        if ($id) {
            $check_name = $check->where('id', '!=', $id)->where('vendor_name', $request->vendor_name)->first();
            if ($check_name) {
                return response()->json(['status' => 0, 'message' => "Vendor Name Already Exist !!!", 'data' => Null]);
            }

            $check_adhar = DB::table('tai_bc_vendor_master')->where('id', '!=', $id)->where('adhar_no', $request->adhar_no)->first();
            if ($check_adhar) {
                return response()->json(['status' => 0, 'message' => "Adhar Number Already Exist !!!", 'data' => Null]);
            }
        } else {
            $check_name = $check->where('vendor_name', $request->vendor_name)->first();
            if ($check_name) {
                return response()->json(['status' => 0, 'message' => "Vendor Name Already Exist !!!", 'data' => Null]);
            }

            if ($request->adhar_no != "") {
                $check_adhar = DB::table('tai_bc_vendor_master')->where('adhar_no', $request->adhar_no)->first();
                // print_r($check_adhar);
                if ($check_adhar) {
                    return response()->json(['status' => 0, 'message' => "Aadhar Number Already Exist !!!", 'data' => null]);
                }
            }

            if ($request->pan_no && $request->pan_no != 'NA') {
                $check_pan = DB::table('tai_bc_vendor_master')->where('pan_no', $request->pan_no)->first();
                if ($check_pan) {
                    return response()->json(['status' => 0, 'message' => "Pan Number Already Exist !!!", 'data' => Null]);
                }
            }


            if ($request->gst_no) {
                $check_pan = DB::table('tai_bc_vendor_master')->where('gst_no', $request->gst_no)->first();
                if ($check_pan) {
                    return response()->json(['status' => 0, 'message' => "GST Number Already Exist !!!", 'data' => Null]);
                }
            }
        }

        //Check Extension of uploaded files;
        // dd(!in_array($request->file('adhar_attachment')->getClientOriginalExtension(),$extensions));
        // if($request->file('adhar_attachment') && !in_array($request->file('adhar_attachment')->getClientOriginalExtension(),$extensions)){
        //     return response()->json([ 'status'=> 0, 'message'=>  "Invalid File Format !!!", 'data' => null]);
        // }
        // if($request->file('pan_attachment') && !in_array($request->file('pan_attachment')->getClientOriginalExtension(),$extensions)){
        //     return response()->json([ 'status'=> 0, 'message'=>  "Invalid File Format !!!",'data' => null]);
        // }
        // elseif($request->file('msme_attachment') && !in_array($request->file('msme_attachment')->getClientOriginalExtension(),$extensions)){
        //     return response()->json([ 'status'=> 0, 'message'=>  "Invalid File Format !!!",'data' => null]);
        // }
        // // elseif($request->file('gst_attachment') && !in_array($request->file('gst_attachment')->getClientOriginalExtension(),$extensions)){
        // //     return response()->json([ 'status'=> 0, 'message'=>  "Invalid File Format !!!",'data' => null]);
        // // }
        // elseif($request->file('bank_passbook_attachment') && !in_array($request->file('bank_passbook_attachment')->getClientOriginalExtension(),$extensions)){
        //     return response()->json([ 'status'=> 0, 'message'=>  "Invalid File Format !!!",'data' => null]);
        // }
        // elseif($request->file('cheque_attachment') && !in_array($request->file('cheque_attachment')->getClientOriginalExtension(),$extensions)){
        //     return response()->json([ 'status'=> 0, 'message'=>  "Invalid File Format !!!",'data' => null]);
        // }

        if ($id) {
            //Update Files
            if ($request->file('adhar_attachment')) {
                $result['adhar_attachment'] = $this->uploadAttachment($request->file('adhar_attachment'), 'ADHAR');
            }
            if ($request->file('pan_attachment')) {
                $result['pan_attachment'] = $this->uploadAttachment($request->file('pan_attachment'), 'PAN');
            }
            if ($request->file('msme_attachment')) {
                $result['msme_attachment'] = $this->uploadAttachment($request->file('msme_attachment'), 'MSME');
            }
            if ($request->file('gst_attachment')) {
                $result['gst_attachment'] = $this->uploadAttachment($request->file('gst_attachment'), 'GST');
            }
            if ($request->file('bank_passbook_attachment')) {
                $result['bank_passbook_attachment'] = $this->uploadAttachment($request->file('bank_passbook_attachment'), 'PASSBOOK');
            }
            if ($request->file('cheque_attachment')) {
                $result['cheque_attachment'] = $this->uploadAttachment($request->file('cheque_attachment'), 'CHEQUE');
            }

            $result['updated_at'] = $request->updated_at ? $request->updated_at : now();
            $result['updated_by'] = $request->updated_by ? $request->updated_by : 0;

            $data = DB::table('tai_bc_vendor_master')->where('id', $id)->update($result);
            if ($data) {
                $table = 'tai_bc_vendor_master';
                $result = (array) DB::table('tai_bc_vendor_master')->where('id', $id)->first();
                CommonController::generateHistory("UPDATE", $table, $id);

                return response()->json(['status' => 1, 'message' => "Record Updated !!!", 'data' => $id]);
            } else {
                return response()->json(['status' => 0, 'message' => "Record Not Updated !!!", 'data' => null]);
            }
        } else {

            if ($request->file('adhar_attachment') && $request->file('pan_attachment') &&  $request->file('gst_attachment') && $request->file('bank_passbook_attachment') && $request->file('cheque_attachment')) {
                $result['adhar_attachment'] = $this->uploadAttachment($request->file('adhar_attachment'), 'ADHAR');
                $result['pan_attachment'] = $this->uploadAttachment($request->file('pan_attachment'), 'PAN');
                $result['msme_attachment'] = $this->uploadAttachment($request->file('msme_attachment'), 'MSME');
                $result['gst_attachment'] = $this->uploadAttachment($request->file('gst_attachment'), 'GST');
                $result['bank_passbook_attachment'] = $this->uploadAttachment($request->file('bank_passbook_attachment'), 'PASSBOOK');
                $result['cheque_attachment'] = $this->uploadAttachment($request->file('cheque_attachment'), 'CHEQUE');
            }
            // else{
            //     return response()->json([ 'status'=> 0, 'message'=> "Attachments Are Required !!!",'data' => NULL]);
            // }

            $result['created_at'] = $request->created_at ? $request->created_at : now();
            $result['created_by'] = $request->created_by ? $request->created_by : 0;

            // dd($result);
            // insert data
            $id = DB::table('tai_bc_vendor_master')->insertGetId($result);

            if ($id) {
                $table = 'tai_bc_vendor_master';
                $result = (array) DB::table('tai_bc_vendor_master')->where('id', $id)->first();
                CommonController::generateHistory('Insert', $table, $id);

                return response()->json(['status' => 1, 'message' => "Record Added !!!", 'data' => $id]);
            } else {


                return response()->json(['status' => 0, 'message' => "Record Not Added !!!", 'data' => null]);
            }
        }
    }

    // upload file 
    public function uploadAttachment($attachment, $name)
    {

        if ($attachment) {

            $fileName = time() . $attachment->getClientOriginalName();

            $file = 'storage/app/Attachment/' . 'BillChecking/Vendor/' . $name . '/';
            $path = storage_path() . '/app/Attachment/BillChecking/Vendor/' . $name . '/';

            // $base = 'http://10.2.11.2/connectus-dummy/storage/app/car-booking/start_km/' . $fileName;
            // dd($file, $base);
            // $attachment->move($file, $fileName);

            $base = $file . $fileName;

            $attachment->move($path, $fileName);
            return $base;
        }
    }

    // mobile validation
    public function validateMobileNo($mobile)
    {
        if (preg_match('/^[1-9]\d{9}$/', $mobile)) {
            return 1;
        } else {
            return 0;
        }
    }


    // // get vendor master history by Id
    public function getVendorMasterHistoryById(Request $request, $id)
    {
        $token_data = JWTAuth::parseToken()->authenticate();
        $tenantId = $token_data->tenant_id;

        $data = DB::table('tai_bc_vendor_master_history as tvm')
            // ->select('tvm.*')
            ->select(
                'tvm.id',
                'tvm.operation',
                'tvm.main_id',
                'tvm.tenant_id',
                'tvm.vendor_name',
                'tvm.address',
                'tc.country',
                'ts.state',
                'tcm.city',
                'tvm.pincode',
                'tvm.mobile_no',
                'tvm.email',
                'tvm.adhar_no',
                'tvm.adhar_attachment',
                'tvm.pan_no',
                'tvm.pan_attachment',
                'tvm.msme_no',
                'tvm.msme_attachment',
                'tvm.gst_no',
                'tvm.gst_attachment',
                'tvm.consider_in_payment',
                'tvm.bank_name',
                'tvm.account_no',
                'tvm.ifsc_code',
                'tvm.bank_passbook_attachment',
                'tvm.cheque_attachment',
                'pt.template_name',
                'tvm.is_active',
                'tvm.remark',
                'tvm.created_at',
                'tvm.updated_at',
                DB::raw("CONCAT(em.first_name,' ',em.last_name) AS created_by"),
                DB::raw("CONCAT(e.first_name,' ',e.last_name) AS updated_by")
            )
            ->leftJoin('tai_country_master as tc', 'tvm.country', 'tc.id')
            ->leftJoin('tai_state_master as ts', 'tvm.state', 'ts.id')
            ->leftJoin('tai_city_master as tcm', 'tvm.city', 'tcm.id')
            ->leftJoin('tai_bc_payment_template as pt', 'tvm.payment_template', 'pt.id')
            ->leftJoin('tai_employee_master as e', 'e.id', '=', 'tvm.updated_by')
            ->leftJoin('tai_employee_master as em', 'em.id', '=', 'tvm.created_by')
            ->where('tvm.id', $id)
            ->get();


        // $data = DB::table('tai_bc_vendor_master_history')->where('id',$id)->get();
        return response()->json(['status' => 1, 'message' => "Found Data", 'data' => $data]);
    }

    // get vendor master history
    public function getVendorMasterHistory(Request $request)
    {
        $data = DB::table('tai_bc_vendor_master_history as tvm')
            // ->select('tvm.*')
            ->select(
                'tvm.id',
                'tvm.operation',
                'tvm.main_id',
                'tvm.tenant_id',
                'tvm.vendor_name',
                'tvm.address',
                'tc.country',
                'ts.state',
                'tcm.city',
                'tvm.pincode',
                'tvm.mobile_no',
                'tvm.email',
                'tvm.adhar_no',
                'tvm.adhar_attachment',
                'tvm.pan_no',
                'tvm.pan_attachment',
                'tvm.msme_no',
                'tvm.msme_attachment',
                'tvm.gst_no',
                'tvm.gst_attachment',
                'tvm.consider_in_payment',
                'tvm.bank_name',
                'tvm.account_no',
                'tvm.ifsc_code',
                'tvm.bank_passbook_attachment',
                'tvm.cheque_attachment',
                'pt.template_name',
                'tvm.is_active',
                'tvm.remark',
                'tvm.created_at',
                'tvm.updated_at',
                DB::raw("CONCAT(em.first_name,' ',em.last_name) AS created_by"),
                DB::raw("CONCAT(e.first_name,' ',e.last_name) AS updated_by")
            )
            ->leftJoin('tai_country_master as tc', 'tvm.country', 'tc.id')
            ->leftJoin('tai_state_master as ts', 'tvm.state', 'ts.id')
            ->leftJoin('tai_city_master as tcm', 'tvm.city', 'tcm.id')
            ->leftJoin('tai_bc_payment_template as pt', 'tvm.payment_template', 'pt.id')
            ->leftJoin('tai_employee_master as e', 'e.id', '=', 'tvm.updated_by')
            ->leftJoin('tai_employee_master as em', 'em.id', '=', 'tvm.created_by')
            ->get();

        return response()->json(['status' => 1, 'message' => "Found Data", 'data' => $data]);
    }

    // Authoruity for update vendor master
    public function checkAuthorityAccess($auth_id = null, $module_name, $auth_name = null)
    {
        $query = DB::table('tai_module_setting')->where('setting_name', $auth_name)
            ->where('field_value', 'Y')
            ->where('setting_type', 'U')
            ->where('module_name', $module_name)
            ->where('is_active', '1')->first();
        if ($query) {
            $check = DB::table('tai_module_setting_details')->where('setting_id', $query->id)
                ->where('user_id', $auth_id)
                ->where('is_active', '1')
                ->where('setting_value', 'Y')->first();
            if ($check) {
                return true;
            } else {
                return false;
            }
        } else {
            return false;
        }
        return false;
    }
}
