<?php

namespace App\Library;

use Illuminate\Http\Request;
use Illuminate\Support\Facades\Hash;
use DB;
use Illuminate\Support\Facades\Storage;

class CommonController
{

    /* 
    Param :=> $userId : 10,$fields : Array
    Return :=> Collection of Single Record
    */

    static function userInfo($userId, $fields = null)
    {
        $query = DB::table('tai_employee_master')->where('id', $userId);
        if ($fields) {
            $query->select($fields);
        }
        $user = $query->first();
        if ($user) {
            return $user;
        } else {
            return null;
        }
    }

    static function addNotification($fromUser, $toUser, $message, $url = NULL)
    {
        // dd($fromUser);
        $tenant_id = static::userInfo($fromUser, ['tenant_id']);
        // dd($tenant_id);/

        $insertedId = DB::table('tai_notification')
            ->insertGetId([
                'tenant_id' => $tenant_id->tenant_id,
                'from_user_id' => $fromUser,
                'to_user_id' => $toUser,
                'message' => $message,
                'url' => $url,
                'created_at' => now()
            ]);
        return $insertedId;
    }


    static function getCurrentFinancialYear()
    {
        if (date('m') > 6) {
            $year = date('Y') . "-" . (date('y') + 1);
        } else {
            $year = (date('Y') - 1) . "-" . date('y');
        }
        return $year;
    }

    static function attachmentPath($type)
    {
        $currentFinancialYear = static::getCurrentFinancialYear();

        switch ($type) {
            case "TICKET":
                return 'Attachment/' . $type . '/' . $currentFinancialYear . '/' . date('Y-m') . '/' . date('m') . '/';
                break;
            case "TASK":
                return 'Attachment/' . $type . '/' . $currentFinancialYear . '/' . date('Y-m') . '/' . date('m') . '/';
                break;
            default:
                return;
        }
    }


    static function uploadAttachment($ref_id, $files, $type, $createdBy, $showToCustomer = NULL, $showToProjectOwner = NULL)
    {
        if (is_array($files)) {
            foreach ($files as $index => $file) {
                $path = static::attachmentPath($type);
                $fileName = $file->getClientOriginalName();

                // dd(storage_path($path),$fileName);  
                $isUploaded = $file->storeAs($path, $fileName);
                // $isUploaded = $file->storeAs(storage_path($path),$fileName);
                // dd($file->storeAs(storage_path($path),$fileName));

                $check = DB::table('tai_attachments')->where(['ref_id' => $ref_id, 'name' => $fileName])->first();
                if ($check) {
                    $insert = DB::table('tai_attachments')->where('id', $check->id)
                        ->update([
                            'path' => "storage/app/" . $path . $fileName,
                            'show_to_customer' => $showToCustomer ? $showToCustomer[$index] : 0,
                            'show_to_project_owner' => $showToProjectOwner ? $showToProjectOwner[$index] : 0,
                            'created_by' => $createdBy,
                            'created_at' => now()
                        ]);
                } else {
                    $insert = DB::table('tai_attachments')->insert([
                        'ref_id' => $ref_id,
                        'name' => $fileName,
                        'path' => "storage/app/" . $path . $fileName,
                        'type' => $type,
                        'show_to_project_owner' => $showToProjectOwner ? $showToProjectOwner[$index] : 0,
                        'created_by' => $createdBy,
                        'created_at' => now()
                    ]);
                }
            }
        } else {
            $path = static::attachmentPath($type);
            $fileName = $files->getClientOriginalName();
            $isUploaded = $files->storeAs($path, $fileName);

            $insert = DB::table('tai_attachments')->insert([
                'ref_id' => $ref_id,
                'name' => $fileName,
                'path' => "storage/app/" . $path . $fileName,
                'type' => $type,
                'show_to_project_owner' => $showToProjectOwner ? $showToProjectOwner[$index] : 0,
                'created_by' => $createdBy,
                'created_at' => now()
            ]);
        }
        return;
    }

    static function getAttachment($ref_id, $type)
    {
        $result = DB::table('tai_attachments')->where('ref_id', $ref_id)->where('type', $type)->get()->toArray();
        if (count($result)) {
            return $result;
        } else {
            return;
        }
    }

    static function min_to_hour($min, $format)
    {

        $returnFormat = [];
        if ($format) {
            if ($min < 1) {
                $hours = 0;
                $minutes = 0;
            } else {
                $hours = floor($min / 60);
                $minutes = ($min % 60);
            }
            if (strpos($format, '/') !== false) {
                $returnFormat = explode("/", $format);
                return "$hours$returnFormat[0]/$minutes$returnFormat[1]";
            } else if (strpos($format, ':') !== false) {
                $returnFormat = explode(":", $format);
                return "$hours$returnFormat[0]:$minutes$returnFormat[1]";
            } else {
                return 0;
            }
        } else {
            return 0;
        }
    }

    static function generateHistory($operation, $tableName, $id)
    {
        $data = (array)DB::table($tableName)->where('id', $id)->first();


        $data['main_id'] = $id;
        unset($data['id']);

        $data['operation'] = $operation;

        DB::table($tableName . "_history")->insert($data);
        return true;
    }

    static function hoursandmins($time, $format = '%02d:%02d')
    {
        if ($time < 1) {
            return;
        }
        $hours = floor($time / 60);
        $minutes = ($time % 60);
        return sprintf($format, $hours, $minutes);
    }

    // punam shinde // 1-12-2022 
    // Authoruity 
    public function checkAuthorityAccess($auth_id = null, $module_name, $auth_name = null)
    {
        $query = DB::table('tai_module_setting')->where('setting_name', $auth_name)
            ->where('field_value', 'Y')
            ->where('setting_type', 'U')
            ->where('module_name', $module_name)
            ->where('is_active', '1')->first();
        if ($query) {
            $check = DB::table('tai_module_setting_details')->where('setting_id', $query->id)
                ->where('user_id', $auth_id)
                ->where('is_active', '1')
                ->where('setting_value', 'Y')->first();
            if ($check) {
                return true;
            } else {
                return false;
            }
        } else {
            return false;
        }
        return false;
    }
}
